Method: ActiveRecord::Validations#save

Defined in:
lib/active_record/validations.rb

#save(**options) ⇒ Object

The validation process on save can be skipped by passing validate: false. The validation context can be changed by passing context: context. The regular ActiveRecord::Base#save method is replaced with this when the validations module is mixed in, which it is by default.



47
48
49
# File 'lib/active_record/validations.rb', line 47

def save(**options)
  perform_validations(options) ? super : false
end