Class: ActiveRecord::Associations::CollectionAssociation

Inherits:
Association
  • Object
show all
Defined in:
lib/active_record/associations/collection_association.rb

Overview

Active Record Association Collection

CollectionAssociation is an abstract class that provides common stuff to ease the implementation of association proxies that represent collections. See the class hierarchy in Association.

CollectionAssociation:
HasManyAssociation => has_many
  HasManyThroughAssociation + ThroughAssociation => has_many :through

The CollectionAssociation class provides common methods to the collections defined by has_and_belongs_to_many, has_many or has_many with the :through association option.

You need to be careful with assumptions regarding the target: The proxy does not fetch records from the database until it needs them, but new ones created with build are added to the target. So, the target may be non-empty and still lack children waiting to be read from the database. If you look directly to the database you cannot assume that's the entire collection because new records may have been added to the target, etc.

If you need to work on all current children, new and existing records, load_target and the loaded flag are your friends.

Direct Known Subclasses

HasManyAssociation

Instance Attribute Summary collapse

Attributes inherited from Association

#disable_joins, #owner, #reflection

Instance Method Summary collapse

Methods inherited from Association

#async_load_target, #create, #create!, #extensions, #initialize, #initialize_attributes, #inversed_from, #inversed_from_queries, #klass, #loaded!, #loaded?, #marshal_dump, #marshal_load, #reload, #remove_inverse_instance, #reset_negative_cache, #reset_scope, #set_inverse_instance, #set_inverse_instance_from_queries, #set_strict_loading, #stale_target?, #target, #violates_strict_loading?

Constructor Details

This class inherits a constructor from ActiveRecord::Associations::Association

Instance Attribute Details

#nested_attributes_target ⇒ Object

:nodoc:



31
32
33
# File 'lib/active_record/associations/collection_association.rb', line 31

def nested_attributes_target
  @nested_attributes_target
end

Instance Method Details

#add_to_target(record, skip_callbacks: false, replace: false, &block) ⇒ Object



285
286
287
# File 'lib/active_record/associations/collection_association.rb', line 285

def add_to_target(record, skip_callbacks: false, replace: false, &block)
  replace_on_target(record, skip_callbacks, replace: replace || association_scope.distinct_value, &block)
end

#build(attributes = nil, &block) ⇒ Object



121
122
123
124
125
126
127
# File 'lib/active_record/associations/collection_association.rb', line 121

def build(attributes = nil, &block)
  if attributes.is_a?(Array)
    attributes.collect { |attr| build(attr, &block) }
  else
    add_to_target(build_record(attributes, &block), replace: true)
  end
end

#collection? ⇒ Boolean

Returns:

  • (Boolean)


320
321
322
# File 'lib/active_record/associations/collection_association.rb', line 320

def collection?
  true
end

#concat(*records) ⇒ Object

Add records to this association. Since << flattens its argument list and inserts each record, push and concat behave identically.



131
132
133
134
135
136
137
138
139
# File 'lib/active_record/associations/collection_association.rb', line 131

def concat(*records)
  records = records.flatten
  if owner.new_record?
    skip_strict_loading { load_target }
    concat_records(records)
  else
    transaction { concat_records(records) }
  end
end

#delete(*records) ⇒ Object

Removes records from this association calling before_remove and after_remove callbacks.

This method is abstract in the sense that delete_records has to be provided by descendants. Note this method does not imply the records are actually removed from the database, that depends precisely on delete_records. They are in any case removed from the collection.



190
191
192
# File 'lib/active_record/associations/collection_association.rb', line 190

def delete(*records)
  delete_or_destroy(records, options[:dependent])
end

#delete_all(dependent = nil) ⇒ Object

Removes all records from the association without calling callbacks on the associated records. It honors the :dependent option. However if the :dependent value is :destroy then in that case the :delete_all deletion strategy for the association is applied.

You can force a particular deletion strategy by passing a parameter.

Example:

@author.books.delete_all(:nullify) @author.books.delete_all(:delete_all)

See delete for more info.



154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
# File 'lib/active_record/associations/collection_association.rb', line 154

def delete_all(dependent = nil)
  if dependent && ![:nullify, :delete_all].include?(dependent)
    raise ArgumentError, "Valid values are :nullify or :delete_all"
  end

  dependent = if dependent
    dependent
  elsif options[:dependent] == :destroy
    :delete_all
  else
    options[:dependent]
  end

  delete_or_nullify_all_records(dependent).tap do
    reset
    loaded!
  end
end

#destroy(*records) ⇒ Object

Deletes the records and removes them from this association calling before_remove, after_remove, before_destroy and after_destroy callbacks.

Note that this method removes records from the database ignoring the :dependent option.



199
200
201
# File 'lib/active_record/associations/collection_association.rb', line 199

def destroy(*records)
  delete_or_destroy(records, :destroy)
end

#destroy_all ⇒ Object

Destroy all the records from this association.

See destroy for more info.



176
177
178
179
180
181
# File 'lib/active_record/associations/collection_association.rb', line 176

def destroy_all
  destroy(load_target).tap do
    reset
    loaded!
  end
end

#empty? ⇒ Boolean

Returns true if the collection is empty.

If the collection has been loaded it is equivalent to collection.size.zero?. If the collection has not been loaded, it is equivalent to !collection.exists?. If the collection has not already been loaded and you are going to fetch the records anyway it is better to check collection.length.zero?.

Returns:

  • (Boolean)


236
237
238
239
240
241
242
# File 'lib/active_record/associations/collection_association.rb', line 236

def empty?
  if loaded? || @association_ids || reflection.has_active_cached_counter?
    size.zero?
  else
    target.empty? && !scope.exists?
  end
end

#find(*args) ⇒ Object



98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
# File 'lib/active_record/associations/collection_association.rb', line 98

def find(*args)
  if options[:inverse_of] && loaded?
    args_flatten = args.flatten
    model = scope.model

    if args_flatten.blank?
      error_message = "Couldn't find #{model.name} without an ID"
      raise RecordNotFound.new(error_message, model.name, model.primary_key, args)
    end

    result = find_by_scan(*args)

    result_size = Array(result).size
    if !result || result_size != args_flatten.size
      scope.raise_record_not_found_exception!(args_flatten, result_size, args_flatten.size)
    else
      result
    end
  else
    scope.find(*args)
  end
end

#find_from_target? ⇒ Boolean

Returns:

  • (Boolean)


312
313
314
315
316
317
318
# File 'lib/active_record/associations/collection_association.rb', line 312

def find_from_target?
  loaded? ||
    (owner.strict_loading? && owner.strict_loading_all?) ||
    reflection.strict_loading? ||
    owner.new_record? ||
    target.any? { |record| record.new_record? || record.changed? }
end

#ids_reader ⇒ Object

Implements the ids reader method, e.g. foo.item_ids for Foo.has_many :items



51
52
53
54
55
56
57
58
59
# File 'lib/active_record/associations/collection_association.rb', line 51

def ids_reader
  if loaded?
    target.pluck(*reflection.association_primary_key)
  elsif !target.empty?
    load_target.pluck(*reflection.association_primary_key)
  else
    @association_ids ||= scope.pluck(*reflection.association_primary_key)
  end
end

#ids_writer(ids) ⇒ Object

Implements the ids writer method, e.g. foo.item_ids= for Foo.has_many :items



62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
# File 'lib/active_record/associations/collection_association.rb', line 62

def ids_writer(ids)
  primary_key = reflection.association_primary_key
  ids = Array(ids).compact_blank

  records = if klass.composite_primary_key?
    pk_types = primary_key.map { |column| klass.type_for_attribute(column) }
    ids.map! { |id| pk_types.zip(id).map! { |type, value| type.cast(value) } }

    klass.where(primary_key => ids).index_by do |record|
      primary_key.map { |primary_key| record._read_attribute(primary_key) }
    end
  else
    pk_type = klass.type_for_attribute(primary_key)
    ids.map! { |id| pk_type.cast(id) }

    klass.where(primary_key => ids).index_by do |record|
      record._read_attribute(primary_key)
    end
  end.values_at(*ids).compact

  if records.size != ids.size
    found_ids = records.map { |record| record._read_attribute(primary_key) }
    not_found_ids = ids - found_ids
    klass.all.raise_record_not_found_exception!(ids, records.size, ids.size, primary_key, not_found_ids)
  else
    replace(records)
  end
end

#include?(record) ⇒ Boolean

Returns:

  • (Boolean)


262
263
264
265
266
267
268
269
270
271
272
273
274
# File 'lib/active_record/associations/collection_association.rb', line 262

def include?(record)
  klass = reflection.klass
  return false unless record.is_a?(klass)

  if loaded?
    target.include?(record)
  elsif record.new_record?
    include_in_memory?(record)
  else
    record_id = klass.composite_primary_key? ? klass.primary_key.zip(record.id).to_h : record.id
    scope.exists?(record_id)
  end
end

#load_target ⇒ Object



276
277
278
279
280
281
282
283
# File 'lib/active_record/associations/collection_association.rb', line 276

def load_target
  if find_target?
    @target = merge_target_lists(find_target, target)
  end

  loaded!
  target
end

#null_scope? ⇒ Boolean

Returns:

  • (Boolean)


308
309
310
# File 'lib/active_record/associations/collection_association.rb', line 308

def null_scope?
  owner.new_record? && !foreign_key_present?
end

#reader ⇒ Object

Implements the reader method, e.g. foo.items for Foo.has_many :items



34
35
36
37
38
39
40
41
42
43
# File 'lib/active_record/associations/collection_association.rb', line 34

def reader
  ensure_klass_exists!

  if stale_target?
    reload
  end

  @proxy ||= CollectionProxy.create(klass, self)
  @proxy.reset_scope
end

#replace(other_array) ⇒ Object

Replace this collection with other_array. This will perform a diff and delete/add only records that have changed.



246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
# File 'lib/active_record/associations/collection_association.rb', line 246

def replace(other_array)
  other_array.each { |val| raise_on_type_mismatch!(val) }
  original_target = skip_strict_loading { load_target }.dup

  if owner.new_record?
    replace_records(other_array, original_target)
  else
    replace_common_records_in_memory(other_array, original_target)
    if other_array != original_target
      transaction { replace_records(other_array, original_target) }
    else
      other_array
    end
  end
end

#reset ⇒ Object



91
92
93
94
95
96
# File 'lib/active_record/associations/collection_association.rb', line 91

def reset
  super
  @target = []
  @replaced_or_added_targets = Set.new.compare_by_identity
  @association_ids = nil
end

#scope ⇒ Object



302
303
304
305
306
# File 'lib/active_record/associations/collection_association.rb', line 302

def scope
  scope = super
  scope.none! if null_scope?
  scope
end

#size ⇒ Object

Returns the size of the collection by executing a SELECT COUNT(*) query if the collection hasn't been loaded, and calling collection.size if it has.

If the collection has been already loaded size and length are equivalent. If not and you are going to need the records anyway length will take one less query. Otherwise size is more efficient.

This method is abstract in the sense that it relies on count_records, which is a method descendants have to provide.



213
214
215
216
217
218
219
220
221
222
223
224
225
226
# File 'lib/active_record/associations/collection_association.rb', line 213

def size
  if !find_target? || loaded?
    target.size
  elsif @association_ids
    @association_ids.size
  elsif !association_scope.group_values.empty?
    load_target.size
  elsif !association_scope.distinct_value && !target.empty?
    unsaved_records = target.select(&:new_record?)
    unsaved_records.size + count_records
  else
    count_records
  end
end

#target=(record) ⇒ Object



289
290
291
292
293
294
295
296
297
298
299
300
# File 'lib/active_record/associations/collection_association.rb', line 289

def target=(record)
  return super unless reflection.klass.has_many_inversing

  case record
  when nil
    # It's not possible to remove the record from the inverse association.
  when Array
    super
  else
    replace_on_target(record, true, replace: true, inversing: true)
  end
end

#writer(records) ⇒ Object

Implements the writer method, e.g. foo.items= for Foo.has_many :items



46
47
48
# File 'lib/active_record/associations/collection_association.rb', line 46

def writer(records)
  replace(records)
end