Method: ActiveRecord::Callbacks#after_create

Defined in:
lib/active_record/callbacks.rb

#after_createObject

Is called after Base.save on new objects that haven't been saved yet (no record exists). Note that this callback is still wrapped in the transaction around save. For example, if you invoke an external indexer at this point it won't see the changes in the database.



234
# File 'lib/active_record/callbacks.rb', line 234

def after_create() end