Class: ActiveRecord::ConnectionAdapters::AbstractAdapter

Inherits:
Object
  • Object
show all
Includes:
DatabaseLimits, DatabaseStatements, QueryCache, Quoting, Savepoints, SchemaStatements, ActiveSupport::Callbacks
Defined in:
lib/active_record/connection_adapters/abstract_adapter.rb

Overview

Active Record Abstract Adapter

Active Record supports multiple database systems. AbstractAdapter and related classes form the abstraction layer which makes this possible. An AbstractAdapter represents a connection to a database, and provides an abstract interface for database-specific functionality such as establishing a connection, escaping values, building the right SQL fragments for :offset and :limit options, etc.

All the concrete database adapters follow the interface laid down in this class. ActiveRecord::Base.lease_connection returns an AbstractAdapter object, which you can use.

Most of the methods in the adapter are useful during migrations. Most notably, the instance methods provided by SchemaStatements are very useful.

Defined Under Namespace

Classes: Version

Constant Summary collapse

ADAPTER_NAME =
"Abstract"
SIMPLE_INT =
/\A\d+\z/
COMMENT_REGEX =
%r{(?:--.*\n)|/\*(?:[^*]|\*[^/])*\*/}
MAX_JITTER =

:nodoc:

0.0..1.0
TYPE_MAP =
Type::TypeMap.new.tap { |m| initialize_type_map(m) }
EXTENDED_TYPE_MAPS =
Concurrent::Map.new

Constants included from QueryCache

QueryCache::DEFAULT_SIZE

Instance Attribute Summary collapse

Attributes included from QueryCache

#query_cache

Attributes included from DatabaseStatements

#transaction_manager

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Savepoints

#create_savepoint, #current_savepoint_name, #exec_rollback_to_savepoint, #release_savepoint

Methods included from QueryCache

#cache, #clear_query_cache, dirties_query_cache, #disable_query_cache!, #enable_query_cache!, included, #query_cache_enabled, #select_all, #uncached

Methods included from DatabaseLimits

#index_name_length, #max_identifier_length, #table_alias_length, #table_name_length

Methods included from Quoting

#cast_bound_value, #lookup_cast_type, #quote, #quote_column_name, #quote_default_expression, #quote_string, #quote_table_name, #quote_table_name_for_assignment, #quoted_binary, #quoted_date, #quoted_false, #quoted_time, #quoted_true, #sanitize_as_sql_comment, #type_cast, #unquoted_false, #unquoted_true

Methods included from DatabaseStatements

#add_transaction_record, #begin_db_transaction, #begin_deferred_transaction, #begin_isolated_db_transaction, #cacheable_query, #commit_db_transaction, #default_insert_value, #default_sequence_name, #delete, #empty_insert_statement_value, #exec_delete, #exec_insert, #exec_insert_all, #exec_query, #exec_restart_db_transaction, #exec_rollback_db_transaction, #exec_update, #execute, #explain, #high_precision_current_timestamp, #insert, #insert_fixture, #insert_fixtures_set, #internal_exec_query, #mark_transaction_written, #query, #query_value, #query_values, #raw_exec_query, #reset_isolation_level, #reset_sequence!, #reset_transaction, #restart_db_transaction, #rollback_db_transaction, #rollback_to_savepoint, #select_all, #select_one, #select_rows, #select_value, #select_values, #to_sql, #transaction, #transaction_isolation_levels, #transaction_open?, #truncate, #truncate_tables, #update, #with_yaml_fallback, #write_query?

Methods included from SchemaStatements

#add_check_constraint, #add_column, #add_columns, #add_foreign_key, #add_index, #add_index_options, #add_reference, #add_timestamps, #assume_migrated_upto_version, #build_add_column_definition, #build_change_column_default_definition, #build_create_index_definition, #build_create_join_table_definition, #build_create_table_definition, #bulk_change_table, #change_column, #change_column_comment, #change_column_default, #change_column_null, #change_table, #change_table_comment, #check_constraint_exists?, #check_constraint_options, #check_constraints, #column_exists?, #columns, #columns_for_distinct, #create_join_table, #create_schema_dumper, #create_table, #data_source_exists?, #data_sources, #disable_index, #distinct_relation_for_primary_key, #drop_join_table, #drop_table, #dump_schema_versions, #enable_index, #foreign_key_column_for, #foreign_key_exists?, #foreign_key_options, #foreign_keys, #index_algorithm, #index_exists?, #index_name, #index_name_exists?, #indexes, #internal_string_options_for_primary_key, #max_index_name_size, #options_include_default?, #primary_key, #quoted_columns_for_index, #remove_check_constraint, #remove_column, #remove_columns, #remove_constraint, #remove_foreign_key, #remove_index, #remove_reference, #remove_timestamps, #rename_column, #rename_index, #rename_table, #schema_creation, #table_alias_for, #table_comment, #table_exists?, #table_options, #tables, #type_to_sql, #update_table_definition, #use_foreign_keys?, #valid_column_definition_options, #valid_primary_key_options, #valid_table_definition_options, #view_exists?, #views

Constructor Details

#initialize(config_or_deprecated_connection, deprecated_logger = nil, deprecated_connection_options = nil, deprecated_config = nil) ⇒ AbstractAdapter

:nodoc:



132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 132

def initialize(config_or_deprecated_connection, deprecated_logger = nil, deprecated_connection_options = nil, deprecated_config = nil) # :nodoc:
  super()

  @raw_connection = nil
  @unconfigured_connection = nil
  @connected_since = nil

  if config_or_deprecated_connection.is_a?(Hash)
    @config = config_or_deprecated_connection.symbolize_keys
    @logger = ActiveRecord::Base.logger

    if deprecated_logger || deprecated_connection_options || deprecated_config
      raise ArgumentError, "when initializing an Active Record adapter with a config hash, that should be the only argument"
    end
  else
    # Soft-deprecated for now; we'll probably warn in future.

    @unconfigured_connection = config_or_deprecated_connection
    @connected_since = Process.clock_gettime(Process::CLOCK_MONOTONIC)
    @logger = deprecated_logger || ActiveRecord::Base.logger
    if deprecated_config
      @config = (deprecated_config || {}).symbolize_keys
      @connection_parameters = deprecated_connection_options
    else
      @config = (deprecated_connection_options || {}).symbolize_keys
      @connection_parameters = nil
    end
  end

  @owner = nil
  @pinned = false
  @pool = ActiveRecord::ConnectionAdapters::NullPool.new
  @idle_since = Process.clock_gettime(Process::CLOCK_MONOTONIC)
  @allow_preconnect = false
  @visitor = arel_visitor
  @statements = build_statement_pool
  self.lock_thread = nil

  @prepared_statements = !ActiveRecord.disable_prepared_statements && self.class.type_cast_config_to_boolean(
    @config.fetch(:prepared_statements) { default_prepared_statements }
  )

  @advisory_locks_enabled = self.class.type_cast_config_to_boolean(
    @config.fetch(:advisory_locks, true)
  )

  @default_timezone = self.class.validate_default_timezone(@config[:default_timezone])

  @raw_connection_dirty = false
  @last_activity = nil
  @verified = false

  @pool_jitter = rand * max_jitter
end

Instance Attribute Details

#allow_preconnect ⇒ Object

:nodoc:



46
47
48
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 46

def allow_preconnect
  @allow_preconnect
end

#lock ⇒ Object (readonly)

Returns the value of attribute lock.



45
46
47
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 45

def lock
  @lock
end

#logger ⇒ Object (readonly)

Returns the value of attribute logger.



45
46
47
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 45

def logger
  @logger
end

#owner ⇒ Object (readonly) Also known as: in_use?

Returns the value of attribute owner.



45
46
47
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 45

def owner
  @owner
end

#pinned ⇒ Object

:nodoc:



47
48
49
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 47

def pinned
  @pinned
end

#pool ⇒ Object

Returns the value of attribute pool.



44
45
46
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 44

def pool
  @pool
end

#visitor ⇒ Object (readonly)

Returns the value of attribute visitor.



45
46
47
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 45

def visitor
  @visitor
end

Class Method Details

.build_read_query_regexp(*parts) ⇒ Object

:nodoc:



93
94
95
96
97
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 93

def self.build_read_query_regexp(*parts) # :nodoc:
  parts += DEFAULT_READ_QUERY
  parts = parts.map { |part| /#{part}/i }
  /\A(?:[(\s]|#{COMMENT_REGEX})*#{Regexp.union(*parts)}/
end

.database_exists?(config) ⇒ Boolean

Does the database for this adapter exist?

Returns:

  • (Boolean)


403
404
405
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 403

def self.database_exists?(config)
  new(config).database_exists?
end

.dbconsole(config, options = {}) ⇒ Object

Opens a database console session.

Raises:

  • (NotImplementedError)


128
129
130
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 128

def self.dbconsole(config, options = {})
  raise NotImplementedError.new("#{self} should define `dbconsole` that accepts a db config and options to implement connecting to the db console")
end

.extended_type_map(default_timezone:) ⇒ Object

:nodoc:



949
950
951
952
953
954
955
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 949

def extended_type_map(default_timezone:) # :nodoc:
  Type::TypeMap.new(self::TYPE_MAP).tap do |m|
    register_class_with_precision m, %r(\A[^\(]*time)i, Type::Time, timezone: default_timezone
    register_class_with_precision m, %r(\A[^\(]*datetime)i, Type::DateTime, timezone: default_timezone
    m.alias_type %r(\A[^\(]*timestamp)i, "datetime"
  end
end

.find_cmd_and_exec(commands, *args) ⇒ Object

:doc:



99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 99

def self.find_cmd_and_exec(commands, *args) # :doc:
  commands = Array(commands)

  dirs_on_path = ENV["PATH"].to_s.split(File::PATH_SEPARATOR)
  unless (ext = RbConfig::CONFIG["EXEEXT"]).empty?
    commands = commands.map { |cmd| "#{cmd}#{ext}" }
  end

  full_path_command = nil
  found = commands.detect do |cmd|
    dirs_on_path.detect do |path|
      full_path_command = File.join(path, cmd)
      begin
        stat = File.stat(full_path_command)
      rescue SystemCallError
      else
        stat.file? && stat.executable?
      end
    end
  end

  if found
    exec full_path_command, *args
  else
    abort("Couldn't find database client: #{commands.join(', ')}. Check your $PATH and try again.")
  end
end

.register_class_with_precision(mapping, key, klass, **kwargs) ⇒ Object

:nodoc:



942
943
944
945
946
947
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 942

def register_class_with_precision(mapping, key, klass, **kwargs) # :nodoc:
  mapping.register_type(key) do |*args|
    precision = extract_precision(args.last)
    klass.new(precision: precision, **kwargs).freeze
  end
end

.type_cast_config_to_boolean(config) ⇒ Object



72
73
74
75
76
77
78
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 72

def self.type_cast_config_to_boolean(config)
  if config == "false"
    false
  else
    config
  end
end

.type_cast_config_to_integer(config) ⇒ Object



62
63
64
65
66
67
68
69
70
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 62

def self.type_cast_config_to_integer(config)
  if config.is_a?(Integer)
    config
  elsif SIMPLE_INT.match?(config)
    config.to_i
  else
    config
  end
end

.valid_type?(type) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


957
958
959
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 957

def valid_type?(type) # :nodoc:
  !native_database_types[type].nil?
end

.validate_default_timezone(config) ⇒ Object



80
81
82
83
84
85
86
87
88
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 80

def self.validate_default_timezone(config)
  case config
  when nil
  when "utc", "local"
    config.to_sym
  else
    raise ArgumentError, "default_timezone must be either 'utc' or 'local'"
  end
end

Instance Method Details

#active? ⇒ Boolean

Checks whether the connection to the database is still active. This includes checking whether the database is actually capable of responding, i.e. whether the connection isn't stale.

Returns:

  • (Boolean)


710
711
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 710

def active?
end

#adapter_name ⇒ Object

Returns the human-readable name of the adapter. Use mixed case - one can always use downcase if needed.



398
399
400
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 398

def adapter_name
  self.class::ADAPTER_NAME
end

#add_enum_value ⇒ Object

This is meant to be implemented by the adapters that support custom enum types



637
638
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 637

def add_enum_value(...) # :nodoc:
end

#advisory_locks_enabled? ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


657
658
659
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 657

def advisory_locks_enabled? # :nodoc:
  supports_advisory_locks? && @advisory_locks_enabled
end

#async_enabled? ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


611
612
613
614
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 611

def async_enabled? # :nodoc:
  supports_concurrent_connections? &&
    !ActiveRecord.async_query_executor.nil? && !pool.async_executor.nil?
end

#build_insert_sql(insert) ⇒ Object

Called by ActiveRecord::InsertAll, Passed an instance of ActiveRecord::InsertAll::Builder, This method implements standard bulk inserts for all databases, but should be overridden by adapters to implement common features with non-standard syntax like handling duplicates or returning values.



915
916
917
918
919
920
921
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 915

def build_insert_sql(insert) # :nodoc:
  if insert.skip_duplicates? || insert.update_duplicates?
    raise NotImplementedError, "#{self.class} should define `build_insert_sql` to implement adapter-specific logic for handling duplicates during INSERT"
  end

  "INSERT #{insert.into} #{insert.values_list}"
end

#case_insensitive_comparison(attribute, value) ⇒ Object

:nodoc:



886
887
888
889
890
891
892
893
894
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 886

def case_insensitive_comparison(attribute, value) # :nodoc:
  column = column_for_attribute(attribute)

  if can_perform_case_insensitive_comparison_for?(column)
    attribute.lower.eq(attribute.relation.lower(value))
  else
    attribute.eq(value)
  end
end

#case_sensitive_comparison(attribute, value) ⇒ Object

:nodoc:



882
883
884
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 882

def case_sensitive_comparison(attribute, value) # :nodoc:
  attribute.eq(value)
end

#check_all_foreign_keys_valid! ⇒ Object

Override to check all foreign key constraints in a database. The adapter should raise a ActiveRecord::StatementInvalid if foreign key constraints are not met.



695
696
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 695

def check_all_foreign_keys_valid!
end

#check_version ⇒ Object

:nodoc:



930
931
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 930

def check_version # :nodoc:
end

#clean! ⇒ Object

:nodoc:



848
849
850
851
852
853
854
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 848

def clean! # :nodoc:
  _run_checkout_callbacks do
    @raw_connection_dirty = false
    @verified = nil
  end
  self
end

#clear_cache!(new_connection: false) ⇒ Object

Clear any caching the database adapter may be doing.



800
801
802
803
804
805
806
807
808
809
810
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 800

def clear_cache!(new_connection: false)
  if @statements
    @lock.synchronize do
      if new_connection
        @statements.reset
      else
        @statements.clear
      end
    end
  end
end

#close ⇒ Object

Check the connection back in to the connection pool



902
903
904
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 902

def close
  pool.checkin self
end

#connect! ⇒ Object



843
844
845
846
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 843

def connect!
  verify!
  self
end

#connected? ⇒ Boolean

Checks whether the connection to the database was established. This doesn't include checking whether the database is actually capable of responding, i.e. whether the connection is stale.

Returns:

  • (Boolean)


703
704
705
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 703

def connected?
  !@raw_connection.nil?
end

#connection_age ⇒ Object

Seconds since this connection was established. nil if not connected; infinity if the connection has been explicitly retired.



377
378
379
380
381
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 377

def connection_age # :nodoc:
  if @raw_connection && @connected_since
    Process.clock_gettime(Process::CLOCK_MONOTONIC) - @connected_since
  end
end

#connection_descriptor ⇒ Object

:nodoc:



304
305
306
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 304

def connection_descriptor # :nodoc:
  @pool.connection_descriptor
end

#connection_retries ⇒ Object



221
222
223
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 221

def connection_retries
  (@config[:connection_retries] || 1).to_i
end

#create_enum ⇒ Object

This is meant to be implemented by the adapters that support custom enum types



625
626
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 625

def create_enum(...) # :nodoc:
end

#create_or_find_by_lock ⇒ Object

Lock used to read an existing record after a duplicate INSERT in a transaction.



653
654
655
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 653

def create_or_find_by_lock # :nodoc:
  true
end

#create_virtual_table ⇒ Object

This is meant to be implemented by the adapters that support virtual tables



645
646
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 645

def create_virtual_table(*) # :nodoc:
end

#database_exists? ⇒ Boolean

Returns:

  • (Boolean)


407
408
409
410
411
412
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 407

def database_exists?
  connect!
  true
rescue ActiveRecord::NoDatabaseError
  false
end

#database_version ⇒ Object

:nodoc:



926
927
928
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 926

def database_version # :nodoc:
  pool.server_version(self)
end

#default_index_type?(index) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


906
907
908
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 906

def default_index_type?(index) # :nodoc:
  index.using.nil?
end

#default_timezone ⇒ Object



237
238
239
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 237

def default_timezone
  @default_timezone || ActiveRecord.default_timezone
end

#default_uniqueness_comparison(attribute, value) ⇒ Object

:nodoc:



878
879
880
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 878

def default_uniqueness_comparison(attribute, value) # :nodoc:
  attribute.eq(value)
end

#disable_extension(name) ⇒ Object

This is meant to be implemented by the adapters that support extensions



617
618
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 617

def disable_extension(name, **)
end

#disable_referential_integrity ⇒ Object

Override to turn off referential integrity while executing &block.



688
689
690
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 688

def disable_referential_integrity
  yield
end

#discard! ⇒ Object

Immediately forget this connection ever existed. Unlike disconnect!, this will not communicate with the server.

After calling this method, the behavior of all other methods becomes undefined. This is called internally just before a forked process gets rid of a connection that belonged to its parent.



773
774
775
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 773

def discard!
  # This should be overridden by concrete adapters.
end

#disconnect! ⇒ Object

Disconnects from the database if already connected. Otherwise, this method does nothing.



756
757
758
759
760
761
762
763
764
765
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 756

def disconnect!
  @lock.synchronize do
    clear_cache!(new_connection: true)
    reset_transaction
    @raw_connection_dirty = false
    @connected_since = nil
    @last_activity = nil
    @verified = false
  end
end

#drop_enum ⇒ Object

This is meant to be implemented by the adapters that support custom enum types



629
630
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 629

def drop_enum(...) # :nodoc:
end

#drop_virtual_table ⇒ Object

This is meant to be implemented by the adapters that support virtual tables



649
650
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 649

def drop_virtual_table(*) # :nodoc:
end

#enable_extension(name) ⇒ Object

This is meant to be implemented by the adapters that support extensions



621
622
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 621

def enable_extension(name, **)
end

#ensure_writes_are_allowed(sql) ⇒ Object

:nodoc:



206
207
208
209
210
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 206

def ensure_writes_are_allowed(sql) # :nodoc:
  if preventing_writes?
    raise ActiveRecord::ReadOnlyError, "Write query attempted while in readonly mode: #{sql}"
  end
end

#expire(update_idle = true) ⇒ Object

this method must only be called while holding connection pool's mutex



329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 329

def expire(update_idle = true) # :nodoc:
  if in_use?
    if @owner != ActiveSupport::IsolatedExecutionState.context
      raise ActiveRecordError, "Cannot expire connection, " \
        "it is owned by a different thread: #{@owner}. " \
        "Current thread: #{ActiveSupport::IsolatedExecutionState.context}."
    end

    _run_checkin_callbacks do
      @idle_since = Process.clock_gettime(Process::CLOCK_MONOTONIC) if update_idle
      @owner = nil
      enable_lazy_transactions!
      unset_query_cache!
    end
  else
    raise ActiveRecordError, "Cannot expire connection, it is not currently leased."
  end
end

#extensions ⇒ Object

A list of extensions, to be filled in by adapters that support them.



676
677
678
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 676

def extensions
  []
end

#force_retirement ⇒ Object

Mark the connection as needing to be retired, as if the age has exceeded the maximum allowed.



385
386
387
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 385

def force_retirement # :nodoc:
  @connected_since &&= -Float::INFINITY
end

#get_advisory_lock(lock_id) ⇒ Object

This is meant to be implemented by the adapters that support advisory locks

Return true if we got the lock, otherwise false



665
666
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 665

def get_advisory_lock(lock_id) # :nodoc:
end

#get_database_version ⇒ Object

:nodoc:



923
924
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 923

def get_database_version # :nodoc:
end

#index_algorithms ⇒ Object

A list of index algorithms, to be filled by adapters that support them.



681
682
683
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 681

def index_algorithms
  {}
end

#inspect ⇒ Object

:nodoc:



187
188
189
190
191
192
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 187

def inspect # :nodoc:
  name_field = " name=#{pool.db_config.name.inspect}" unless pool.db_config.name == "primary"
  shard_field = " shard=#{shard.inspect}" unless shard == :default

  "#<#{self.class.name}:#{'%#016x' % (object_id << 1)} env_name=#{pool.db_config.env_name.inspect}#{name_field} role=#{role.inspect}#{shard_field}>"
end

#lease ⇒ Object

this method must only be called while holding connection pool's mutex



289
290
291
292
293
294
295
296
297
298
299
300
301
302
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 289

def lease
  if in_use?
    msg = +"Cannot lease connection, "
    if @owner == ActiveSupport::IsolatedExecutionState.context
      msg << "it is already leased by the current thread."
    else
      msg << "it is already in use by a different thread: #{@owner}. " \
             "Current thread: #{ActiveSupport::IsolatedExecutionState.context}."
    end
    raise ActiveRecordError, msg
  end

  @owner = ActiveSupport::IsolatedExecutionState.context
end

#lock_thread=(lock_thread) ⇒ Object

:nodoc:



194
195
196
197
198
199
200
201
202
203
204
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 194

def lock_thread=(lock_thread) # :nodoc:
  @lock =
  case lock_thread
  when Thread
    ActiveSupport::Concurrency::ThreadMonitor.new
  when Fiber
    ::Monitor.new
  else
    ActiveSupport::Concurrency::NullLock
  end
end

#max_jitter ⇒ Object



213
214
215
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 213

def max_jitter
  (@config[:pool_jitter] || 0.2).to_f.clamp(MAX_JITTER)
end

#native_database_types ⇒ Object

:nodoc:



284
285
286
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 284

def native_database_types # :nodoc:
  self.class.native_database_types
end

#pool_jitter(duration) ⇒ Object



324
325
326
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 324

def pool_jitter(duration)
  duration * (1.0 - @pool_jitter)
end

#prefetch_primary_key?(table_name = nil) ⇒ Boolean

Should primary key values be selected from their corresponding sequence before the insert statement? If true, next_sequence_value is called before each insert to set the record's primary key.

Returns:

  • (Boolean)


447
448
449
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 447

def prefetch_primary_key?(table_name = nil)
  false
end

#prepared_statements? ⇒ Boolean Also known as: prepared_statements

Returns:

  • (Boolean)


252
253
254
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 252

def prepared_statements?
  @prepared_statements && !prepared_statements_disabled_cache.include?(object_id)
end

#prepared_statements_disabled_cache ⇒ Object

:nodoc:



257
258
259
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 257

def prepared_statements_disabled_cache # :nodoc:
  ActiveSupport::IsolatedExecutionState[:active_record_prepared_statements_disabled_cache] ||= Set.new
end

#preventing_writes? ⇒ Boolean

Determines whether writes are currently being prevented.

Returns true if the connection is a replica or returns the value of current_preventing_writes.

Returns:

  • (Boolean)


245
246
247
248
249
250
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 245

def preventing_writes?
  return true if replica?
  return false if connection_descriptor.nil?

  connection_descriptor.current_preventing_writes
end

#raw_connection ⇒ Object

Provides access to the underlying database driver for this adapter. For example, this method returns a Mysql2::Client object in case of Mysql2Adapter, and a PG::Connection object in case of PostgreSQLAdapter.

This is useful for when you need to call a proprietary method such as PostgreSQL's lo_* methods.

Active Record cannot track if the database is getting modified using this client. If that is the case, generally you'll want to invalidate the query cache using ActiveRecord::Base.clear_query_cache.



870
871
872
873
874
875
876
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 870

def raw_connection
  with_raw_connection do |conn|
    disable_lazy_transactions!
    @raw_connection_dirty = true
    conn
  end
end

#reconnect!(restore_transactions: false) ⇒ Object

Disconnects from the database if already connected, and establishes a new connection with the database. Implementors should define private #reconnect instead.



716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
751
752
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 716

def reconnect!(restore_transactions: false)
  retries_available = connection_retries
  deadline = retry_deadline && Process.clock_gettime(Process::CLOCK_MONOTONIC) + retry_deadline

  @lock.synchronize do
    attempt_configure_connection do
      @allow_preconnect = false

      reconnect

      enable_lazy_transactions!
      @raw_connection_dirty = false
      @last_activity = @connected_since = Process.clock_gettime(Process::CLOCK_MONOTONIC)
      @verified = true
      @allow_preconnect = true

      reset_transaction(restore: restore_transactions) do
        clear_cache!(new_connection: true)
        configure_connection
      end
    rescue => original_exception
      translated_exception = translate_exception_class(original_exception, nil, nil)
      retry_deadline_exceeded = deadline && deadline < Process.clock_gettime(Process::CLOCK_MONOTONIC)

      if !retry_deadline_exceeded && retries_available > 0
        retries_available -= 1

        if retryable_connection_error?(translated_exception)
          backoff(connection_retries - retries_available)
          retry
        end
      end

      raise translated_exception
    end
  end
end

#release_advisory_lock(lock_id) ⇒ Object

This is meant to be implemented by the adapters that support advisory locks.

Return true if we released the lock, otherwise false



672
673
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 672

def release_advisory_lock(lock_id) # :nodoc:
end

#rename_enum ⇒ Object

This is meant to be implemented by the adapters that support custom enum types



633
634
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 633

def rename_enum(...) # :nodoc:
end

#rename_enum_value ⇒ Object

This is meant to be implemented by the adapters that support custom enum types



641
642
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 641

def rename_enum_value(...) # :nodoc:
end

#replica? ⇒ Boolean

Returns:

  • (Boolean)


217
218
219
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 217

def replica?
  @config[:replica] || false
end

#requires_reloading? ⇒ Boolean

Returns true if its required to reload the connection between requests for development mode.

Returns:

  • (Boolean)


813
814
815
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 813

def requires_reloading?
  false
end

#reset! ⇒ Object

Reset the state of this connection, directing the DBMS to clear transactions and other connection-related server-side state. Usually a database-dependent operation.

If a database driver or protocol does not support such a feature, implementors may alias this to #reconnect!. Otherwise, implementors should call super immediately after resetting the connection (and while still holding @lock).



785
786
787
788
789
790
791
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 785

def reset!
  attempt_configure_connection do
    clear_cache!(new_connection: true)
    reset_transaction
    configure_connection
  end
end

#retry_deadline ⇒ Object



229
230
231
232
233
234
235
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 229

def retry_deadline
  if @config[:retry_deadline]
    @config[:retry_deadline].to_f
  else
    nil
  end
end

#return_value_after_insert?(column) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


607
608
609
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 607

def return_value_after_insert?(column) # :nodoc:
  column.auto_populated?
end

#role ⇒ Object

The role (e.g. :writing) for the current connection. In a non-multi role application, :writing is returned.



310
311
312
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 310

def role
  @pool.role
end

#savepoint_errors_invalidate_transactions? ⇒ Boolean

Do TransactionRollbackErrors on savepoints affect the parent transaction?

Returns:

  • (Boolean)


431
432
433
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 431

def savepoint_errors_invalidate_transactions?
  false
end

#schema_cache ⇒ Object



320
321
322
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 320

def schema_cache
  @pool.schema_cache || (@schema_cache ||= BoundSchemaReflection.for_lone_connection(@pool.schema_reflection, self))
end

#schema_version ⇒ Object

Returns the version identifier of the schema currently available in the database. This is generally equal to the number of the highest- numbered migration that has been executed, or 0 if no schema information is present / the database is empty.



937
938
939
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 937

def schema_version
  pool.migration_context.current_version
end

#seconds_idle ⇒ Object

Seconds since this connection was returned to the pool



362
363
364
365
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 362

def seconds_idle # :nodoc:
  return 0 if in_use?
  Process.clock_gettime(Process::CLOCK_MONOTONIC) - @idle_since
end

#seconds_since_last_activity ⇒ Object

Seconds since this connection last communicated with the server



368
369
370
371
372
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 368

def seconds_since_last_activity # :nodoc:
  if @raw_connection && @last_activity
    Process.clock_gettime(Process::CLOCK_MONOTONIC) - @last_activity
  end
end

#shard ⇒ Object

The shard (e.g. :default) for the current connection. In a non-sharded application, :default is returned.



316
317
318
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 316

def shard
  @pool.shard
end

#steal! ⇒ Object

this method must only be called while holding connection pool's mutex (and a desire for segfaults)



349
350
351
352
353
354
355
356
357
358
359
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 349

def steal! # :nodoc:
  if in_use?
    if @owner != ActiveSupport::IsolatedExecutionState.context
      pool.send :remove_connection_from_thread_cache, self, @owner

      @owner = ActiveSupport::IsolatedExecutionState.context
    end
  else
    raise ActiveRecordError, "Cannot steal connection, it is not currently leased."
  end
end

#supports_advisory_locks? ⇒ Boolean

Does this adapter support application-enforced advisory locking?

Returns:

  • (Boolean)


440
441
442
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 440

def supports_advisory_locks?
  false
end

#supports_bulk_alter? ⇒ Boolean

Returns:

  • (Boolean)


420
421
422
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 420

def supports_bulk_alter?
  false
end

#supports_check_constraints? ⇒ Boolean

Does this adapter support creating check constraints?

Returns:

  • (Boolean)


512
513
514
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 512

def supports_check_constraints?
  false
end

#supports_comments? ⇒ Boolean

Does this adapter support metadata comments on database objects (tables, columns, indexes)?

Returns:

  • (Boolean)


547
548
549
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 547

def supports_comments?
  false
end

#supports_comments_in_create? ⇒ Boolean

Can comments for tables, columns, and indexes be specified in create/alter table statements?

Returns:

  • (Boolean)


552
553
554
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 552

def supports_comments_in_create?
  false
end

#supports_common_table_expressions? ⇒ Boolean

Returns:

  • (Boolean)


571
572
573
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 571

def supports_common_table_expressions?
  false
end

#supports_concurrent_connections? ⇒ Boolean

Returns:

  • (Boolean)


595
596
597
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 595

def supports_concurrent_connections?
  true
end

#supports_datetime_with_precision? ⇒ Boolean

Does this adapter support datetime with precision?

Returns:

  • (Boolean)


537
538
539
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 537

def supports_datetime_with_precision?
  false
end

#supports_ddl_transactions? ⇒ Boolean

Does this adapter support DDL rollbacks in transactions? That is, would CREATE TABLE or ALTER TABLE get rolled back by a transaction?

Returns:

  • (Boolean)


416
417
418
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 416

def supports_ddl_transactions?
  false
end

#supports_deferrable_constraints? ⇒ Boolean

Does this adapter support creating deferrable constraints?

Returns:

  • (Boolean)


507
508
509
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 507

def supports_deferrable_constraints?
  false
end

#supports_disabling_indexes? ⇒ Boolean

Returns:

  • (Boolean)


603
604
605
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 603

def supports_disabling_indexes?
  false
end

#supports_exclusion_constraints? ⇒ Boolean

Does this adapter support creating exclusion constraints?

Returns:

  • (Boolean)


517
518
519
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 517

def supports_exclusion_constraints?
  false
end

#supports_explain? ⇒ Boolean

Does this adapter support explain?

Returns:

  • (Boolean)


476
477
478
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 476

def supports_explain?
  false
end

#supports_expression_index? ⇒ Boolean

Does this adapter support expression indices?

Returns:

  • (Boolean)


471
472
473
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 471

def supports_expression_index?
  false
end

#supports_extensions? ⇒ Boolean

Does this adapter support database extensions?

Returns:

  • (Boolean)


486
487
488
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 486

def supports_extensions?
  false
end

#supports_foreign_keys? ⇒ Boolean

Does this adapter support creating foreign key constraints?

Returns:

  • (Boolean)


497
498
499
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 497

def supports_foreign_keys?
  false
end

#supports_foreign_tables? ⇒ Boolean

Does this adapter support foreign/external tables?

Returns:

  • (Boolean)


562
563
564
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 562

def supports_foreign_tables?
  false
end

#supports_index_include? ⇒ Boolean

Does this adapter support including non-key columns?

Returns:

  • (Boolean)


466
467
468
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 466

def supports_index_include?
  false
end

#supports_index_sort_order? ⇒ Boolean

Does this adapter support index sort order?

Returns:

  • (Boolean)


456
457
458
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 456

def supports_index_sort_order?
  false
end

#supports_indexes_in_create? ⇒ Boolean

Does this adapter support creating indexes in the same statement as creating the table?

Returns:

  • (Boolean)


492
493
494
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 492

def supports_indexes_in_create?
  false
end

#supports_insert_conflict_target? ⇒ Boolean

Returns:

  • (Boolean)


591
592
593
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 591

def supports_insert_conflict_target?
  false
end

#supports_insert_on_duplicate_skip? ⇒ Boolean

Returns:

  • (Boolean)


583
584
585
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 583

def supports_insert_on_duplicate_skip?
  false
end

#supports_insert_on_duplicate_update? ⇒ Boolean

Returns:

  • (Boolean)


587
588
589
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 587

def supports_insert_on_duplicate_update?
  false
end

#supports_insert_returning? ⇒ Boolean

Returns:

  • (Boolean)


579
580
581
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 579

def supports_insert_returning?
  false
end

#supports_json? ⇒ Boolean

Does this adapter support JSON data type?

Returns:

  • (Boolean)


542
543
544
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 542

def supports_json?
  false
end

#supports_lazy_transactions? ⇒ Boolean

Returns:

  • (Boolean)


575
576
577
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 575

def supports_lazy_transactions?
  false
end

#supports_materialized_views? ⇒ Boolean

Does this adapter support materialized views?

Returns:

  • (Boolean)


532
533
534
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 532

def supports_materialized_views?
  false
end

#supports_nulls_not_distinct? ⇒ Boolean

Returns:

  • (Boolean)


599
600
601
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 599

def supports_nulls_not_distinct?
  false
end

#supports_optimizer_hints? ⇒ Boolean

Does this adapter support optimizer hints?

Returns:

  • (Boolean)


567
568
569
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 567

def supports_optimizer_hints?
  false
end

#supports_partial_index? ⇒ Boolean

Does this adapter support partial indices?

Returns:

  • (Boolean)


461
462
463
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 461

def supports_partial_index?
  false
end

#supports_partitioned_indexes? ⇒ Boolean

Returns:

  • (Boolean)


451
452
453
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 451

def supports_partitioned_indexes?
  false
end

#supports_restart_db_transaction? ⇒ Boolean

Returns:

  • (Boolean)


435
436
437
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 435

def supports_restart_db_transaction?
  false
end

#supports_savepoints? ⇒ Boolean

Does this adapter support savepoints?

Returns:

  • (Boolean)


425
426
427
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 425

def supports_savepoints?
  false
end

#supports_transaction_isolation? ⇒ Boolean

Does this adapter support setting the isolation level for a transaction?

Returns:

  • (Boolean)


481
482
483
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 481

def supports_transaction_isolation?
  false
end

#supports_unique_constraints? ⇒ Boolean

Does this adapter support creating unique constraints?

Returns:

  • (Boolean)


522
523
524
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 522

def supports_unique_constraints?
  false
end

#supports_validate_constraints? ⇒ Boolean

Does this adapter support creating invalid constraints?

Returns:

  • (Boolean)


502
503
504
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 502

def supports_validate_constraints?
  false
end

#supports_views? ⇒ Boolean

Does this adapter support views?

Returns:

  • (Boolean)


527
528
529
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 527

def supports_views?
  false
end

#supports_virtual_columns? ⇒ Boolean

Does this adapter support virtual columns?

Returns:

  • (Boolean)


557
558
559
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 557

def supports_virtual_columns?
  false
end

#throw_away! ⇒ Object

Removes the connection from the pool and disconnect it.



794
795
796
797
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 794

def throw_away!
  pool.remove self
  disconnect!
end

#unprepared_statement ⇒ Object



389
390
391
392
393
394
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 389

def unprepared_statement
  cache = prepared_statements_disabled_cache.add?(object_id) if @prepared_statements
  yield
ensure
  cache&.delete(object_id)
end

#valid_type?(type) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


280
281
282
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 280

def valid_type?(type) # :nodoc:
  self.class.valid_type?(type)
end

#verified? ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


856
857
858
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 856

def verified? # :nodoc:
  @verified
end

#verify! ⇒ Object

Checks whether the connection to the database is still active (i.e. not stale). This is done under the hood by calling #active?. If the connection is no longer active, then this method will reconnect to the database.



820
821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 820

def verify!
  unless active?
    @lock.synchronize do
      if @unconfigured_connection
        attempt_configure_connection do
          @raw_connection = @unconfigured_connection
          @unconfigured_connection = nil
          configure_connection
          @last_activity = Process.clock_gettime(Process::CLOCK_MONOTONIC)
          @verified = true
          @allow_preconnect = true
        end
        return
      end

      reconnect!(restore_transactions: true)
    end
  end

  @last_activity = Process.clock_gettime(Process::CLOCK_MONOTONIC)
  @verified = true
end

#verify_timeout ⇒ Object



225
226
227
# File 'lib/active_record/connection_adapters/abstract_adapter.rb', line 225

def verify_timeout
  (@config[:verify_timeout] || 2).to_i
end