Method: ActiveRecord::ConnectionAdapters::ConnectionPool#disconnect!
- Defined in:
- lib/active_record/connection_adapters/abstract/connection_pool.rb
#disconnect! ⇒ Object
Disconnects all connections in the pool, and clears the pool.
The pool first tries to gain ownership of all connections. If unable to do so within a timeout interval (default duration is spec.db_config.checkout_timeout * 2 seconds), then the pool is forcefully disconnected without any regard for other connection owning threads.
546 547 548 |
# File 'lib/active_record/connection_adapters/abstract/connection_pool.rb', line 546 def disconnect! disconnect(false) end |