Method: ActiveRecord::ConnectionAdapters::DatabaseStatements#exec_insert

Defined in:
lib/active_record/connection_adapters/abstract/database_statements.rb

#exec_insert(sql, name, binds) ⇒ Object

Executes insert sql statement in the context of this connection using binds as the bind substitutes. name is the logged along with the executed sql statement.



62
63
64
# File 'lib/active_record/connection_adapters/abstract/database_statements.rb', line 62

def exec_insert(sql, name, binds)
  exec_query(sql, name, binds)
end