Method: ActiveRecord::ConnectionAdapters::DatabaseStatements#exec_query

Defined in:
lib/active_record/connection_adapters/abstract/database_statements.rb

#exec_query(sql, name = 'SQL', binds = []) ⇒ Object

Executes sql statement in the context of this connection using binds as the bind substitutes. name is logged along with the executed sql statement.



61
62
# File 'lib/active_record/connection_adapters/abstract/database_statements.rb', line 61

def exec_query(sql, name = 'SQL', binds = [])
end