Method: ActiveRecord::ConnectionAdapters::DatabaseStatements#exec_update

Defined in:
lib/active_record/connection_adapters/abstract/database_statements.rb

#exec_update(sql, name = nil, binds = []) ⇒ Object

Executes update sql statement in the context of this connection using binds as the bind substitutes. name is logged along with the executed sql statement.



135
136
137
# File 'lib/active_record/connection_adapters/abstract/database_statements.rb', line 135

def exec_update(sql, name = nil, binds = [])
  exec_query(sql, name, binds)
end