Module: ActiveRecord::ConnectionAdapters::PostgreSQL::DatabaseStatements

Included in:
ActiveRecord::ConnectionAdapters::PostgreSQLAdapter
Defined in:
lib/active_record/connection_adapters/postgresql/database_statements.rb

Instance Method Summary collapse

Instance Method Details

#begin_db_transaction ⇒ Object

Begins a transaction.



49
50
51
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 49

def begin_db_transaction # :nodoc:
  internal_execute("BEGIN", "TRANSACTION", allow_retry: true, materialize_transactions: false)
end

#begin_isolated_db_transaction(isolation) ⇒ Object

:nodoc:



53
54
55
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 53

def begin_isolated_db_transaction(isolation) # :nodoc:
  internal_execute("BEGIN ISOLATION LEVEL #{transaction_isolation_levels.fetch(isolation)}", "TRANSACTION", allow_retry: true, materialize_transactions: false)
end

#build_explain_clause(options = []) ⇒ Object



81
82
83
84
85
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 81

def build_explain_clause(options = [])
  return "EXPLAIN" if options.empty?

  "EXPLAIN (#{options.join(", ").upcase})"
end

#commit_db_transaction ⇒ Object

Commits a transaction.



58
59
60
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 58

def commit_db_transaction # :nodoc:
  internal_execute("COMMIT", "TRANSACTION", allow_retry: false, materialize_transactions: true)
end

#exec_insert(sql, name = nil, binds = [], pk = nil, sequence_name = nil, returning: nil) ⇒ Object

:nodoc:



30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 30

def exec_insert(sql, name = nil, binds = [], pk = nil, sequence_name = nil, returning: nil) # :nodoc:
  if use_insert_returning? || pk == false
    super
  else
    result = internal_exec_query(sql, name, binds)
    unless sequence_name
      table_ref = extract_table_ref_from_insert_sql(sql)
      if table_ref
        pk = primary_key(table_ref) if pk.nil?
        pk = suppress_composite_primary_key(pk)
        sequence_name = default_sequence_name(table_ref, pk)
      end
      return result unless sequence_name
    end
    last_insert_id_result(sequence_name)
  end
end

#exec_restart_db_transaction ⇒ Object

:nodoc:



68
69
70
71
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 68

def exec_restart_db_transaction # :nodoc:
  cancel_any_running_query
  internal_execute("ROLLBACK AND CHAIN", "TRANSACTION", allow_retry: false, materialize_transactions: true)
end

#exec_rollback_db_transaction ⇒ Object

Aborts a transaction.



63
64
65
66
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 63

def exec_rollback_db_transaction # :nodoc:
  cancel_any_running_query
  internal_execute("ROLLBACK", "TRANSACTION", allow_retry: false, materialize_transactions: true)
end

#explain(arel, binds = [], options = []) ⇒ Object



7
8
9
10
11
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 7

def explain(arel, binds = [], options = [])
  sql    = build_explain_clause(options) + " " + to_sql(arel, binds)
  result = internal_exec_query(sql, "EXPLAIN", binds)
  PostgreSQL::ExplainPrettyPrinter.new.pp(result)
end

#high_precision_current_timestamp ⇒ Object



77
78
79
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 77

def high_precision_current_timestamp
  HIGH_PRECISION_CURRENT_TIMESTAMP
end

#query(sql, name = nil, allow_retry: true, materialize_transactions: true) ⇒ Object

Queries the database and returns the results in an Array-like object



14
15
16
17
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 14

def query(sql, name = nil, allow_retry: true, materialize_transactions: true) # :nodoc:
  result = internal_execute(sql, name, allow_retry:, materialize_transactions:)
  result.map_types!(@type_map_for_results).values
end

#set_constraints(deferred, *constraints) ⇒ Object

Set when constraints will be checked for the current transaction.

Not passing any specific constraint names will set the value for all deferrable constraints.

[deferred] Valid values are :deferred or :immediate.

See https://www.postgresql.org/docs/current/sql-set-constraints.html



95
96
97
98
99
100
101
102
103
104
105
106
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 95

def set_constraints(deferred, *constraints)
  unless %i[deferred immediate].include?(deferred)
    raise ArgumentError, "deferred must be :deferred or :immediate"
  end

  constraints = if constraints.empty?
    "ALL"
  else
    constraints.map { |c| quote_table_name(c) }.join(", ")
  end
  execute("SET CONSTRAINTS #{constraints} #{deferred.to_s.upcase}")
end

#write_query?(sql) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


24
25
26
27
28
# File 'lib/active_record/connection_adapters/postgresql/database_statements.rb', line 24

def write_query?(sql) # :nodoc:
  !READ_QUERY.match?(sql)
rescue ArgumentError # Invalid encoding
  !READ_QUERY.match?(sql.b)
end