Module: ActiveRecord::ConnectionAdapters::QueryCache

Included in:
AbstractAdapter
Defined in:
lib/active_record/connection_adapters/abstract/query_cache.rb

Defined Under Namespace

Modules: ConnectionPoolConfiguration Classes: QueryCacheRegistry, Store

Constant Summary collapse

DEFAULT_SIZE =

:nodoc:

100

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#query_cacheObject



201
202
203
204
205
206
207
208
209
210
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 201

def query_cache
  if @pinned && @owner != ActiveSupport::IsolatedExecutionState.context
    # With transactional tests, if the connection is pinned, any thread
    # other than the one that pinned the connection need to go through the
    # query cache pool, so each thread get a different cache.
    pool.query_cache
  else
    @query_cache
  end
end

Class Method Details

.dirties_query_cache(base, *method_names) ⇒ Object



20
21
22
23
24
25
26
27
28
29
30
31
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 20

def dirties_query_cache(base, *method_names)
  method_names.each do |method_name|
    base.class_eval <<-end_code, __FILE__, __LINE__ + 1
      def #{method_name}(...)
        if pool.dirties_query_cache
          ActiveRecord::Base.clear_query_caches_for_current_thread
        end
        super
      end
    end_code
  end
end

.included(base) ⇒ Object

:nodoc:



12
13
14
15
16
17
18
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 12

def included(base) # :nodoc:
  dirties_query_cache base, :exec_query, :execute, :create, :insert, :update, :delete, :truncate,
    :truncate_tables, :rollback_to_savepoint, :rollback_db_transaction, :restart_db_transaction,
    :exec_insert_all

  base.set_callback :checkin, :after, :unset_query_cache!
end

Instance Method Details

#cache(&block) ⇒ Object

Enable the query cache within the block.



217
218
219
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 217

def cache(&block)
  pool.enable_query_cache(&block)
end

#clear_query_cacheObject

Clears the query cache.

One reason you may wish to call this method explicitly is between queries that ask the database to randomize results. Otherwise the cache would see the same SQL query and repeatedly return the same result each time, silently undermining the randomness you were expecting.



243
244
245
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 243

def clear_query_cache
  pool.clear_query_cache
end

#disable_query_cache!Object



233
234
235
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 233

def disable_query_cache!
  pool.disable_query_cache!
end

#enable_query_cache!Object



221
222
223
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 221

def enable_query_cache!
  pool.enable_query_cache!
end

#initializeObject



194
195
196
197
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 194

def initialize(*)
  super
  @query_cache = nil
end

#query_cache_enabledObject



212
213
214
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 212

def query_cache_enabled
  query_cache&.enabled?
end

#select_all(arel, name = nil, binds = [], preparable: nil, async: false, allow_retry: false) ⇒ Object

:nodoc:



247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 247

def select_all(arel, name = nil, binds = [], preparable: nil, async: false, allow_retry: false) # :nodoc:
  arel = arel_from_relation(arel)

  # If arel is locked this is a SELECT ... FOR UPDATE or somesuch.
  # Such queries should not be cached.
  if query_cache_enabled && !(arel.respond_to?(:locked) && arel.locked)
    sql, binds, preparable, allow_retry = to_sql_and_binds(arel, binds, preparable)

    if async
      result = lookup_sql_cache(sql, name, binds) || super(sql, name, binds, preparable: preparable, async: async, allow_retry: allow_retry)
      FutureResult.wrap(result)
    else
      cache_sql(sql, name, binds) { super(sql, name, binds, preparable: preparable, async: async, allow_retry: allow_retry) }
    end
  else
    super
  end
end

#uncached(dirties: true, &block) ⇒ Object

Disable the query cache within the block.

Set dirties: false to prevent query caches on all connections from being cleared by write operations. (By default, write operations dirty all connections’ query caches in case they are replicas whose cache would now be outdated.)



229
230
231
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 229

def uncached(dirties: true, &block)
  pool.disable_query_cache(dirties: dirties, &block)
end