Method: ActiveRecord::ConnectionAdapters::QueryCache#uncached

Defined in:
lib/active_record/connection_adapters/abstract/query_cache.rb

#uncached(dirties: true, &block) ⇒ Object

Disable the query cache within the block.

Set dirties: false to prevent query caches on all connections from being cleared by write operations. (By default, write operations dirty all connections’ query caches in case they are replicas whose cache would now be outdated.)



245
246
247
# File 'lib/active_record/connection_adapters/abstract/query_cache.rb', line 245

def uncached(dirties: true, &block)
  pool.disable_query_cache(dirties: dirties, &block)
end