Class: ActiveRecord::ConnectionAdapters::Table

Inherits:
Object
  • Object
show all
Defined in:
lib/active_record/connection_adapters/abstract/schema_definitions.rb

Overview

Represents an SQL table in an abstract way for updating a table. Also see TableDefinition and SchemaStatements#create_table

Available transformations are:

change_table :table do |t|
t.column
t.index
t.rename_index
t.timestamps
t.change
t.change_default
t.rename
t.references
t.belongs_to
t.string
t.text
t.integer
t.float
t.decimal
t.datetime
t.timestamp
t.time
t.date
t.binary
t.boolean
t.remove
t.remove_references
t.remove_belongs_to
t.remove_index
t.remove_timestamps
end

Direct Known Subclasses

PostgreSQL::Table

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(table_name, base) ⇒ Table

Returns a new instance of Table.



419
420
421
422
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 419

def initialize(table_name, base)
  @name = table_name
  @base = base
end

Instance Attribute Details

#nameObject (readonly)

Returns the value of attribute name.



417
418
419
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 417

def name
  @name
end

Instance Method Details

#change(column_name, type, options = {}) ⇒ Object

Changes the column's definition according to the new options. See TableDefinition#column for details of the options you can use.

t.change(:name, :string, limit: 80) t.change(:description, :text)



475
476
477
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 475

def change(column_name, type, options = {})
  @base.change_column(name, column_name, type, options)
end

#change_default(column_name, default) ⇒ Object

Sets a new default value for a column. See SchemaStatements#change_column_default

t.change_default(:qualification, 'new') t.change_default(:authorized, 1)



483
484
485
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 483

def change_default(column_name, default)
  @base.change_column_default(name, column_name, default)
end

#column(column_name, type, options = {}) ⇒ Object

Adds a new column to the named table. See TableDefinition#column for details of the options you can use.

Creating a simple column

t.column(:name, :string)



429
430
431
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 429

def column(column_name, type, options = {})
  @base.add_column(name, column_name, type, options)
end

#column_exists?(column_name, type = nil, options = {}) ⇒ Boolean

Checks to see if a column exists. See SchemaStatements#column_exists?

Returns:

  • (Boolean)


434
435
436
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 434

def column_exists?(column_name, type = nil, options = {})
  @base.column_exists?(name, column_name, type, options)
end

#index(column_name, options = {}) ⇒ Object

Adds a new index to the table. column_name can be a single Symbol, or an Array of Symbols. See SchemaStatements#add_index

Creating a simple index

t.index(:name)

Creating a unique index

t.index([:branch_id, :party_id], unique: true)

Creating a named index

t.index([:branch_id, :party_id], unique: true, name: 'by_branch_party')



447
448
449
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 447

def index(column_name, options = {})
  @base.add_index(name, column_name, options)
end

#index_exists?(column_name, options = {}) ⇒ Boolean

Checks to see if an index exists. See SchemaStatements#index_exists?

Returns:

  • (Boolean)


452
453
454
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 452

def index_exists?(column_name, options = {})
  @base.index_exists?(name, column_name, options)
end

#references(*args) ⇒ Object Also known as: belongs_to

Adds a reference. Optionally adds a type column, if :polymorphic option is provided. references and belongs_to are acceptable. The reference column will be an integer by default, the :type option can be used to specify a different type.

t.references(:user) t.references(:user, type: "string") t.belongs_to(:supplier, polymorphic: true)

See SchemaStatements#add_reference



532
533
534
535
536
537
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 532

def references(*args)
  options = args.extract_options!
  args.each do |ref_name|
    @base.add_reference(name, ref_name, options)
  end
end

#remove(*column_names) ⇒ Object

Removes the column(s) from the table definition.

t.remove(:qualification) t.remove(:qualification, :experience)



491
492
493
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 491

def remove(*column_names)
  @base.remove_columns(name, *column_names)
end

#remove_index(options = {}) ⇒ Object

Removes the given index from the table.

Remove the index_table_name_on_column in the table_name table

t.remove_index :column

Remove the index named index_table_name_on_branch_id in the table_name table

t.remove_index column: :branch_id

Remove the index named index_table_name_on_branch_id_and_party_id in the table_name table

t.remove_index column: [:branch_id, :party_id]

Remove the index named by_branch_party in the table_name table

t.remove_index name: :by_branch_party



505
506
507
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 505

def remove_index(options = {})
  @base.remove_index(name, options)
end

#remove_references(*args) ⇒ Object Also known as: remove_belongs_to

Removes a reference. Optionally removes a type column. remove_references and remove_belongs_to are acceptable.

t.remove_references(:user) t.remove_belongs_to(:supplier, polymorphic: true)

See SchemaStatements#remove_reference



547
548
549
550
551
552
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 547

def remove_references(*args)
  options = args.extract_options!
  args.each do |ref_name|
    @base.remove_reference(name, ref_name, options)
  end
end

#remove_timestamps(options = {}) ⇒ Object

Removes the timestamp columns (+created_at+ and updated_at) from the table.

t.remove_timestamps



512
513
514
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 512

def remove_timestamps(options = {})
  @base.remove_timestamps(name, options)
end

#rename(column_name, new_column_name) ⇒ Object

Renames a column.

t.rename(:description, :name)



519
520
521
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 519

def rename(column_name, new_column_name)
  @base.rename_column(name, column_name, new_column_name)
end

#rename_index(index_name, new_index_name) ⇒ Object

Renames the given index on the table.

t.rename_index(:user_id, :account_id)



459
460
461
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 459

def rename_index(index_name, new_index_name)
  @base.rename_index(name, index_name, new_index_name)
end

#timestamps(options = {}) ⇒ Object

Adds timestamps (+created_at+ and updated_at) columns to the table. See SchemaStatements#add_timestamps

t.timestamps null: false



466
467
468
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 466

def timestamps(options = {})
  @base.add_timestamps(name, options)
end