Method: ActiveRecord::Integration#to_param
- Defined in:
- lib/active_record/integration.rb
#to_param ⇒ Object
Returns a String, which Action Pack uses for constructing an URL to this object. The default implementation returns this record’s id as a String, or nil if this record’s unsaved.
For example, suppose that you have a User model, and that you have a resources :users route. Normally, user_path will construct a path with the user object’s ‘id’ in it:
user = User.find_by(name: 'Phusion')
user_path(user) # => "/users/1"
You can override to_param in your model to make user_path construct a path using the user’s name instead of the user’s id:
class User < ActiveRecord::Base
def to_param # overridden
name
end
end
user = User.find_by(name: 'Phusion')
user_path(user) # => "/users/Phusion"
38 39 40 41 |
# File 'lib/active_record/integration.rb', line 38 def to_param # We can't use alias_method here, because method 'id' optimizes itself on the fly. id && id.to_s # Be sure to stringify the id for routes end |