Method: ActiveRecord::Persistence#destroy

Defined in:
lib/active_record/persistence.rb

#destroyObject

Deletes the record in the database and freezes this instance to reflect that no changes should be made (since they can’t be persisted).

There’s a series of callbacks associated with destroy. If the before_destroy callback return false the action is cancelled and destroy returns false. See ActiveRecord::Callbacks for further details.

Raises:



151
152
153
154
155
156
157
# File 'lib/active_record/persistence.rb', line 151

def destroy
  raise ReadOnlyRecord if readonly?
  destroy_associations
  destroy_row if persisted?
  @destroyed = true
  freeze
end