Method: ActiveRecord::ProtectedEnvironmentError#initialize
- Defined in:
- lib/active_record/migration.rb
#initialize(env = "production") ⇒ ProtectedEnvironmentError
Returns a new instance of ProtectedEnvironmentError.
159 160 161 162 163 164 |
# File 'lib/active_record/migration.rb', line 159 def initialize(env = "production") msg = "You are attempting to run a destructive action against your '#{env}' database.\n" msg << "If you are sure you want to continue, run the same command with the environment variable:\n" msg << "DISABLE_DATABASE_ENVIRONMENT_CHECK=1" super(msg) end |