Refresh

This website rubydoc.info/gems/activerecord/3.1.10/ActiveRecord/QueryCache/ClassMethods is currently offline. Cloudflare\'s Always Online™ shows a snapshot of this web page from the Internet Archive\'s Wayback Machine. To check for the live version, click Refresh.

Module: ActiveRecord::QueryCache::ClassMethods

Defined in:
lib/active_record/query_cache.rb

Instance Method Summary collapse

Instance Method Details

#cache(&block) ⇒ Object

Enable the query cache within the block if Active Record is configured.



8
9
10
11
12
13
14
# File 'lib/active_record/query_cache.rb', line 8

def cache(&block)
  if ActiveRecord::Base.configurations.blank?
    yield
  else
    connection.cache(&block)
  end
end

#uncached(&block) ⇒ Object

Disable the query cache within the block if Active Record is configured.



17
18
19
20
21
22
23
# File 'lib/active_record/query_cache.rb', line 17

def uncached(&block)
  if ActiveRecord::Base.configurations.blank?
    yield
  else
    connection.uncached(&block)
  end
end