Module: ActiveRecord::QueryMethods

Includes:
ActiveModel::ForbiddenAttributesProtection
Included in:
Relation
Defined in:
lib/active_record/relation/query_methods.rb

Defined Under Namespace

Classes: CTEJoin, WhereChain

Constant Summary collapse

FROZEN_EMPTY_ARRAY =
[].freeze
FROZEN_EMPTY_HASH =
{}.freeze
VALID_UNSCOPING_VALUES =
Set.new([:where, :select, :group, :order, :lock,
:limit, :offset, :joins, :left_outer_joins, :annotate,
:includes, :eager_load, :preload, :from, :readonly,
:having, :optimizer_hints, :with])

Instance Method Summary collapse

Instance Method Details

#_select!(*fields) ⇒ Object

:nodoc:



428
429
430
431
# File 'lib/active_record/relation/query_methods.rb', line 428

def _select!(*fields) # :nodoc:
  self.select_values |= fields
  self
end

#all ⇒ Object

:nodoc:



260
261
262
# File 'lib/active_record/relation/query_methods.rb', line 260

def all # :nodoc:
  spawn
end

#and(other) ⇒ Object

Returns a new relation, which is the logical intersection of this relation and the one passed as an argument.

The two relations must be structurally compatible: they must be scoping the same model, and they must differ only by #where (if no #group has been defined) or #having (if a #group is present).

Post.where(id: [1, 2]).and(Post.where(id: [2, 3]))
# SELECT `posts`.* FROM `posts` WHERE `posts`.`id` IN (1, 2) AND `posts`.`id` IN (2, 3)


1163
1164
1165
1166
1167
1168
1169
# File 'lib/active_record/relation/query_methods.rb', line 1163

def and(other)
  if other.is_a?(Relation)
    spawn.and!(other)
  else
    raise ArgumentError, "You have passed #{other.class.name} object to #and. Pass an ActiveRecord::Relation object instead."
  end
end

#and!(other) ⇒ Object

:nodoc:



1171
1172
1173
1174
1175
1176
1177
1178
1179
1180
1181
1182
1183
# File 'lib/active_record/relation/query_methods.rb', line 1171

def and!(other) # :nodoc:
  incompatible_values = structurally_incompatible_values_for(other)

  unless incompatible_values.empty?
    raise ArgumentError, "Relation passed to #and must be structurally compatible. Incompatible values: #{incompatible_values}"
  end

  self.where_clause |= other.where_clause
  self.having_clause |= other.having_clause
  self.references_values |= other.references_values

  self
end

#annotate(*args) ⇒ Object

Adds an SQL comment to queries generated from this relation. For example:

User.annotate("selecting user names").select(:name)
# SELECT "users"."name" FROM "users" /* selecting user names */

User.annotate("selecting", "user", "names").select(:name)
# SELECT "users"."name" FROM "users" /* selecting */ /* user */ /* names */

The SQL block comment delimiters, "/" and "/", will be added automatically.

Some escaping is performed, however untrusted user input should not be used.



1558
1559
1560
1561
# File 'lib/active_record/relation/query_methods.rb', line 1558

def annotate(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.annotate!(*args)
end

#annotate!(*args) ⇒ Object

Like #annotate, but modifies relation in place.



1564
1565
1566
1567
# File 'lib/active_record/relation/query_methods.rb', line 1564

def annotate!(*args) # :nodoc:
  self.annotate_values |= args
  self
end

#arel(aliases = nil) ⇒ Object

Returns the Arel object associated with the relation.



1623
1624
1625
# File 'lib/active_record/relation/query_methods.rb', line 1623

def arel(aliases = nil) # :nodoc:
  @arel ||= build_arel(aliases)
end

#construct_join_dependency(associations, join_type) ⇒ Object

:nodoc:



1627
1628
1629
1630
1631
# File 'lib/active_record/relation/query_methods.rb', line 1627

def construct_join_dependency(associations, join_type) # :nodoc:
  ActiveRecord::Associations::JoinDependency.new(
    model, table, associations, join_type
  )
end

#create_with(value) ⇒ Object

Sets attributes to be used when creating new records from a relation object.

users = User.where(name: 'Oscar')
users.new.name # => 'Oscar'

users = users.create_with(name: 'DHH')
users.new.name # => 'DHH'

You can pass nil to #create_with to reset attributes:

users = users.create_with(nil)
users.new.name # => 'Oscar'


1375
1376
1377
# File 'lib/active_record/relation/query_methods.rb', line 1375

def create_with(value)
  spawn.create_with!(value)
end

#create_with!(value) ⇒ Object

:nodoc:



1379
1380
1381
1382
1383
1384
1385
1386
1387
1388
# File 'lib/active_record/relation/query_methods.rb', line 1379

def create_with!(value) # :nodoc:
  if value
    value = sanitize_forbidden_attributes(value)
    self.create_with_value = create_with_value.merge(value)
  else
    self.create_with_value = FROZEN_EMPTY_HASH
  end

  self
end

#distinct(value = true) ⇒ Object

Specifies whether the records should be unique or not. For example:

User.select(:name)
# Might return two records with the same name

User.select(:name).distinct
# Returns 1 record per distinct name

User.select(:name).distinct.distinct(false)
# You can also remove the uniqueness


1439
1440
1441
# File 'lib/active_record/relation/query_methods.rb', line 1439

def distinct(value = true)
  spawn.distinct!(value)
end

#distinct!(value = true) ⇒ Object

Like #distinct, but modifies relation in place.



1444
1445
1446
1447
# File 'lib/active_record/relation/query_methods.rb', line 1444

def distinct!(value = true) # :nodoc:
  self.distinct_value = value
  self
end

#eager_load(*args) ⇒ Object

Specify associations args to be eager loaded using a LEFT OUTER JOIN. Performs a single query joining all specified associations. For example:

users = User.eager_load(:address).limit(5)
users.each do |user|
user.address.city
end

# SELECT "users"."id" AS t0_r0, "users"."name" AS t0_r1, ... FROM "users"
#   LEFT OUTER JOIN "addresses" ON "addresses"."id" = "users"."address_id"
#   LIMIT 5

Instead of loading the 5 addresses with 5 separate queries, all addresses are loaded with a single joined query.

Loading multiple and nested associations is possible using Hashes and Arrays, similar to #includes:

User.eager_load(:address, friends: [:address, :followers])
# SELECT "users"."id" AS t0_r0, "users"."name" AS t0_r1, ... FROM "users"
#   LEFT OUTER JOIN "addresses" ON "addresses"."id" = "users"."address_id"
#   LEFT OUTER JOIN "friends" ON "friends"."user_id" = "users"."id"
#   ...

NOTE: Loading the associations in a join can result in many rows that contain redundant data and it performs poorly at scale.



290
291
292
293
# File 'lib/active_record/relation/query_methods.rb', line 290

def eager_load(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.eager_load!(*args)
end

#eager_load!(*args) ⇒ Object

:nodoc:



295
296
297
298
# File 'lib/active_record/relation/query_methods.rb', line 295

def eager_load!(*args) # :nodoc:
  self.eager_load_values |= args
  self
end

#excluding(*records) ⇒ Object Also known as: without

Excludes the specified record (or collection of records) from the resulting relation. For example:

Post.excluding(post)
# SELECT "posts".* FROM "posts" WHERE "posts"."id" != 1

Post.excluding(post_one, post_two)
# SELECT "posts".* FROM "posts" WHERE "posts"."id" NOT IN (1, 2)

Post.excluding(Post.drafts)
# SELECT "posts".* FROM "posts" WHERE "posts"."id" NOT IN (3, 4, 5)

This can also be called on associations. As with the above example, either a single record of collection thereof may be specified:

post = Post.find(1)
comment = Comment.find(2)
post.comments.excluding(comment)
# SELECT "comments".* FROM "comments" WHERE "comments"."post_id" = 1 AND "comments"."id" != 2

This is short-hand for .where.not(id: post.id) and .where.not(id: [post_one.id, post_two.id]).

An ArgumentError will be raised if either no records are specified, or if any of the records in the collection (if a collection is passed in) are not instances of the same model that the relation is scoping.



1603
1604
1605
1606
1607
1608
1609
1610
1611
1612
1613
# File 'lib/active_record/relation/query_methods.rb', line 1603

def excluding(*records)
  relations = records.extract! { |element| element.is_a?(Relation) }
  records.flatten!(1)
  records.compact!

  unless records.all?(model) && relations.all? { |relation| relation.model == model }
    raise ArgumentError, "You must only pass a single or collection of #{model.name} objects to ##{__callee__}."
  end

  spawn.excluding!(records + relations.flat_map(&:ids))
end

#excluding!(records) ⇒ Object

:nodoc:



1616
1617
1618
1619
1620
# File 'lib/active_record/relation/query_methods.rb', line 1616

def excluding!(records) # :nodoc:
  ids = records.map { |record| record.is_a?(model) ? record.id : record }
  self.where_clause += build_where_clause(primary_key => ids).invert
  self
end

#extending(*modules, &block) ⇒ Object

Used to extend a scope with additional methods, either through a module or through a block provided.

The object returned is a relation, which can be further extended.

Using a Module

module Pagination
def page(number)
  # pagination code goes here
end
end

scope = Model.all.extending(Pagination)
scope.page(params[:page])

You can also pass a list of modules:

scope = Model.all.extending(Pagination, SomethingElse)

Using a Block

scope = Model.all.extending do
def page(number)
  # pagination code goes here
end
end
scope.page(params[:page])

You can also use a block and a module list:

scope = Model.all.extending(Pagination) do
def per_page(number)
  # pagination code goes here
end
end


1485
1486
1487
1488
1489
1490
1491
# File 'lib/active_record/relation/query_methods.rb', line 1485

def extending(*modules, &block)
  if modules.any? || block
    spawn.extending!(*modules, &block)
  else
    self
  end
end

#extending!(*modules, &block) ⇒ Object

:nodoc:



1493
1494
1495
1496
1497
1498
1499
1500
1501
# File 'lib/active_record/relation/query_methods.rb', line 1493

def extending!(*modules, &block) # :nodoc:
  modules << Module.new(&block) if block
  modules.flatten!

  self.extending_values |= modules
  extend(*extending_values) if extending_values.any?

  self
end

#extract_associated(association) ⇒ Object

Extracts a named association from the relation. The named association is first preloaded, then the individual association records are collected from the relation. Like so:

.memberships.extract_associated(:user)
# => Returns collection of User records

This is short-hand for:

.memberships.preload(:user).collect(&:user)


341
342
343
# File 'lib/active_record/relation/query_methods.rb', line 341

def extract_associated(association)
  preload(association).collect(&association)
end

#from(value, subquery_name = nil) ⇒ Object

Specifies the table from which the records will be fetched. For example:

Topic.select('title').from('posts')
# SELECT title FROM posts

Can accept other relation objects. For example:

Topic.select('title').from(Topic.approved)
# SELECT title FROM (SELECT * FROM topics WHERE approved = 't') subquery

Passing a second argument (string or symbol), creates the alias for the SQL from clause. Otherwise the alias "subquery" is used:

Topic.select('a.title').from(Topic.approved, :a)
# SELECT a.title FROM (SELECT * FROM topics WHERE approved = 't') a

It does not add multiple arguments to the SQL from clause. The last from chained is the one used:

Topic.select('title').from(Topic.approved).from(Topic.inactive)
# SELECT title FROM (SELECT topics.* FROM topics WHERE topics.active = 'f') subquery

For multiple arguments for the SQL from clause, you can pass a string with the exact elements in the SQL from list:

color = "red"
Color
.from("colors c, JSONB_ARRAY_ELEMENTS(colored_things) AS colorvalues(colorvalue)")
.where("colorvalue->>'color' = ?", color)
.select("c.*").to_a
# SELECT c.*
# FROM colors c, JSONB_ARRAY_ELEMENTS(colored_things) AS colorvalues(colorvalue)
# WHERE (colorvalue->>'color' = 'red')


1420
1421
1422
# File 'lib/active_record/relation/query_methods.rb', line 1420

def from(value, subquery_name = nil)
  spawn.from!(value, subquery_name)
end

#from!(value, subquery_name = nil) ⇒ Object

:nodoc:



1424
1425
1426
1427
# File 'lib/active_record/relation/query_methods.rb', line 1424

def from!(value, subquery_name = nil) # :nodoc:
  self.from_clause = Relation::FromClause.new(value, subquery_name)
  self
end

#group(*args) ⇒ Object

Allows to specify a group attribute:

User.group(:name)
# SELECT "users".* FROM "users" GROUP BY name

Returns an array with distinct records based on the group attribute:

User.select([:id, :name])
# => [#<User id: 1, name: "Oscar">, #<User id: 2, name: "Oscar">, #<User id: 3, name: "Foo">]

User.group(:name)
# => [#<User id: 3, name: "Foo", ...>, #<User id: 2, name: "Oscar", ...>]

User.group('name AS grouped_name, age')
# => [#<User id: 3, name: "Foo", age: 21, ...>, #<User id: 2, name: "Oscar", age: 21, ...>, #<User id: 5, name: "Foo", age: 23, ...>]

Passing in an array of attributes to group by is also supported.

User.select([:id, :first_name]).group(:id, :first_name).first(3)
# => [#<User id: 1, first_name: "Bill">, #<User id: 2, first_name: "Earl">, #<User id: 3, first_name: "Beto">]


573
574
575
576
# File 'lib/active_record/relation/query_methods.rb', line 573

def group(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.group!(*args)
end

#group!(*args) ⇒ Object

:nodoc:



578
579
580
581
# File 'lib/active_record/relation/query_methods.rb', line 578

def group!(*args) # :nodoc:
  self.group_values |= args
  self
end

#having(opts, *rest) ⇒ Object

Allows to specify a HAVING clause. Note that you can't use HAVING without also specifying a GROUP clause.

Order.having('SUM(price) > 30').group('user_id')


1225
1226
1227
# File 'lib/active_record/relation/query_methods.rb', line 1225

def having(opts, *rest)
  opts.blank? ? self : spawn.having!(opts, *rest)
end

#having!(opts, *rest) ⇒ Object

:nodoc:



1229
1230
1231
1232
# File 'lib/active_record/relation/query_methods.rb', line 1229

def having!(opts, *rest) # :nodoc:
  self.having_clause += build_having_clause(opts, rest)
  self
end

#in_order_of(column, values, filter: true) ⇒ Object

Applies an ORDER BY clause based on a given column, ordered and filtered by a specific set of values.

User.in_order_of(:id, [1, 5, 3])
# SELECT "users".* FROM "users"
#   WHERE "users"."id" IN (1, 5, 3)
#   ORDER BY CASE
#     WHEN "users"."id" = 1 THEN 1
#     WHEN "users"."id" = 5 THEN 2
#     WHEN "users"."id" = 3 THEN 3
#   END ASC

column can point to an enum column; the actual query generated may be different depending on the database adapter and the column definition.

class Conversation < ActiveRecord::Base
enum :status, [ :active, :archived ]
end

Conversation.in_order_of(:status, [:archived, :active])
# SELECT "conversations".* FROM "conversations"
#   WHERE "conversations"."status" IN (1, 0)
#   ORDER BY CASE
#     WHEN "conversations"."status" = 1 THEN 1
#     WHEN "conversations"."status" = 0 THEN 2
#   END ASC

values can also include nil.

Conversation.in_order_of(:status, [nil, :archived, :active])
# SELECT "conversations".* FROM "conversations"
#   WHERE ("conversations"."status" IN (1, 0) OR "conversations"."status" IS NULL)
#   ORDER BY CASE
#     WHEN "conversations"."status" IS NULL THEN 1
#     WHEN "conversations"."status" = 1 THEN 2
#     WHEN "conversations"."status" = 0 THEN 3
#   END ASC

filter can be set to false to include all results instead of only the ones specified in values.

Conversation.in_order_of(:status, [:archived, :active], filter: false)
# SELECT "conversations".* FROM "conversations"
#   ORDER BY CASE
#     WHEN "conversations"."status" = 1 THEN 1
#     WHEN "conversations"."status" = 0 THEN 2
#     ELSE 3
#   END ASC


717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
# File 'lib/active_record/relation/query_methods.rb', line 717

def in_order_of(column, values, filter: true)
  model.disallow_raw_sql!([column], permit: model.adapter_class.column_name_with_order_matcher)
  return spawn.none! if values.empty?

  references = column_references([column])
  self.references_values |= references unless references.empty?

  if column.is_a?(Arel::Nodes::SqlLiteral)
    arel_column = column
  else
    arel_column = order_column(column.to_s)

    unless arel_column.is_a?(Arel::Nodes::SqlLiteral)
      values = cast_values_for_in_order_of(values, arel_column.type_caster)
    end
    return spawn.none! if values.empty?
  end

  scope = spawn.order!(build_case_for_value_position(arel_column, values, filter: filter))

  if filter
    where_clause =
      if values.include?(nil)
        arel_column.in(values.compact).or(arel_column.eq(nil))
      else
        arel_column.in(values)
      end

    scope = scope.where!(where_clause)
  end

  scope
end

#includes(*args) ⇒ Object

Specify associations args to be eager loaded to prevent N + 1 queries. A separate query is performed for each association, unless a join is required by conditions.

For example:

users = User.includes(:address).limit(5)
users.each do |user|
user.address.city
end

# SELECT "users".* FROM "users" LIMIT 5
# SELECT "addresses".* FROM "addresses" WHERE "addresses"."id" IN (1,2,3,4,5)

Instead of loading the 5 addresses with 5 separate queries, all addresses are loaded with a single query.

Loading the associations in a separate query will often result in a performance improvement over a simple join, as a join can result in many rows that contain redundant data and it performs poorly at scale.

You can also specify multiple associations. Each association will result in an additional query:

User.includes(:address, :friends).to_a
# SELECT "users".* FROM "users"
# SELECT "addresses".* FROM "addresses" WHERE "addresses"."id" IN (1,2,3,4,5)
# SELECT "friends".* FROM "friends" WHERE "friends"."user_id" IN (1,2,3,4,5)

Loading nested associations is possible using a Hash:

User.includes(:address, friends: [:address, :followers])

Conditions

If you want to add string conditions to your included models, you'll have to explicitly reference them. For example:

User.includes(:posts).where('posts.name = ?', 'example').to_a

Will throw an error, but this will work:

User.includes(:posts).where('posts.name = ?', 'example').references(:posts).to_a
# SELECT "users"."id" AS t0_r0, ... FROM "users"
#   LEFT OUTER JOIN "posts" ON "posts"."user_id" = "users"."id"
#   WHERE "posts"."name" = ?  [["name", "example"]]

As the LEFT OUTER JOIN already contains the posts, the second query for the posts is no longer performed.

Note that #includes works with association names while #references needs the actual table name.

If you pass the conditions via a Hash, you don't need to call #references explicitly, as #where references the tables for you. For example, this will work correctly:

User.includes(:posts).where(posts: { name: 'example' })

NOTE: Conditions affect both sides of an association. For example, the above code will return only users that have a post named "example", and will only include posts named "example", even when a matching user has other additional posts.



250
251
252
253
# File 'lib/active_record/relation/query_methods.rb', line 250

def includes(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.includes!(*args)
end

#includes!(*args) ⇒ Object

:nodoc:



255
256
257
258
# File 'lib/active_record/relation/query_methods.rb', line 255

def includes!(*args) # :nodoc:
  self.includes_values |= args
  self
end

#invert_where ⇒ Object

Allows you to invert an entire where clause instead of manually applying conditions.

class User
scope :active, -> { where(accepted: true, locked: false) }
end

User.where(accepted: true)
# WHERE `accepted` = true

User.where(accepted: true).invert_where
# WHERE `accepted` != true

User.active
# WHERE `accepted` = true AND `locked` = false

User.active.invert_where
# WHERE NOT (`accepted` = true AND `locked` = false)

Be careful because this inverts all conditions before invert_where call.

class User
scope :active, -> { where(accepted: true, locked: false) }
scope :inactive, -> { active.invert_where } # Do not attempt it
end

# It also inverts `where(role: 'admin')` unexpectedly.
User.where(role: 'admin').inactive
# WHERE NOT (`role` = 'admin' AND `accepted` = true AND `locked` = false)


1129
1130
1131
# File 'lib/active_record/relation/query_methods.rb', line 1129

def invert_where
  spawn.invert_where!
end

#invert_where! ⇒ Object

:nodoc:



1133
1134
1135
1136
# File 'lib/active_record/relation/query_methods.rb', line 1133

def invert_where! # :nodoc:
  self.where_clause = where_clause.invert
  self
end

#joins(*args) ⇒ Object

Performs JOINs on args. The given symbol(s) should match the name of the association(s).

User.joins(:posts)
# SELECT "users".*
# FROM "users"
# INNER JOIN "posts" ON "posts"."user_id" = "users"."id"

Multiple joins:

User.joins(:posts, :account)
# SELECT "users".*
# FROM "users"
# INNER JOIN "posts" ON "posts"."user_id" = "users"."id"
# INNER JOIN "accounts" ON "accounts"."id" = "users"."account_id"

Nested joins:

User.joins(posts: [:comments])
# SELECT "users".*
# FROM "users"
# INNER JOIN "posts" ON "posts"."user_id" = "users"."id"
# INNER JOIN "comments" ON "comments"."post_id" = "posts"."id"

You can use strings in order to customize your joins:

User.joins("LEFT JOIN bookmarks ON bookmarks.bookmarkable_type = 'Post' AND bookmarks.user_id = users.id")
# SELECT "users".* FROM "users" LEFT JOIN bookmarks ON bookmarks.bookmarkable_type = 'Post' AND bookmarks.user_id = users.id


896
897
898
899
# File 'lib/active_record/relation/query_methods.rb', line 896

def joins(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.joins!(*args)
end

#joins!(*args) ⇒ Object

:nodoc:



901
902
903
904
# File 'lib/active_record/relation/query_methods.rb', line 901

def joins!(*args) # :nodoc:
  self.joins_values |= args
  self
end

#left_outer_joins(*args) ⇒ Object Also known as: left_joins

Performs LEFT OUTER JOINs on args:

User.left_outer_joins(:posts)
# SELECT "users".* FROM "users" LEFT OUTER JOIN "posts" ON "posts"."user_id" = "users"."id"


911
912
913
914
# File 'lib/active_record/relation/query_methods.rb', line 911

def left_outer_joins(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.left_outer_joins!(*args)
end

#left_outer_joins!(*args) ⇒ Object

:nodoc:



917
918
919
920
# File 'lib/active_record/relation/query_methods.rb', line 917

def left_outer_joins!(*args) # :nodoc:
  self.left_outer_joins_values |= args
  self
end

#limit(value) ⇒ Object

Specifies a limit for the number of records to retrieve.

User.limit(10) # generated SQL has 'LIMIT 10'

User.limit(10).limit(20) # generated SQL has 'LIMIT 20'


1239
1240
1241
# File 'lib/active_record/relation/query_methods.rb', line 1239

def limit(value)
  spawn.limit!(value)
end

#limit!(value) ⇒ Object

:nodoc:



1243
1244
1245
1246
1247
# File 'lib/active_record/relation/query_methods.rb', line 1243

def limit!(value) # :nodoc:
  value = Integer(value) unless value.nil?
  self.limit_value = value
  self
end

#lock(locks = true) ⇒ Object

Specifies locking settings (default to true). For more information on locking, please see ActiveRecord::Locking.



1267
1268
1269
# File 'lib/active_record/relation/query_methods.rb', line 1267

def lock(locks = true)
  spawn.lock!(locks)
end

#lock!(locks = true) ⇒ Object

:nodoc:



1271
1272
1273
1274
1275
1276
1277
1278
1279
1280
# File 'lib/active_record/relation/query_methods.rb', line 1271

def lock!(locks = true) # :nodoc:
  case locks
  when String, TrueClass, NilClass
    self.lock_value = locks || true
  else
    self.lock_value = false
  end

  self
end

#none ⇒ Object

Returns a chainable relation with zero records.

The returned relation implements the Null Object pattern. It is an object with defined null behavior and always returns an empty array of records without querying the database.

Any subsequent condition chained to the returned relation will continue generating an empty relation and will not fire any query to the database.

Used in cases where a method or scope could return zero records but the result needs to be chainable.

For example:

@posts = current_user.visible_posts.where(name: params[:name])
# the visible_posts method is expected to return a chainable Relation

def visible_posts
case role
when 'Country Manager'
  Post.where(country: country)
when 'Reviewer'
  Post.published
when 'Bad User'
  Post.none # It can't be chained if [] is returned.
end
end


1310
1311
1312
# File 'lib/active_record/relation/query_methods.rb', line 1310

def none
  spawn.none!
end

#none! ⇒ Object

:nodoc:



1314
1315
1316
1317
1318
1319
1320
# File 'lib/active_record/relation/query_methods.rb', line 1314

def none! # :nodoc:
  unless @none
    where!("1=0")
    @none = true
  end
  self
end

#null_relation? ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


1322
1323
1324
# File 'lib/active_record/relation/query_methods.rb', line 1322

def null_relation? # :nodoc:
  @none
end

#offset(value) ⇒ Object

Specifies the number of rows to skip before returning rows.

User.offset(10) # generated SQL has "OFFSET 10"

Should be used with order.

User.offset(10).order("name ASC")


1256
1257
1258
# File 'lib/active_record/relation/query_methods.rb', line 1256

def offset(value)
  spawn.offset!(value)
end

#offset!(value) ⇒ Object

:nodoc:



1260
1261
1262
1263
# File 'lib/active_record/relation/query_methods.rb', line 1260

def offset!(value) # :nodoc:
  self.offset_value = value
  self
end

#optimizer_hints(*args) ⇒ Object

Specify optimizer hints to be used in the SELECT statement.

Example (for MySQL):

Topic.optimizer_hints("MAX_EXECUTION_TIME(50000)", "NO_INDEX_MERGE(topics)")
# SELECT /*+ MAX_EXECUTION_TIME(50000) NO_INDEX_MERGE(topics) */ `topics`.* FROM `topics`

Example (for PostgreSQL with pg_hint_plan):

Topic.optimizer_hints("SeqScan(topics)", "Parallel(topics 8)")
# SELECT /*+ SeqScan(topics) Parallel(topics 8) */ "topics".* FROM "topics"


1514
1515
1516
1517
# File 'lib/active_record/relation/query_methods.rb', line 1514

def optimizer_hints(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.optimizer_hints!(*args)
end

#optimizer_hints!(*args) ⇒ Object

:nodoc:



1519
1520
1521
1522
# File 'lib/active_record/relation/query_methods.rb', line 1519

def optimizer_hints!(*args) # :nodoc:
  self.optimizer_hints_values |= args
  self
end

#or(other) ⇒ Object

Returns a new relation, which is the logical union of this relation and the one passed as an argument.

The two relations must be structurally compatible: they must be scoping the same model, and they must differ only by #where (if no #group has been defined) or #having (if a #group is present).

Post.where("id = 1").or(Post.where("author_id = 3"))
# SELECT `posts`.* FROM `posts` WHERE ((id = 1) OR (author_id = 3))


1195
1196
1197
1198
1199
1200
1201
1202
1203
1204
1205
# File 'lib/active_record/relation/query_methods.rb', line 1195

def or(other)
  if other.is_a?(Relation)
    if @none
      other.spawn
    else
      spawn.or!(other)
    end
  else
    raise ArgumentError, "You have passed #{other.class.name} object to #or. Pass an ActiveRecord::Relation object instead."
  end
end

#or!(other) ⇒ Object

:nodoc:



1207
1208
1209
1210
1211
1212
1213
1214
1215
1216
1217
1218
1219
# File 'lib/active_record/relation/query_methods.rb', line 1207

def or!(other) # :nodoc:
  incompatible_values = structurally_incompatible_values_for(other)

  unless incompatible_values.empty?
    raise ArgumentError, "Relation passed to #or must be structurally compatible. Incompatible values: #{incompatible_values}"
  end

  self.where_clause = where_clause.or(other.where_clause)
  self.having_clause = having_clause.or(other.having_clause)
  self.references_values |= other.references_values

  self
end

#order(*args) ⇒ Object

Applies an ORDER BY clause to a query.

#order accepts arguments in one of several formats.

symbols

The symbol represents the name of the column you want to order the results by.

User.order(:name)
# SELECT "users".* FROM "users" ORDER BY "users"."name" ASC

By default, the order is ascending. If you want descending order, you can map the column name symbol to :desc.

User.order(email: :desc)
# SELECT "users".* FROM "users" ORDER BY "users"."email" DESC

Multiple columns can be passed this way, and they will be applied in the order specified.

User.order(:name, email: :desc)
# SELECT "users".* FROM "users" ORDER BY "users"."name" ASC, "users"."email" DESC

strings

Strings are passed directly to the database, allowing you to specify simple SQL expressions.

This could be a source of SQL injection, so only strings composed of plain column names and simple function(column_name) expressions with optional +ASC+/+DESC+ modifiers are allowed.

User.order('name')
# SELECT "users".* FROM "users" ORDER BY name

User.order('name DESC')
# SELECT "users".* FROM "users" ORDER BY name DESC

User.order('name DESC, email')
# SELECT "users".* FROM "users" ORDER BY name DESC, email

Arel

If you need to pass in complicated expressions that you have verified are safe for the database, you can use Arel.

User.order(Arel.sql('end_date - start_date'))
# SELECT "users".* FROM "users" ORDER BY end_date - start_date

Custom query syntax, like JSON columns for PostgreSQL, is supported in this way.

User.order(Arel.sql("payload->>'kind'"))
# SELECT "users".* FROM "users" ORDER BY payload->>'kind'


656
657
658
659
660
661
# File 'lib/active_record/relation/query_methods.rb', line 656

def order(*args)
  check_if_method_has_arguments!(__callee__, args) do
    sanitize_order_arguments(args)
  end
  spawn.order!(*args)
end

#order!(*args) ⇒ Object

Same as #order but operates on relation in-place instead of copying.



664
665
666
667
668
# File 'lib/active_record/relation/query_methods.rb', line 664

def order!(*args) # :nodoc:
  preprocess_order_args(args) unless args.empty?
  self.order_values |= args
  self
end

#preload(*args) ⇒ Object

Specify associations args to be eager loaded using separate queries. A separate query is performed for each association.

users = User.preload(:address).limit(5)
users.each do |user|
user.address.city
end

# SELECT "users".* FROM "users" LIMIT 5
# SELECT "addresses".* FROM "addresses" WHERE "addresses"."id" IN (1,2,3,4,5)

Instead of loading the 5 addresses with 5 separate queries, all addresses are loaded with a separate query.

Loading multiple and nested associations is possible using Hashes and Arrays, similar to #includes:

User.preload(:address, friends: [:address, :followers])
# SELECT "users".* FROM "users"
# SELECT "addresses".* FROM "addresses" WHERE "addresses"."id" IN (1,2,3,4,5)
# SELECT "friends".* FROM "friends" WHERE "friends"."user_id" IN (1,2,3,4,5)
# SELECT ...


322
323
324
325
# File 'lib/active_record/relation/query_methods.rb', line 322

def preload(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.preload!(*args)
end

#preload!(*args) ⇒ Object

:nodoc:



327
328
329
330
# File 'lib/active_record/relation/query_methods.rb', line 327

def preload!(*args) # :nodoc:
  self.preload_values |= args
  self
end

#readonly(value = true) ⇒ Object

Mark a relation as readonly. Attempting to update a record will result in an error.

users = User.readonly
users.first.save
# => ActiveRecord::ReadOnlyRecord: User is marked as readonly

To make a readonly relation writable, pass false.

users.readonly(false)
users.first.save
# => true


1338
1339
1340
# File 'lib/active_record/relation/query_methods.rb', line 1338

def readonly(value = true)
  spawn.readonly!(value)
end

#readonly!(value = true) ⇒ Object

:nodoc:



1342
1343
1344
1345
# File 'lib/active_record/relation/query_methods.rb', line 1342

def readonly!(value = true) # :nodoc:
  self.readonly_value = value
  self
end

#references(*table_names) ⇒ Object

Use to indicate that the given table_names are referenced by an SQL string, and should therefore be +JOIN+ed in any query rather than loaded separately. This method only works in conjunction with #includes. See #includes for more details.

User.includes(:posts).where("posts.name = 'foo'")
# Doesn't JOIN the posts table, resulting in an error.

User.includes(:posts).where("posts.name = 'foo'").references(:posts)
# Query now knows the string references posts, so adds a JOIN


355
356
357
358
# File 'lib/active_record/relation/query_methods.rb', line 355

def references(*table_names)
  check_if_method_has_arguments!(__callee__, table_names)
  spawn.references!(*table_names)
end

#references!(*table_names) ⇒ Object

:nodoc:



360
361
362
363
# File 'lib/active_record/relation/query_methods.rb', line 360

def references!(*table_names) # :nodoc:
  self.references_values |= table_names
  self
end

#regroup(*args) ⇒ Object

Allows you to change a previously set group statement.

Post.group(:title, :body)
# SELECT `posts`.`*` FROM `posts` GROUP BY `posts`.`title`, `posts`.`body`

Post.group(:title, :body).regroup(:title)
# SELECT `posts`.`*` FROM `posts` GROUP BY `posts`.`title`

This is short-hand for unscope(:group).group(fields). Note that we're unscoping the entire group statement.



593
594
595
596
# File 'lib/active_record/relation/query_methods.rb', line 593

def regroup(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.regroup!(*args)
end

#regroup!(*args) ⇒ Object

Same as #regroup but operates on relation in-place instead of copying.



599
600
601
602
# File 'lib/active_record/relation/query_methods.rb', line 599

def regroup!(*args) # :nodoc:
  self.group_values = args
  self
end

#reorder(*args) ⇒ Object

Replaces any existing order defined on the relation with the specified order.

User.order('email DESC').reorder('id ASC') # generated SQL has 'ORDER BY id ASC'

Subsequent calls to order on the same relation will be appended. For example:

User.order('email DESC').reorder('id ASC').order('name ASC')

generates a query with ORDER BY id ASC, name ASC.



760
761
762
763
764
765
# File 'lib/active_record/relation/query_methods.rb', line 760

def reorder(*args)
  check_if_method_has_arguments!(__callee__, args) do
    sanitize_order_arguments(args)
  end
  spawn.reorder!(*args)
end

#reorder!(*args) ⇒ Object

Same as #reorder but operates on relation in-place instead of copying.



768
769
770
771
772
773
774
# File 'lib/active_record/relation/query_methods.rb', line 768

def reorder!(*args) # :nodoc:
  preprocess_order_args(args)
  args.uniq!
  self.reordering_value = true
  self.order_values = args
  self
end

#reselect(*args) ⇒ Object

Allows you to change a previously set select statement.

Post.select(:title, :body)
# SELECT `posts`.`title`, `posts`.`body` FROM `posts`

Post.select(:title, :body).reselect(:created_at)
# SELECT `posts`.`created_at` FROM `posts`

This is short-hand for unscope(:select).select(fields). Note that we're unscoping the entire select statement.



541
542
543
544
545
# File 'lib/active_record/relation/query_methods.rb', line 541

def reselect(*args)
  check_if_method_has_arguments!(__callee__, args)
  args = process_select_args(args)
  spawn.reselect!(*args)
end

#reselect!(*args) ⇒ Object

Same as #reselect but operates on relation in-place instead of copying.



548
549
550
551
# File 'lib/active_record/relation/query_methods.rb', line 548

def reselect!(*args) # :nodoc:
  self.select_values = args
  self
end

#reverse_order ⇒ Object

Reverse the existing order clause on the relation.

User.order('name ASC').reverse_order # generated SQL has 'ORDER BY name DESC'


1527
1528
1529
# File 'lib/active_record/relation/query_methods.rb', line 1527

def reverse_order
  spawn.reverse_order!
end

#reverse_order! ⇒ Object

:nodoc:



1531
1532
1533
1534
1535
# File 'lib/active_record/relation/query_methods.rb', line 1531

def reverse_order! # :nodoc:
  orders = order_values.compact_blank
  self.order_values = reverse_sql_order(orders)
  self
end

#rewhere(conditions) ⇒ Object

Allows you to change a previously set where condition for a given attribute, instead of appending to that condition.

Post.where(trashed: true).where(trashed: false)
# WHERE `trashed` = true AND `trashed` = false

Post.where(trashed: true).rewhere(trashed: false)
# WHERE `trashed` = false

Post.where(active: true).where(trashed: true).rewhere(trashed: false)
# WHERE `active` = true AND `trashed` = false

This is short-hand for unscope(where: conditions.keys).where(conditions). Note that unlike reorder, we're only unscoping the named conditions -- not the entire where statement.



1089
1090
1091
1092
1093
1094
1095
1096
1097
1098
# File 'lib/active_record/relation/query_methods.rb', line 1089

def rewhere(conditions)
  return unscope(:where) if conditions.nil?

  scope = spawn
  where_clause = scope.build_where_clause(conditions)

  scope.unscope!(where: where_clause.extract_attributes)
  scope.where_clause += where_clause
  scope
end

#select(*fields) ⇒ Object

Works in two unique ways.

First: takes a block so it can be used just like Array#select.

Model.all.select { |m| m.field == value }

This will build an array of objects from the database for the scope, converting them into an array and iterating through them using Array#select.

Second: Modifies the SELECT statement for the query so that only certain fields are retrieved:

Model.select(:field)
# => [#<Model id: nil, field: "value">]

Although in the above example it looks as though this method returns an array, it actually returns a relation object and can have other query methods appended to it, such as the other methods in ActiveRecord::QueryMethods.

The argument to the method can also be an array of fields.

Model.select(:field, :other_field, :and_one_more)
# => [#<Model id: nil, field: "value", other_field: "value", and_one_more: "value">]

The argument also can be a hash of fields and aliases.

Model.select(models: { field: :alias, other_field: :other_alias })
# => [#<Model id: nil, alias: "value", other_alias: "value">]

Model.select(models: [:field, :other_field])
# => [#<Model id: nil, field: "value", other_field: "value">]

You can also use one or more strings, which will be used unchanged as SELECT fields.

Model.select('field AS field_one', 'other_field AS field_two')
# => [#<Model id: nil, field_one: "value", field_two: "value">]

If an alias was specified, it will be accessible from the resulting objects:

Model.select('field AS field_one').first.field_one
# => "value"

Accessing attributes of an object that do not have fields retrieved by a select except id will throw ActiveModel::MissingAttributeError:

Model.select(:field).first.other_field
# => ActiveModel::MissingAttributeError: missing attribute 'other_field' for Model


413
414
415
416
417
418
419
420
421
422
423
424
425
426
# File 'lib/active_record/relation/query_methods.rb', line 413

def select(*fields)
  if block_given?
    if fields.any?
      raise ArgumentError, "`select' with block doesn't take arguments."
    end

    return super()
  end

  check_if_method_has_arguments!(__callee__, fields, "Call `select' with at least one field.")

  fields = process_select_args(fields)
  spawn._select!(*fields)
end

#skip_preloading! ⇒ Object

:nodoc:



1542
1543
1544
1545
# File 'lib/active_record/relation/query_methods.rb', line 1542

def skip_preloading! # :nodoc:
  self.skip_preloading_value = true
  self
end

#skip_query_cache!(value = true) ⇒ Object

:nodoc:



1537
1538
1539
1540
# File 'lib/active_record/relation/query_methods.rb', line 1537

def skip_query_cache!(value = true) # :nodoc:
  self.skip_query_cache_value = value
  self
end

#strict_loading(value = true) ⇒ Object

Sets the returned relation to strict_loading mode. This will raise an error if the record tries to lazily load an association.

user = User.strict_loading.first
user.comments.to_a
# => ActiveRecord::StrictLoadingViolationError


1353
1354
1355
# File 'lib/active_record/relation/query_methods.rb', line 1353

def strict_loading(value = true)
  spawn.strict_loading!(value)
end

#strict_loading!(value = true) ⇒ Object

:nodoc:



1357
1358
1359
1360
# File 'lib/active_record/relation/query_methods.rb', line 1357

def strict_loading!(value = true) # :nodoc:
  self.strict_loading_value = value
  self
end

#structurally_compatible?(other) ⇒ Boolean

Checks whether the given relation is structurally compatible with this relation, to determine if it's possible to use the #and and #or methods without raising an error. Structurally compatible is defined as: they must be scoping the same model, and they must differ only by #where (if no #group has been defined) or #having (if a #group is present).

Post.where("id = 1").structurally_compatible?(Post.where("author_id = 3"))
# => true

Post.joins(:comments).structurally_compatible?(Post.where("id = 1"))
# => false

Returns:

  • (Boolean)


1149
1150
1151
# File 'lib/active_record/relation/query_methods.rb', line 1149

def structurally_compatible?(other)
  structurally_incompatible_values_for(other).empty?
end

#table_name_qualified_unscope_values ⇒ Object



848
849
850
851
852
853
854
855
856
857
858
859
860
861
862
863
864
865
866
# File 'lib/active_record/relation/query_methods.rb', line 848

def table_name_qualified_unscope_values
  self.unscope_values.map do |scope|
    case scope
    when Hash
      scope.transform_values do |target_value|
        case target_value
        when Array
          target_value.map { |value| qualify_attribute_with_table_name(value) }
        when Symbol, String
          qualify_attribute_with_table_name(target_value)
        else
          target_value
        end
      end
    else
      scope
    end
  end
end

#uniq!(name) ⇒ Object

Deduplicate multiple values.



1570
1571
1572
1573
1574
1575
# File 'lib/active_record/relation/query_methods.rb', line 1570

def uniq!(name)
  if values = @values[name]
    values.uniq! if values.is_a?(Array) && !values.empty?
  end
  self
end

#unscope(*args) ⇒ Object

Removes an unwanted relation that is already defined on a chain of relations. This is useful when passing around chains of relations and would like to modify the relations without reconstructing the entire chain.

User.order('email DESC').unscope(:order) == User.all

The method arguments are symbols which correspond to the names of the methods which should be unscoped. The valid arguments are given in VALID_UNSCOPING_VALUES. The method can also be called with multiple arguments. For example:

User.order('email DESC').select('id').where(name: "John")
  .unscope(:order, :select, :where) == User.all

One can additionally pass a hash as an argument to unscope specific :where values. This is done by passing a hash with a single key-value pair. The key should be :where and the value should be the where value to unscope. For example:

User.where(name: "John", active: true).unscope(where: :name)
  == User.where(active: true)

This method is similar to #except, but unlike #except, it persists across merges:

User.order('email').merge(User.except(:order))
  == User.order('email')

User.order('email').merge(User.unscope(:order))
  == User.all

This means it can be used in association definitions:

has_many :comments, -> { unscope(where: :trashed) }


814
815
816
817
# File 'lib/active_record/relation/query_methods.rb', line 814

def unscope(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.unscope!(*args)
end

#unscope!(*args) ⇒ Object

:nodoc:



819
820
821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
846
# File 'lib/active_record/relation/query_methods.rb', line 819

def unscope!(*args) # :nodoc:
  self.unscope_values |= args

  args.each do |scope|
    case scope
    when Symbol
      scope = :left_outer_joins if scope == :left_joins
      if !VALID_UNSCOPING_VALUES.include?(scope)
        raise ArgumentError, "Called unscope() with invalid unscoping argument ':#{scope}'. Valid arguments are :#{VALID_UNSCOPING_VALUES.to_a.join(", :")}."
      end
      assert_modifiable!
      @values.delete(scope)
    when Hash
      scope.each do |key, target_value|
        if key != :where
          raise ArgumentError, "Hash arguments in .unscope(*args) must have :where as the key."
        end

        target_values = resolve_arel_attributes(Array.wrap(target_value))
        self.where_clause = where_clause.except(*target_values)
      end
    else
      raise ArgumentError, "Unrecognized scoping: #{args.inspect}. Use .unscope(where: :attribute_name) or .unscope(:order), for example."
    end
  end

  self
end

#where(*args) ⇒ Object

Returns a new relation, which is the result of filtering the current relation according to the conditions in the arguments.

#where accepts conditions in one of several formats. In the examples below, the resulting SQL is given as an illustration; the actual query generated may be different depending on the database adapter.

String

A single string, without additional arguments, is passed to the query constructor as an SQL fragment, and used in the where clause of the query.

Client.where("orders_count = '2'")
# SELECT * from clients where orders_count = '2';

Note that building your own string from user input may expose your application to injection attacks if not done properly. As an alternative, it is recommended to use one of the following methods.

Array

If an array is passed, then the first element of the array is treated as a template, and the remaining elements are inserted into the template to generate the condition. Active Record takes care of building the query to avoid injection attacks, and will convert from the ruby type to the database type where needed. Elements are inserted into the string in the order in which they appear.

User.where(["name = ? and email = ?", "Joe", "[email protected]"])
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]';

Alternatively, you can use named placeholders in the template, and pass a hash as the second element of the array. The names in the template are replaced with the corresponding values from the hash.

User.where(["name = :name and email = :email", { name: "Joe", email: "[email protected]" }])
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]';

This can make for more readable code in complex queries.

Lastly, you can use sprintf-style % escapes in the template. This works slightly differently than the previous methods; you are responsible for ensuring that the values in the template are properly quoted. The values are passed to the connector for quoting, but the caller is responsible for ensuring they are enclosed in quotes in the resulting SQL. After quoting, the values are inserted using the same escapes as the Ruby core method Kernel::sprintf.

User.where(["name = '%s' and email = '%s'", "Joe", "[email protected]"])
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]';

If #where is called with multiple arguments, these are treated as if they were passed as the elements of a single array.

User.where("name = :name and email = :email", { name: "Joe", email: "[email protected]" })
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]';

When using strings to specify conditions, you can use any operator available from the database. While this provides the most flexibility, you can also unintentionally introduce dependencies on the underlying database. If your code is intended for general consumption, test with multiple database backends.

Hash

#where will also accept a hash condition, in which the keys are fields and the values are values to be searched for.

Fields can be symbols or strings. Values can be single values, arrays, or ranges.

User.where(name: "Joe", email: "[email protected]")
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]'

User.where(name: ["Alice", "Bob"])
# SELECT * FROM users WHERE name IN ('Alice', 'Bob')

User.where(created_at: (Time.now.midnight - 1.day)..Time.now.midnight)
# SELECT * FROM users WHERE (created_at BETWEEN '2012-06-09 07:00:00.000000' AND '2012-06-10 07:00:00.000000')

In the case of a belongs_to relationship, an association key can be used to specify the model if an ActiveRecord object is used as the value.

author = Author.find(1)

# The following queries will be equivalent:
Post.where(author: author)
Post.where(author_id: author)

This also works with polymorphic belongs_to relationships:

treasure = Treasure.create(name: 'gold coins')
treasure.price_estimates << PriceEstimate.create(price: 125)

# The following queries will be equivalent:
PriceEstimate.where(estimate_of: treasure)
PriceEstimate.where(estimate_of_type: 'Treasure', estimate_of_id: treasure)

Hash conditions may also be specified in a tuple-like syntax. Hash keys may be an array of columns with an array of tuples as values.

Article.where([:author_id, :id] => [[15, 1], [15, 2]])
# SELECT * FROM articles WHERE author_id = 15 AND id = 1 OR author_id = 15 AND id = 2

Joins

If the relation is the result of a join, you may create a condition which uses any of the tables in the join. For string and array conditions, use the table name in the condition.

User.joins(:posts).where("posts.created_at < ?", Time.now)

For hash conditions, you can either use the table name in the key, or use a sub-hash.

User.joins(:posts).where("posts.published" => true)
User.joins(:posts).where(posts: { published: true })

No Argument

If no argument is passed, #where returns a new instance of WhereChain, that can be chained with WhereChain#not, WhereChain#missing, or WhereChain#associated.

Chaining with WhereChain#not:

User.where.not(name: "Jon")
# SELECT * FROM users WHERE name != 'Jon'

Chaining with WhereChain#associated:

Post.where.associated(:author)
# SELECT "posts".* FROM "posts"
# INNER JOIN "authors" ON "authors"."id" = "posts"."author_id"
# WHERE "authors"."id" IS NOT NULL

Chaining with WhereChain#missing:

Post.where.missing(:author)
# SELECT "posts".* FROM "posts"
# LEFT OUTER JOIN "authors" ON "authors"."id" = "posts"."author_id"
# WHERE "authors"."id" IS NULL

Blank Condition

If the condition is any blank-ish object, then #where is a no-op and returns the current relation.



1061
1062
1063
1064
1065
1066
1067
1068
1069
# File 'lib/active_record/relation/query_methods.rb', line 1061

def where(*args)
  if args.empty?
    WhereChain.new(spawn)
  elsif args.length == 1 && args.first.blank?
    self
  else
    spawn.where!(*args)
  end
end

#where!(opts, *rest) ⇒ Object

:nodoc:



1071
1072
1073
1074
# File 'lib/active_record/relation/query_methods.rb', line 1071

def where!(opts, *rest) # :nodoc:
  self.where_clause += build_where_clause(opts, rest)
  self
end

#with(*args) ⇒ Object

Add a Common Table Expression (CTE) that you can then reference within another SELECT statement.

Note: CTE's are only supported in MySQL for versions 8.0 and above. You will not be able to use CTE's with MySQL 5.7.

Post.with(posts_with_tags: Post.where("tags_count > ?", 0))
# => ActiveRecord::Relation
# WITH posts_with_tags AS (
#   SELECT * FROM posts WHERE (tags_count > 0)
# )
# SELECT * FROM posts

You can also pass an array of sub-queries to be joined in a UNION ALL.

Post.with(posts_with_tags_or_comments: [Post.where("tags_count > ?", 0), Post.where("comments_count > ?", 0)])
# => ActiveRecord::Relation
# WITH posts_with_tags_or_comments AS (
#  (SELECT * FROM posts WHERE (tags_count > 0))
#  UNION ALL
#  (SELECT * FROM posts WHERE (comments_count > 0))
# )
# SELECT * FROM posts

Once you define Common Table Expression you can use custom FROM value or JOIN to reference it.

Post.with(posts_with_tags: Post.where("tags_count > ?", 0)).from("posts_with_tags AS posts")
# => ActiveRecord::Relation
# WITH posts_with_tags AS (
#  SELECT * FROM posts WHERE (tags_count > 0)
# )
# SELECT * FROM posts_with_tags AS posts

Post.with(posts_with_tags: Post.where("tags_count > ?", 0)).joins("JOIN posts_with_tags ON posts_with_tags.id = posts.id")
# => ActiveRecord::Relation
# WITH posts_with_tags AS (
#   SELECT * FROM posts WHERE (tags_count > 0)
# )
# SELECT * FROM posts JOIN posts_with_tags ON posts_with_tags.id = posts.id

It is recommended to pass a query as ActiveRecord::Relation. If that is not possible and you have verified it is safe for the database, you can pass it as SQL literal using Arel.

Post.with(popular_posts: Arel.sql("... complex sql to calculate posts popularity ..."))

Great caution should be taken to avoid SQL injection vulnerabilities. This method should not be used with unsafe values that include unsanitized input.

To add multiple CTEs just pass multiple key-value pairs

Post.with(
posts_with_comments: Post.where("comments_count > ?", 0),
posts_with_tags: Post.where("tags_count > ?", 0)
)

or chain multiple .with calls

Post
.with(posts_with_comments: Post.where("comments_count > ?", 0))
.with(posts_with_tags: Post.where("tags_count > ?", 0))

Raises:

  • (ArgumentError)


493
494
495
496
497
# File 'lib/active_record/relation/query_methods.rb', line 493

def with(*args)
  raise ArgumentError, "ActiveRecord::Relation#with does not accept a block" if block_given?
  check_if_method_has_arguments!(__callee__, args)
  spawn.with!(*args)
end

#with!(*args) ⇒ Object

Like #with, but modifies relation in place.



500
501
502
503
504
# File 'lib/active_record/relation/query_methods.rb', line 500

def with!(*args) # :nodoc:
  args = process_with_args(args)
  self.with_values |= args
  self
end

#with_recursive(*args) ⇒ Object

Add a recursive Common Table Expression (CTE) that you can then reference within another SELECT statement.

Post.with_recursive(post_and_replies: [Post.where(id: 42), Post.joins('JOIN post_and_replies ON posts.in_reply_to_id = post_and_replies.id')])
# => ActiveRecord::Relation
# WITH RECURSIVE post_and_replies AS (
#   (SELECT * FROM posts WHERE id = 42)
#   UNION ALL
#   (SELECT * FROM posts JOIN post_and_replies ON posts.in_reply_to_id = post_and_replies.id)
# )
# SELECT * FROM posts

See #with for more information.



518
519
520
521
# File 'lib/active_record/relation/query_methods.rb', line 518

def with_recursive(*args)
  check_if_method_has_arguments!(__callee__, args)
  spawn.with_recursive!(*args)
end

#with_recursive!(*args) ⇒ Object

Like #with_recursive but modifies the relation in place.



524
525
526
527
528
529
# File 'lib/active_record/relation/query_methods.rb', line 524

def with_recursive!(*args) # :nodoc:
  args = process_with_args(args)
  self.with_values |= args
  @with_is_recursive = true
  self
end