Method: ActiveRecord::QueryMethods#and

Defined in:
lib/active_record/relation/query_methods.rb

#and(other) ⇒ Object

Returns a new relation, which is the logical intersection of this relation and the one passed as an argument.

The two relations must be structurally compatible: they must be scoping the same model, and they must differ only by #where (if no #group has been defined) or #having (if a #group is present).

Post.where(id: [1, 2]).and(Post.where(id: [2, 3]))
# SELECT `posts`.* FROM `posts` WHERE `posts`.`id` IN (1, 2) AND `posts`.`id` IN (2, 3)


1135
1136
1137
1138
1139
1140
1141
# File 'lib/active_record/relation/query_methods.rb', line 1135

def and(other)
  if other.is_a?(Relation)
    spawn.and!(other)
  else
    raise ArgumentError, "You have passed #{other.class.name} object to #and. Pass an ActiveRecord::Relation object instead."
  end
end