Method: ActiveRecord::QueryMethods#reorder
- Defined in:
- lib/active_record/relation/query_methods.rb
#reorder(*args) ⇒ Object
Replaces any existing order defined on the relation with the specified order.
User.order('email DESC').reorder('id ASC') # generated SQL has 'ORDER BY id ASC'
Subsequent calls to order on the same relation will be appended. For example:
User.order('email DESC').reorder('id ASC').order('name ASC')
generates a query with ‘ORDER BY id ASC, name ASC’.
321 322 323 324 |
# File 'lib/active_record/relation/query_methods.rb', line 321 def reorder(*args) check_if_method_has_arguments!(:reorder, args) spawn.reorder!(*args) end |