Method: ActiveRecord::Reflection::AbstractReflection#scopes

Defined in:
lib/active_record/reflection.rb

#scopesObject

Returns a list of scopes that should be applied for this Reflection object when querying the database.



196
197
198
# File 'lib/active_record/reflection.rb', line 196

def scopes
  scope ? [scope] : []
end