Class: ActiveRecord::Relation

Inherits:
Object
  • Object
show all
Includes:
Batches, Calculations, Delegation, Explain, FinderMethods, QueryMethods, SignedId::RelationMethods, SpawnMethods, TokenFor::RelationMethods, Enumerable
Defined in:
lib/active_record/relation.rb,
lib/active_record/relation/merger.rb,
lib/active_record/relation/from_clause.rb,
lib/active_record/relation/where_clause.rb,
lib/active_record/relation/query_attribute.rb

Overview

Active Record Relation

Defined Under Namespace

Classes: ExplainProxy, FromClause, HashMerger, Merger, QueryAttribute, StrictLoadingScope, WhereClause

Constant Summary collapse

MULTI_VALUE_METHODS =
[:includes, :eager_load, :preload, :select, :group,
:order, :joins, :left_outer_joins, :references,
:extending, :unscope, :optimizer_hints, :annotate,
:with]
SINGLE_VALUE_METHODS =
[:limit, :offset, :lock, :readonly, :reordering, :strict_loading,
:reverse_order, :distinct, :create_with, :skip_query_cache]
CLAUSE_METHODS =
[:where, :having, :from]
INVALID_METHODS_FOR_UPDATE_AND_DELETE_ALL =
[:distinct, :with, :with_recursive]
VALUE_METHODS =
MULTI_VALUE_METHODS + SINGLE_VALUE_METHODS + CLAUSE_METHODS

Constants included from FinderMethods

FinderMethods::ONE_AS_ONE

Constants included from QueryMethods

QueryMethods::FROZEN_EMPTY_ARRAY, QueryMethods::FROZEN_EMPTY_HASH, QueryMethods::VALID_UNSCOPING_VALUES

Constants included from Batches

Batches::DEFAULT_ORDER, Batches::ORDER_IGNORE_MESSAGE

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from SignedId::RelationMethods

#find_signed, #find_signed!

Methods included from TokenFor::RelationMethods

#find_by_token_for, #find_by_token_for!

Methods included from FinderMethods

#exists?, #fifth, #fifth!, #find, #find_by, #find_by!, #find_sole_by, #first, #first!, #forty_two, #forty_two!, #fourth, #fourth!, #include?, #last, #last!, #raise_record_not_found_exception!, #second, #second!, #second_to_last, #second_to_last!, #sole, #take, #take!, #third, #third!, #third_to_last, #third_to_last!

Methods included from Calculations

#async_average, #async_count, #async_ids, #async_maximum, #async_minimum, #async_pick, #async_pluck, #async_sum, #average, #calculate, #count, #ids, #maximum, #minimum, #pick, #pluck, #sum

Methods included from SpawnMethods

#except, #merge, #merge!, #only, #spawn

Methods included from QueryMethods

#_select!, #all, #and, #and!, #annotate, #annotate!, #arel, #construct_join_dependency, #create_with, #create_with!, #distinct, #distinct!, #eager_load, #eager_load!, #excluding, #excluding!, #extending, #extending!, #extract_associated, #from, #from!, #group, #group!, #having, #having!, #in_order_of, #includes, #includes!, #invert_where, #invert_where!, #joins, #joins!, #left_outer_joins, #left_outer_joins!, #limit, #limit!, #lock, #lock!, #none, #none!, #null_relation?, #offset, #offset!, #optimizer_hints, #optimizer_hints!, #or, #or!, #order, #order!, #preload, #preload!, #readonly, #readonly!, #references, #references!, #regroup, #regroup!, #reorder, #reorder!, #reselect, #reselect!, #reverse_order, #reverse_order!, #rewhere, #select, #skip_preloading!, #skip_query_cache!, #strict_loading, #strict_loading!, #structurally_compatible?, #table_name_qualified_unscope_values, #uniq!, #unscope, #unscope!, #where, #where!, #with, #with!, #with_recursive, #with_recursive!

Methods included from Batches

#find_each, #find_in_batches, #in_batches

Methods included from Explain

#collecting_queries_for_explain, #exec_explain

Methods included from Delegation

delegated_classes, uncacheable_methods

Constructor Details

#initialize(model, table: nil, predicate_builder: nil, values: {}) ⇒ Relation

Returns a new instance of Relation.



77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
# File 'lib/active_record/relation.rb', line 77

def initialize(model, table: nil, predicate_builder: nil, values: {})
  if table
    predicate_builder ||= model.predicate_builder.with(TableMetadata.new(model, table))
  else
    table = model.arel_table
    predicate_builder ||= model.predicate_builder
  end

  @model  = model
  @table  = table
  @values = values
  @loaded = false
  @predicate_builder = predicate_builder
  @delegate_to_model = false
  @future_result = nil
  @records = nil
  @async = false
  @none = false
end

Instance Attribute Details

#loaded ⇒ Object (readonly) Also known as: loaded?

Returns the value of attribute loaded.



71
72
73
# File 'lib/active_record/relation.rb', line 71

def loaded
  @loaded
end

#model ⇒ Object (readonly) Also known as: klass

Returns the value of attribute model.



71
72
73
# File 'lib/active_record/relation.rb', line 71

def model
  @model
end

#predicate_builder ⇒ Object (readonly)

Returns the value of attribute predicate_builder.



71
72
73
# File 'lib/active_record/relation.rb', line 71

def predicate_builder
  @predicate_builder
end

#skip_preloading_value ⇒ Object

Returns the value of attribute skip_preloading_value.



72
73
74
# File 'lib/active_record/relation.rb', line 72

def skip_preloading_value
  @skip_preloading_value
end

#table ⇒ Object (readonly)

Returns the value of attribute table.



71
72
73
# File 'lib/active_record/relation.rb', line 71

def table
  @table
end

Instance Method Details

#==(other) ⇒ Object

Compares two relations for equality.



1281
1282
1283
1284
1285
1286
1287
1288
1289
1290
# File 'lib/active_record/relation.rb', line 1281

def ==(other)
  case other
  when Associations::CollectionProxy, AssociationRelation
    self == other.records
  when Relation
    other.to_sql == to_sql
  when Array
    records == other
  end
end

#_exec_scope ⇒ Object

:nodoc:



566
567
568
569
570
571
572
# File 'lib/active_record/relation.rb', line 566

def _exec_scope(...) # :nodoc:
  @delegate_to_model = true
  registry = model.scope_registry
  _scoping(nil, registry) { instance_exec(...) || self }
ensure
  @delegate_to_model = false
end

#alias_tracker(joins = [], aliases = nil) ⇒ Object

:nodoc:



1335
1336
1337
# File 'lib/active_record/relation.rb', line 1335

def alias_tracker(joins = [], aliases = nil) # :nodoc:
  ActiveRecord::Associations::AliasTracker.create(model.connection_pool, table.name, joins, aliases)
end

#any?(*args) ⇒ Boolean

Returns true if there are any records.

When an argument is given, returns true if at least one records matches the argument via the case-equality operator (===).

posts.any?(Post)    # => true if at least one record
posts.any?(Comment) # => false

Returns:

  • (Boolean)


404
405
406
407
408
409
# File 'lib/active_record/relation.rb', line 404

def any?(*args)
  return false if @none

  return super if args.present? || block_given?
  !empty?
end

#bind_attribute(name, value) {|attr, bind| ... } ⇒ Object

:nodoc:

Yields:

  • (attr, bind)


102
103
104
105
106
107
108
109
110
111
# File 'lib/active_record/relation.rb', line 102

def bind_attribute(name, value) # :nodoc:
  if reflection = model._reflect_on_association(name)
    name = reflection.foreign_key
    value = value.read_attribute(reflection.association_primary_key) unless value.nil?
  end

  attr = table[name]
  bind = predicate_builder.build_bind_attribute(attr.name, value)
  yield attr, bind
end

#blank? ⇒ Boolean

Returns true if relation is blank.

Returns:

  • (Boolean)


1302
1303
1304
# File 'lib/active_record/relation.rb', line 1302

def blank?
  records.blank?
end

#cache_key(timestamp_column = "updated_at") ⇒ Object

Returns a stable cache key that can be used to identify this query. The cache key is built with a fingerprint of the SQL query.

Product.where("name like ?", "%Cosmic Encounter%").cache_key
# => "products/query-1850ab3d302391b85b8693e941286659"

If ActiveRecord::Base.collection_cache_versioning is turned off, as it was in Rails 6.0 and earlier, the cache key will also include a version.

ActiveRecord::Base.collection_cache_versioning = false
Product.where("name like ?", "%Cosmic Encounter%").cache_key
# => "products/query-1850ab3d302391b85b8693e941286659-1-20150714212553907087000"

You can also pass a custom timestamp column to fetch the timestamp of the last updated record.

Product.where("name like ?", "%Game%").cache_key(:last_reviewed_at)


452
453
454
455
# File 'lib/active_record/relation.rb', line 452

def cache_key(timestamp_column = "updated_at")
  @cache_keys ||= {}
  @cache_keys[timestamp_column] ||= model.collection_cache_key(self, timestamp_column)
end

#cache_key_with_version ⇒ Object

Returns a cache key along with the version.



533
534
535
536
537
538
539
# File 'lib/active_record/relation.rb', line 533

def cache_key_with_version
  if version = cache_version
    "#{cache_key}-#{version}"
  else
    cache_key
  end
end

#cache_version(timestamp_column = :updated_at) ⇒ Object

Returns a cache version that can be used together with the cache key to form a recyclable caching scheme. The cache version is built with the number of records matching the query, and the timestamp of the last updated record. When a new record comes to match the query, or any of the existing records is updated or deleted, the cache version changes.

If the collection is loaded, the method will iterate through the records to generate the timestamp, otherwise it will trigger one SQL query like:

SELECT COUNT(*), MAX("products"."updated_at") FROM "products" WHERE (name like '%Cosmic Encounter%')


479
480
481
482
483
484
# File 'lib/active_record/relation.rb', line 479

def cache_version(timestamp_column = :updated_at)
  if model.collection_cache_versioning
    @cache_versions ||= {}
    @cache_versions[timestamp_column] ||= compute_cache_version(timestamp_column)
  end
end

#create(attributes = nil, &block) ⇒ Object

Tries to create a new record with the same scoped attributes defined in the relation. Returns the initialized object if validation fails.

Expects arguments in the same format as ActiveRecord::Base.create.

Examples

users = User.where(name: 'Oscar')
users.create # => #<User id: 3, name: "Oscar", ...>

users.create(name: 'fxn')
users.create # => #<User id: 4, name: "fxn", ...>

users.create { |user| user.name = 'tenderlove' }
# => #<User id: 5, name: "tenderlove", ...>

users.create(name: nil) # validation on name
# => #<User id: nil, name: nil, ...>


154
155
156
157
158
159
160
161
# File 'lib/active_record/relation.rb', line 154

def create(attributes = nil, &block)
  if attributes.is_a?(Array)
    attributes.collect { |attr| create(attr, &block) }
  else
    block = current_scope_restoring_block(&block)
    scoping { _create(attributes, &block) }
  end
end

#create!(attributes = nil, &block) ⇒ Object

Similar to #create, but calls create! on the base class. Raises an exception if a validation error occurs.

Expects arguments in the same format as ActiveRecord::Base.create!.



169
170
171
172
173
174
175
176
# File 'lib/active_record/relation.rb', line 169

def create!(attributes = nil, &block)
  if attributes.is_a?(Array)
    attributes.collect { |attr| create!(attr, &block) }
  else
    block = current_scope_restoring_block(&block)
    scoping { _create!(attributes, &block) }
  end
end

#create_or_find_by(attributes, &block) ⇒ Object

Attempts to create a record with the given attributes in a table that has a unique database constraint on one or several of its columns. If a row already exists with one or several of these unique constraints, the exception such an insertion would normally raise is caught, and the existing record with those attributes is found using #find_by!.

This is similar to #find_or_create_by, but tries to create the record first. As such it is better suited for cases where the record is most likely not to exist yet.

There are several drawbacks to #create_or_find_by, though:

  • The underlying table must have the relevant columns defined with unique database constraints.
  • A unique constraint violation may be triggered by only one, or at least less than all, of the given attributes. This means that the subsequent #find_by! may fail to find a matching record, which will then raise an ActiveRecord::RecordNotFound exception, rather than a record with the given attributes.
  • While we avoid the race condition between SELECT -> INSERT from #find_or_create_by, we actually have another race condition between INSERT -> SELECT, which can be triggered if a DELETE between those two statements is run by another client. But for most applications, that's a significantly less likely condition to hit.
  • It relies on exception handling to handle control flow, which may be marginally slower.
  • The primary key may auto-increment on each create, even if it fails. This can accelerate the problem of running out of integers, if the underlying table is still stuck on a primary key of type int (note: All Rails apps since 5.1+ have defaulted to bigint, which is not liable to this problem).
  • Columns with unique database constraints should not have uniqueness validations defined, otherwise #create will fail due to validation errors and #find_by will never be called.

This method will return a record if all given attributes are covered by unique constraints (unless the INSERT -> DELETE -> SELECT race condition is triggered), but if creation was attempted and failed due to validation errors it won't be persisted, you get what #create returns in such situation.



273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
# File 'lib/active_record/relation.rb', line 273

def create_or_find_by(attributes, &block)
  with_connection do |connection|
    record = nil
    transaction(requires_new: true) do
      record = create(attributes, &block)
      record._last_transaction_return_status || raise(ActiveRecord::Rollback)
    end
    record
  rescue ActiveRecord::RecordNotUnique
    if connection.transaction_open?
      rewhere(attributes).lock(connection.create_or_find_by_lock).take!
    else
      find_by!(attributes)
    end
  end
end

#create_or_find_by!(attributes, &block) ⇒ Object

Like #create_or_find_by, but calls create! so an exception is raised if the created record is invalid.



293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
# File 'lib/active_record/relation.rb', line 293

def create_or_find_by!(attributes, &block)
  with_connection do |connection|
    record = nil
    transaction(requires_new: true) do
      record = create!(attributes, &block)
      record._last_transaction_return_status || raise(ActiveRecord::Rollback)
    end
    record
  rescue ActiveRecord::RecordNotUnique
    if connection.transaction_open?
      rewhere(attributes).lock(connection.create_or_find_by_lock).take!
    else
      find_by!(attributes)
    end
  end
end

#delete(id_or_array) ⇒ Object

Deletes the row with a primary key matching the id argument, using an SQL DELETE statement, and returns the number of rows deleted. Active Record objects are not instantiated, so the object's callbacks are not executed, including any :dependent association options.

You can delete multiple rows at once by passing an Array of ids.

Note: Although it is often much faster than the alternative, #destroy, skipping callbacks might bypass business logic in your application that ensures referential integrity or performs other essential jobs.

Examples

# Delete a single row
Todo.delete(1)

# Delete multiple rows
Todo.delete([2,3,4])


1081
1082
1083
1084
1085
1086
1087
1088
1089
# File 'lib/active_record/relation.rb', line 1081

def delete(id_or_array)
  return 0 if id_or_array.nil? || (id_or_array.is_a?(Array) && id_or_array.empty?)

  if model.composite_primary_key? && !id_or_array.first.is_a?(Array)
    id_or_array = [id_or_array]
  end

  where(model.primary_key => id_or_array).delete_all
end

#delete_all ⇒ Object

Deletes the records without instantiating the records first, and hence not calling the #destroy method nor invoking callbacks. This is a single SQL DELETE statement that goes straight to the database, much more efficient than #destroy_all. Be careful with relations though, in particular :dependent rules defined on associations are not honored. Returns the number of rows affected.

Post.where(person_id: 5).where(category: ['Something', 'Else']).delete_all

This call deletes the affected posts all at once with a single DELETE statement. If you need to destroy dependent associations or call your before_* or after_destroy callbacks, use the #destroy_all method instead.

If an invalid method is supplied, #delete_all raises an ActiveRecordError:

Post.distinct.delete_all
# => ActiveRecord::ActiveRecordError: delete_all doesn't support distinct


1037
1038
1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
1049
1050
1051
1052
1053
1054
1055
1056
1057
1058
1059
1060
1061
# File 'lib/active_record/relation.rb', line 1037

def delete_all
  return 0 if @none

  invalid_methods = INVALID_METHODS_FOR_UPDATE_AND_DELETE_ALL.select do |method|
    value = @values[method]
    method == :distinct ? value : value&.any?
  end
  if invalid_methods.any?
    raise ActiveRecordError.new("delete_all doesn't support #{invalid_methods.join(', ')}")
  end

  model.with_connection do |c|
    arel = eager_loading? ? apply_join_dependency.arel : arel()
    arel.source.left = table

    key = if model.composite_primary_key?
      primary_key.map { |pk| table[pk] }
    else
      table[primary_key]
    end
    stmt = arel.compile_delete(key)

    c.delete(stmt, "#{model} Delete All").tap { reset }
  end
end

#delete_by(*args) ⇒ Object

Finds and deletes all records matching the specified conditions. This is short-hand for relation.where(condition).delete_all. Returns the number of rows affected.

If no record is found, returns 0 as zero rows were affected.

Person.delete_by(id: 13)
Person.delete_by(name: 'Spartacus', rating: 4)
Person.delete_by("published_at < ?", 2.weeks.ago)


1147
1148
1149
# File 'lib/active_record/relation.rb', line 1147

def delete_by(*args)
  where(*args).delete_all
end

#destroy(id) ⇒ Object

Destroy an object (or multiple objects) that has the given id. The object is instantiated first, therefore all callbacks and filters are fired off before the object is deleted. This method is less efficient than #delete but allows cleanup methods and other actions to be run.

This essentially finds the object (or multiple objects) with the given id, creates a new object from the attributes, and then calls destroy on it.

Parameters

  • id - This should be the id or an array of ids to be destroyed.

Examples

# Destroy a single object
Todo.destroy(1)

# Destroy multiple objects
todos = [1,2,3]
Todo.destroy(todos)


1111
1112
1113
1114
1115
1116
1117
1118
1119
1120
1121
1122
1123
# File 'lib/active_record/relation.rb', line 1111

def destroy(id)
  multiple_ids = if model.composite_primary_key?
    id.first.is_a?(Array)
  else
    id.is_a?(Array)
  end

  if multiple_ids
    find(id).each(&:destroy)
  else
    find(id).destroy
  end
end

#destroy_all ⇒ Object

Destroys the records by instantiating each record and calling its #destroy method. Each object's callbacks are executed (including :dependent association options). Returns the collection of objects that were destroyed; each will be frozen, to reflect that no changes should be made (since they can't be persisted).

Note: Instantiation, callback execution, and deletion of each record can be time consuming when you're removing many records at once. It generates at least one SQL DELETE query per record (or possibly more, to enforce your callbacks). If you want to delete many rows quickly, without concern for their associations or callbacks, use #delete_all instead.

Examples

Person.where(age: 0..18).destroy_all


1015
1016
1017
# File 'lib/active_record/relation.rb', line 1015

def destroy_all
  records.each(&:destroy).tap { reset }
end

#destroy_by(*args) ⇒ Object

Finds and destroys all records matching the specified conditions. This is short-hand for relation.where(condition).destroy_all. Returns the collection of objects that were destroyed.

If no record is found, returns empty array.

Person.destroy_by(id: 13)
Person.destroy_by(name: 'Spartacus', rating: 4)
Person.destroy_by("published_at < ?", 2.weeks.ago)


1134
1135
1136
# File 'lib/active_record/relation.rb', line 1134

def destroy_by(*args)
  where(*args).destroy_all
end

#eager_loading? ⇒ Boolean

Returns true if relation needs eager loading.

Returns:

  • (Boolean)


1266
1267
1268
1269
1270
# File 'lib/active_record/relation.rb', line 1266

def eager_loading?
  @should_eager_load ||=
    eager_load_values.any? ||
    includes_values.any? && (joined_includes_values.any? || references_eager_loaded_tables?)
end

#empty? ⇒ Boolean

Returns true if there are no records.

Returns:

  • (Boolean)


372
373
374
375
376
377
378
379
380
# File 'lib/active_record/relation.rb', line 372

def empty?
  return true if @none

  if loaded?
    records.empty?
  else
    !exists?
  end
end

#empty_scope? ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


1327
1328
1329
# File 'lib/active_record/relation.rb', line 1327

def empty_scope? # :nodoc:
  @values == model.unscoped.values
end

#encode_with(coder) ⇒ Object

Serializes the relation objects Array.



358
359
360
# File 'lib/active_record/relation.rb', line 358

def encode_with(coder)
  coder.represent_seq(nil, records)
end

#explain(*options) ⇒ Object

Runs EXPLAIN on the query or queries triggered by this relation and returns the result as a string. The string is formatted imitating the ones printed by the database shell.

User.all.explain
# EXPLAIN SELECT `users`.* FROM `users`
# ...

Note that this method actually runs the queries, since the results of some are needed by the next ones when eager loading is going on.

To run EXPLAIN on queries created by first, pluck and count, call these methods on explain:

User.all.explain.count
# EXPLAIN SELECT COUNT(*) FROM `users`
# ...

The column name can be passed if required:

User.all.explain.maximum(:id)
# EXPLAIN SELECT MAX(`users`.`id`) FROM `users`
# ...

Please see further details in the Active Record Query Interface guide.



342
343
344
# File 'lib/active_record/relation.rb', line 342

def explain(*options)
  ExplainProxy.new(self, options)
end

#find_or_create_by(attributes, &block) ⇒ Object

Finds the first record with the given attributes, or creates a record with the attributes if one is not found:

# Find the first user named "Penélope" or create a new one.
User.find_or_create_by(first_name: 'Penélope')
# => #<User id: 1, first_name: "Penélope", last_name: nil>

# Find the first user named "Penélope" or create a new one.
# We already have one so the existing record will be returned.
User.find_or_create_by(first_name: 'Penélope')
# => #<User id: 1, first_name: "Penélope", last_name: nil>

# Find the first user named "Scarlett" or create a new one with
# a particular last name.
User.create_with(last_name: 'Johansson').find_or_create_by(first_name: 'Scarlett')
# => #<User id: 2, first_name: "Scarlett", last_name: "Johansson">

This method accepts a block, which is passed down to #create. The last example above can be alternatively written this way:

# Find the first user named "Scarlett" or create a new one with a
# particular last name.
User.find_or_create_by(first_name: 'Scarlett') do |user|
user.last_name = 'Johansson'
end
# => #<User id: 2, first_name: "Scarlett", last_name: "Johansson">

This method always returns a record, but if creation was attempted and failed due to validation errors it won't be persisted, you get what #create returns in such situation.

If creation failed because of a unique constraint, this method will assume it encountered a race condition and will try finding the record once more. If somehow the second find still does not find a record because a concurrent DELETE happened, it will then raise an ActiveRecord::RecordNotFound exception.

Please note this method is not atomic, it runs first a SELECT, and if there are no results an INSERT is attempted. So if the table doesn't have a relevant unique constraint it could be the case that you end up with two or more similar records.



231
232
233
# File 'lib/active_record/relation.rb', line 231

def find_or_create_by(attributes, &block)
  find_by(attributes) || create_or_find_by(attributes, &block)
end

#find_or_create_by!(attributes, &block) ⇒ Object

Like #find_or_create_by, but calls create! so an exception is raised if the created record is invalid.



238
239
240
# File 'lib/active_record/relation.rb', line 238

def find_or_create_by!(attributes, &block)
  find_by(attributes) || create_or_find_by!(attributes, &block)
end

#find_or_initialize_by(attributes, &block) ⇒ Object

Like #find_or_create_by, but calls new instead of create.



312
313
314
# File 'lib/active_record/relation.rb', line 312

def find_or_initialize_by(attributes, &block)
  find_by(attributes) || new(attributes, &block)
end

#first_or_create(attributes = nil, &block) ⇒ Object

:nodoc:



178
179
180
# File 'lib/active_record/relation.rb', line 178

def first_or_create(attributes = nil, &block) # :nodoc:
  first || create(attributes, &block)
end

#first_or_create!(attributes = nil, &block) ⇒ Object

:nodoc:



182
183
184
# File 'lib/active_record/relation.rb', line 182

def first_or_create!(attributes = nil, &block) # :nodoc:
  first || create!(attributes, &block)
end

#first_or_initialize(attributes = nil, &block) ⇒ Object

:nodoc:



186
187
188
# File 'lib/active_record/relation.rb', line 186

def first_or_initialize(attributes = nil, &block) # :nodoc:
  first || new(attributes, &block)
end

#has_limit_or_offset? ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


1331
1332
1333
# File 'lib/active_record/relation.rb', line 1331

def has_limit_or_offset? # :nodoc:
  limit_value || offset_value
end

#initialize_copy(other) ⇒ Object



97
98
99
100
# File 'lib/active_record/relation.rb', line 97

def initialize_copy(other)
  @values = @values.dup
  reset
end

#insert(attributes, returning: nil, unique_by: nil, record_timestamps: nil) ⇒ Object

Inserts a single record into the database in a single SQL INSERT statement. It does not instantiate any models nor does it trigger Active Record callbacks or validations. Though passed values go through Active Record's type casting and serialization.

See #insert_all for documentation.



668
669
670
# File 'lib/active_record/relation.rb', line 668

def insert(attributes, returning: nil, unique_by: nil, record_timestamps: nil)
  insert_all([ attributes ], returning: returning, unique_by: unique_by, record_timestamps: record_timestamps)
end

#insert!(attributes, returning: nil, record_timestamps: nil) ⇒ Object

Inserts a single record into the database in a single SQL INSERT statement. It does not instantiate any models nor does it trigger Active Record callbacks or validations. Though passed values go through Active Record's type casting and serialization.

See #insert_all! for more.



757
758
759
# File 'lib/active_record/relation.rb', line 757

def insert!(attributes, returning: nil, record_timestamps: nil)
  insert_all!([ attributes ], returning: returning, record_timestamps: record_timestamps)
end

#insert_all(attributes, returning: nil, unique_by: nil, record_timestamps: nil) ⇒ Object

Inserts multiple records into the database in a single SQL INSERT statement. It does not instantiate any models nor does it trigger Active Record callbacks or validations. Though passed values go through Active Record's type casting and serialization.

The attributes parameter is an Array of Hashes. Every Hash determines the attributes for a single row and must have the same keys.

Rows are considered to be unique by every unique index on the table. Any duplicate rows are skipped. Override with :unique_by (see below).

Returns an ActiveRecord::Result with its contents based on :returning (see below).

Options

[:returning] (PostgreSQL, SQLite3, and MariaDB only) An array of attributes to return for all successfully inserted records, which by default is the primary key. Pass returning: %w[ id name ] for both id and name or returning: false to omit the underlying RETURNING SQL clause entirely.

You can also pass an SQL string if you need more control on the return values
(for example, <tt>returning: Arel.sql("id, name as new_name")</tt>).

[:unique_by] (PostgreSQL and SQLite only) By default rows are considered to be unique by every unique index on the table. Any duplicate rows are skipped.

To skip rows according to just one unique index pass <tt>:unique_by</tt>.

Consider a Book model where no duplicate ISBNs make sense, but if any
row has an existing id, or is not unique by another unique index,
ActiveRecord::RecordNotUnique is raised.

Unique indexes can be identified by columns or name:

unique_by: :isbn
unique_by: %i[ author_id name ]
unique_by: :index_books_on_isbn

[:record_timestamps] By default, automatic setting of timestamp columns is controlled by the model's record_timestamps config, matching typical behavior.

To override this and force automatic setting of timestamp columns one
way or the other, pass <tt>:record_timestamps</tt>:

record_timestamps: true  # Always set timestamps automatically
record_timestamps: false # Never set timestamps automatically

Because it relies on the index information from the database :unique_by is recommended to be paired with Active Record's schema_cache.

Example

# Insert records and skip inserting any duplicates.
# Here "Eloquent Ruby" is skipped because its id is not unique.

Book.insert_all([
{ id: 1, title: "Rework", author: "David" },
{ id: 1, title: "Eloquent Ruby", author: "Russ" }
])

# insert_all works on chained scopes, and you can use create_with
# to set default attributes for all inserted records.

author.books.create_with(created_at: Time.now).insert_all([
{ id: 1, title: "Rework" },
{ id: 2, title: "Eloquent Ruby" }
])


747
748
749
# File 'lib/active_record/relation.rb', line 747

def insert_all(attributes, returning: nil, unique_by: nil, record_timestamps: nil)
  InsertAll.execute(self, attributes, on_duplicate: :skip, returning: returning, unique_by: unique_by, record_timestamps: record_timestamps)
end

#insert_all!(attributes, returning: nil, record_timestamps: nil) ⇒ Object

Inserts multiple records into the database in a single SQL INSERT statement. It does not instantiate any models nor does it trigger Active Record callbacks or validations. Though passed values go through Active Record's type casting and serialization.

The attributes parameter is an Array of Hashes. Every Hash determines the attributes for a single row and must have the same keys.

Raises ActiveRecord::RecordNotUnique if any rows violate a unique index on the table. In that case, no rows are inserted.

To skip duplicate rows, see #insert_all. To replace them, see #upsert_all.

Returns an ActiveRecord::Result with its contents based on :returning (see below).

Options

[:returning] (PostgreSQL, SQLite3, and MariaDB only) An array of attributes to return for all successfully inserted records, which by default is the primary key. Pass returning: %w[ id name ] for both id and name or returning: false to omit the underlying RETURNING SQL clause entirely.

You can also pass an SQL string if you need more control on the return values
(for example, <tt>returning: Arel.sql("id, name as new_name")</tt>).

[:record_timestamps] By default, automatic setting of timestamp columns is controlled by the model's record_timestamps config, matching typical behavior.

To override this and force automatic setting of timestamp columns one
way or the other, pass <tt>:record_timestamps</tt>:

record_timestamps: true  # Always set timestamps automatically
record_timestamps: false # Never set timestamps automatically

Examples

# Insert multiple records
Book.insert_all!([
{ title: "Rework", author: "David" },
{ title: "Eloquent Ruby", author: "Russ" }
])

# Raises ActiveRecord::RecordNotUnique because "Eloquent Ruby"
# does not have a unique id.
Book.insert_all!([
{ id: 1, title: "Rework", author: "David" },
{ id: 1, title: "Eloquent Ruby", author: "Russ" }
])


814
815
816
# File 'lib/active_record/relation.rb', line 814

def insert_all!(attributes, returning: nil, record_timestamps: nil)
  InsertAll.execute(self, attributes, on_duplicate: :raise, returning: returning, record_timestamps: record_timestamps)
end

#inspect ⇒ Object



1318
1319
1320
1321
1322
1323
1324
1325
# File 'lib/active_record/relation.rb', line 1318

def inspect
  subject = loaded? ? records : annotate("loading for inspect")
  entries = subject.take([limit_value, 11].compact.min).map!(&:inspect)

  entries[10] = "..." if entries.size == 11

  "#<#{self.class.name} [#{entries.join(', ')}]>"
end

#joined_includes_values ⇒ Object

Joins that are also marked for preloading. In which case we should just eager load them. Note that this is a naive implementation because we could have strings and symbols which represent the same association, but that aren't matched by this. Also, we could have nested hashes which partially match, e.g. { a: :b } & { a: [:b, :c] }



1276
1277
1278
# File 'lib/active_record/relation.rb', line 1276

def joined_includes_values
  includes_values & joins_values
end

#load(&block) ⇒ Object

Causes the records to be loaded from the database if they have not been loaded already. You can use this if for some reason you need to explicitly load some records before actually using them. The return value is the relation itself, not the records.

Post.where(published: true).load # => #<ActiveRecord::Relation>


1207
1208
1209
1210
1211
1212
1213
1214
# File 'lib/active_record/relation.rb', line 1207

def load(&block)
  if !loaded? || scheduled?
    @records = exec_queries(&block)
    @loaded = true
  end

  self
end

#load_async ⇒ Object

Schedule the query to be performed from a background thread pool.

Post.where(published: true).load_async # => #<ActiveRecord::Relation>

When the Relation is iterated, if the background query wasn't executed yet, it will be performed by the foreground thread.

Note that config.active_record.async_query_executor must be configured for queries to actually be executed concurrently. Otherwise it defaults to executing them in the foreground.

If the query was actually executed in the background, the Active Record logs will show it by prefixing the log line with ASYNC:

ASYNC Post Load (0.0ms) (db time 2ms)  SELECT "posts".* FROM "posts" LIMIT 100


1166
1167
1168
1169
1170
1171
1172
1173
1174
1175
1176
1177
1178
1179
1180
1181
1182
1183
# File 'lib/active_record/relation.rb', line 1166

def load_async
  with_connection do |c|
    return load if !c.async_enabled?

    unless loaded?
      result = exec_main_query(async: !c.current_transaction.joinable?)

      if result.is_a?(Array)
        @records = result
      else
        @future_result = result
      end
      @loaded = true
    end
  end

  self
end

#many? ⇒ Boolean

Returns true if there is more than one record.

Returns:

  • (Boolean)


427
428
429
430
431
432
433
# File 'lib/active_record/relation.rb', line 427

def many?
  return false if @none

  return super if block_given?
  return records.many? if loaded?
  limited_count > 1
end

#new(attributes = nil, &block) ⇒ Object Also known as: build

Initializes new record from relation while maintaining the current scope.

Expects arguments in the same format as ActiveRecord::Base.new.

users = User.where(name: 'DHH')
user = users.new # => #<User id: nil, name: "DHH", created_at: nil, updated_at: nil>

You can also pass a block to new with the new record as argument:

user = users.new { |user| user.name = 'Oscar' }
user.name # => Oscar


125
126
127
128
129
130
131
132
# File 'lib/active_record/relation.rb', line 125

def new(attributes = nil, &block)
  if attributes.is_a?(Array)
    attributes.collect { |attr| new(attr, &block) }
  else
    block = current_scope_restoring_block(&block)
    scoping { _new(attributes, &block) }
  end
end

#none?(*args) ⇒ Boolean

Returns true if there are no records.

When an argument is given, returns true if no records match the argument via the case-equality operator (===).

posts.none?(Post)    # => true if posts is empty
posts.none?(Comment) # => true

Returns:

  • (Boolean)


389
390
391
392
393
394
# File 'lib/active_record/relation.rb', line 389

def none?(*args)
  return true if @none

  return super if args.present? || block_given?
  empty?
end

#one?(*args) ⇒ Boolean

Returns true if there is exactly one record.

When an argument is given, returns true if exactly one records matches the argument via the case-equality operator (===).

posts.one?(Post) # => true if exactly one record
posts.any?(Comment) # => false

Returns:

  • (Boolean)


418
419
420
421
422
423
424
# File 'lib/active_record/relation.rb', line 418

def one?(*args)
  return false if @none

  return super if args.present? || block_given?
  return records.one? if loaded?
  limited_count == 1
end

#preload_associations(records) ⇒ Object

:nodoc:



1349
1350
1351
1352
1353
1354
1355
1356
# File 'lib/active_record/relation.rb', line 1349

def preload_associations(records) # :nodoc:
  preload = preload_values
  preload += includes_values unless eager_loading?
  scope = strict_loading_value ? StrictLoadingScope : nil
  preload.each do |associations|
    ActiveRecord::Associations::Preloader.new(records: records, associations: associations, scope: scope).call
  end
end

#pretty_print(pp) ⇒ Object



1292
1293
1294
1295
1296
1297
1298
1299
# File 'lib/active_record/relation.rb', line 1292

def pretty_print(pp)
  subject = loaded? ? records : annotate("loading for pp")
  entries = subject.take([limit_value, 11].compact.min)

  entries[10] = "..." if entries.size == 11

  pp.pp(entries)
end

#readonly? ⇒ Boolean

Returns:

  • (Boolean)


1306
1307
1308
# File 'lib/active_record/relation.rb', line 1306

def readonly?
  readonly_value
end

#records ⇒ Object

:nodoc:



352
353
354
355
# File 'lib/active_record/relation.rb', line 352

def records # :nodoc:
  load
  @records
end

#reload ⇒ Object

Forces reloading of relation.



1217
1218
1219
1220
# File 'lib/active_record/relation.rb', line 1217

def reload
  reset
  load
end

#reset ⇒ Object



1222
1223
1224
1225
1226
1227
1228
1229
1230
1231
1232
# File 'lib/active_record/relation.rb', line 1222

def reset
  @future_result&.cancel
  @future_result = nil
  @delegate_to_model = false
  @to_sql = @arel = @loaded = @should_eager_load = nil
  @offsets = @take = nil
  @cache_keys = nil
  @cache_versions = nil
  @records = nil
  self
end

#scheduled? ⇒ Boolean

Returns true if the relation was scheduled on the background thread pool.

Returns:

  • (Boolean)


1197
1198
1199
# File 'lib/active_record/relation.rb', line 1197

def scheduled?
  !!@future_result
end

#scope_for_create ⇒ Object



1259
1260
1261
1262
1263
# File 'lib/active_record/relation.rb', line 1259

def scope_for_create
  hash = where_clause.to_h(model.table_name, equality_only: true)
  create_with_value.each { |k, v| hash[k.to_s] = v } unless create_with_value.empty?
  hash
end

#scoping(all_queries: nil, &block) ⇒ Object

Scope all queries to the current scope.

Comment.where(post_id: 1).scoping do
Comment.first
end
# SELECT "comments".* FROM "comments" WHERE "comments"."post_id" = 1 ORDER BY "comments"."id" ASC LIMIT 1

If all_queries: true is passed, scoping will apply to all queries for the relation including update and delete on instances. Once all_queries is set to true it cannot be set to false in a nested block.

Please check unscoped if you want to remove all previous scopes (including the default_scope) during the execution of a block.



555
556
557
558
559
560
561
562
563
564
# File 'lib/active_record/relation.rb', line 555

def scoping(all_queries: nil, &block)
  registry = model.scope_registry
  if global_scope?(registry) && all_queries == false
    raise ArgumentError, "Scoping is set to apply to all queries and cannot be unset in a nested block."
  elsif already_in_scope?(registry)
    yield
  else
    _scoping(self, registry, all_queries, &block)
  end
end

#size ⇒ Object

Returns size of the records.



363
364
365
366
367
368
369
# File 'lib/active_record/relation.rb', line 363

def size
  if loaded?
    records.length
  else
    count(:all)
  end
end

#then(&block) ⇒ Object

:nodoc:



1185
1186
1187
1188
1189
1190
1191
1192
1193
# File 'lib/active_record/relation.rb', line 1185

def then(&block) # :nodoc:
  if @future_result
    @future_result.then do
      yield self
    end
  else
    super
  end
end

#to_ary ⇒ Object Also known as: to_a

Converts relation objects to Array.



347
348
349
# File 'lib/active_record/relation.rb', line 347

def to_ary
  records.dup
end

#to_sql ⇒ Object

Returns sql statement for the relation.

User.where(name: 'Oscar').to_sql
# SELECT "users".* FROM "users"  WHERE "users"."name" = 'Oscar'


1238
1239
1240
1241
1242
1243
1244
1245
1246
1247
1248
1249
# File 'lib/active_record/relation.rb', line 1238

def to_sql
  @to_sql ||= if eager_loading?
    apply_join_dependency do |relation, join_dependency|
      relation = join_dependency.apply_column_aliases(relation)
      relation.to_sql
    end
  else
    model.with_connection do |conn|
      conn.unprepared_statement { conn.to_sql(arel) }
    end
  end
end

#touch_all(*names, time: nil) ⇒ Object

Touches all records in the current relation, setting the +updated_at+/+updated_on+ attributes to the current time or the time specified. It does not instantiate the involved models, and it does not trigger Active Record callbacks or validations. This method can be passed attribute names and an optional time argument. If attribute names are passed, they are updated along with +updated_at+/+updated_on+ attributes. If no time argument is passed, the current time is used as default.

Examples

# Touch all records
Person.all.touch_all
# => "UPDATE \"people\" SET \"updated_at\" = '2018-01-04 22:55:23.132670'"

# Touch multiple records with a custom attribute
Person.all.touch_all(:created_at)
# => "UPDATE \"people\" SET \"updated_at\" = '2018-01-04 22:55:23.132670', \"created_at\" = '2018-01-04 22:55:23.132670'"

# Touch multiple records with a specified time
Person.all.touch_all(time: Time.new(2020, 5, 16, 0, 0, 0))
# => "UPDATE \"people\" SET \"updated_at\" = '2020-05-16 00:00:00'"

# Touch records with scope
Person.where(name: 'David').touch_all
# => "UPDATE \"people\" SET \"updated_at\" = '2018-01-04 22:55:23.132670' WHERE \"people\".\"name\" = 'David'"


995
996
997
# File 'lib/active_record/relation.rb', line 995

def touch_all(*names, time: nil)
  update_all model.touch_attributes_with_time(*names, time: time)
end

#update(id = :all, attributes) ⇒ Object

:nodoc:



645
646
647
648
649
650
651
# File 'lib/active_record/relation.rb', line 645

def update(id = :all, attributes) # :nodoc:
  if id == :all
    each { |record| record.update(attributes) }
  else
    model.update(id, attributes)
  end
end

#update!(id = :all, attributes) ⇒ Object

:nodoc:



653
654
655
656
657
658
659
# File 'lib/active_record/relation.rb', line 653

def update!(id = :all, attributes) # :nodoc:
  if id == :all
    each { |record| record.update!(attributes) }
  else
    model.update!(id, attributes)
  end
end

#update_all(updates) ⇒ Object

Updates all records in the current relation with details given. This method constructs a single SQL UPDATE statement and sends it straight to the database. It does not instantiate the involved models and it does not trigger Active Record callbacks or validations. However, values passed to #update_all will still go through Active Record's normal type casting and serialization. Returns the number of rows affected.

Note: As Active Record callbacks are not triggered, this method will not automatically update +updated_at+/+updated_on+ columns.

Parameters

  • updates - A string, array, or hash representing the SET part of an SQL statement. Any strings provided will be type cast, unless you use Arel.sql. (Don't pass user-provided values to Arel.sql.)

Examples

# Update all customers with the given attributes
Customer.update_all wants_email: true

# Update all books with 'Rails' in their title
Book.where('title LIKE ?', '%Rails%').update_all(author: 'David')

# Update all books that match conditions, but limit it to 5 ordered by date
Book.where('title LIKE ?', '%Rails%').order(:created_at).limit(5).update_all(author: 'David')

# Update all invoices and set the number column to its id value.
Invoice.update_all('number = id')

# Update all books with 'Rails' in their title
Book.where('title LIKE ?', '%Rails%').update_all(title: Arel.sql("title + ' - volume 1'"))

Raises:

  • (ArgumentError)


602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
# File 'lib/active_record/relation.rb', line 602

def update_all(updates)
  raise ArgumentError, "Empty list of attributes to change" if updates.blank?

  return 0 if @none

  invalid_methods = INVALID_METHODS_FOR_UPDATE_AND_DELETE_ALL.select do |method|
    value = @values[method]
    method == :distinct ? value : value&.any?
  end
  if invalid_methods.any?
    ActiveRecord.deprecator.warn <<~MESSAGE
      `#{invalid_methods.join(', ')}` is not supported by `update_all` and was never included in the generated query.

      Calling `#{invalid_methods.join(', ')}` with `update_all` will raise an error in Rails 8.2.
    MESSAGE
  end

  if updates.is_a?(Hash)
    if model.locking_enabled? &&
        !updates.key?(model.locking_column) &&
        !updates.key?(model.locking_column.to_sym)
      attr = table[model.locking_column]
      updates[attr.name] = _increment_attribute(attr)
    end
    values = _substitute_values(updates)
  else
    values = Arel.sql(model.sanitize_sql_for_assignment(updates, table.name))
  end

  model.with_connection do |c|
    arel = eager_loading? ? apply_join_dependency.arel : arel()
    arel.source.left = table

    key = if model.composite_primary_key?
      primary_key.map { |pk| table[pk] }
    else
      table[primary_key]
    end
    stmt = arel.compile_update(values, key)
    c.update(stmt, "#{model} Update All").tap { reset }
  end
end

#update_counters(counters) ⇒ Object

Updates the counters of the records in the current relation.

Parameters

  • counter - A Hash containing the names of the fields to update as keys and the amount to update as values.
  • :touch option - Touch the timestamp columns when updating.
  • If attributes names are passed, they are updated along with update_at/on attributes.

Examples

# For Posts by a given author increment the comment_count by 1.
Post.where(author_id: author.id).update_counters(comment_count: 1)


952
953
954
955
956
957
958
959
960
961
962
963
964
965
966
967
968
969
970
# File 'lib/active_record/relation.rb', line 952

def update_counters(counters)
  touch = counters.delete(:touch)

  updates = {}
  counters.each do |counter_name, value|
    attr = table[counter_name]
    updates[attr.name] = _increment_attribute(attr, value)
  end

  if touch
    names = touch if touch != true
    names = Array.wrap(names)
    options = names.extract_options!
    touch_updates = model.touch_attributes_with_time(*names, **options)
    updates.merge!(touch_updates) unless touch_updates.empty?
  end

  update_all updates
end

#upsert(attributes, **kwargs) ⇒ Object

Updates or inserts (upserts) a single record into the database in a single SQL INSERT statement. It does not instantiate any models nor does it trigger Active Record callbacks or validations. Though passed values go through Active Record's type casting and serialization.

See #upsert_all for documentation.



824
825
826
# File 'lib/active_record/relation.rb', line 824

def upsert(attributes, **kwargs)
  upsert_all([ attributes ], **kwargs)
end

#upsert_all(attributes, on_duplicate: :update, update_only: nil, returning: nil, unique_by: nil, record_timestamps: nil) ⇒ Object

Updates or inserts (upserts) multiple records into the database in a single SQL INSERT statement. It does not instantiate any models nor does it trigger Active Record callbacks or validations. Though passed values go through Active Record's type casting and serialization.

The attributes parameter is an Array of Hashes. Every Hash determines the attributes for a single row and must have the same keys.

Returns an ActiveRecord::Result with its contents based on :returning (see below).

By default, upsert_all will update all the columns that can be updated when there is a conflict. These are all the columns except primary keys, read-only columns, and columns covered by the optional unique_by.

Options

[:returning] (PostgreSQL, SQLite3, and MariaDB only) An array of attributes to return for all successfully upserted records, which by default is the primary key. Pass returning: %w[ id name ] for both id and name or returning: false to omit the underlying RETURNING SQL clause entirely.

You can also pass an SQL string if you need more control on the return values
(for example, <tt>returning: Arel.sql("id, name as new_name")</tt>).

[:unique_by] (PostgreSQL and SQLite only) By default rows are considered to be unique by every unique index on the table. Any duplicate rows are skipped.

To skip rows according to just one unique index pass <tt>:unique_by</tt>.

Consider a Book model where no duplicate ISBNs make sense, but if any
row has an existing id, or is not unique by another unique index,
ActiveRecord::RecordNotUnique is raised.

Unique indexes can be identified by columns or name:

unique_by: :isbn
unique_by: %i[ author_id name ]
unique_by: :index_books_on_isbn

Because it relies on the index information from the database :unique_by is recommended to be paired with Active Record's schema_cache.

[:on_duplicate] Configure the behavior that will be used in case of conflict. Use :skip to ignore any conflicts or provide a safe SQL fragment wrapped with Arel.sql.

NOTE: If you use this option you must provide all the columns you want to update
by yourself.

Example:

Commodity.upsert_all(
  [
    { id: 2, name: "Copper", price: 4.84 },
    { id: 4, name: "Gold", price: 1380.87 },
    { id: 6, name: "Aluminium", price: 0.35 }
  ],
  on_duplicate: Arel.sql("price = GREATEST(commodities.price, EXCLUDED.price)")
)

See the related +:update_only+ option. Both options can't be used at the same time.

[:update_only] Provide a list of column names that will be updated in case of conflict. If not provided, upsert_all will update all the columns that can be updated. These are all the columns except primary keys, read-only columns, and columns covered by the optional unique_by

Example:

Commodity.upsert_all(
  [
    { id: 2, name: "Copper", price: 4.84 },
    { id: 4, name: "Gold", price: 1380.87 },
    { id: 6, name: "Aluminium", price: 0.35 }
  ],
  update_only: [:price] # Only prices will be updated
)

See the related +:on_duplicate+ option. Both options can't be used at the same time.

[:record_timestamps] By default, automatic setting of timestamp columns is controlled by the model's record_timestamps config, matching typical behavior.

To override this and force automatic setting of timestamp columns one
way or the other, pass <tt>:record_timestamps</tt>:

record_timestamps: true  # Always set timestamps automatically
record_timestamps: false # Never set timestamps automatically

Examples

# Inserts multiple records, performing an upsert when records have duplicate ISBNs.
# Here "Eloquent Ruby" overwrites "Rework" because its ISBN is duplicate.

Book.upsert_all([
{ title: "Rework", author: "David", isbn: "1" },
{ title: "Eloquent Ruby", author: "Russ", isbn: "1" }
], unique_by: :isbn)

Book.find_by(isbn: "1").title # => "Eloquent Ruby"


936
937
938
# File 'lib/active_record/relation.rb', line 936

def upsert_all(attributes, on_duplicate: :update, update_only: nil, returning: nil, unique_by: nil, record_timestamps: nil)
  InsertAll.execute(self, attributes, on_duplicate: on_duplicate, update_only: update_only, returning: returning, unique_by: unique_by, record_timestamps: record_timestamps)
end

#values ⇒ Object



1310
1311
1312
# File 'lib/active_record/relation.rb', line 1310

def values
  @values.dup
end

#values_for_queries ⇒ Object

:nodoc:



1314
1315
1316
# File 'lib/active_record/relation.rb', line 1314

def values_for_queries # :nodoc:
  @values.except(:extending, :skip_query_cache, :strict_loading)
end

#where_values_hash(relation_table_name = model.table_name) ⇒ Object

Returns a hash of where conditions.

User.where(name: 'Oscar').where_values_hash
# => {name: "Oscar"}


1255
1256
1257
# File 'lib/active_record/relation.rb', line 1255

def where_values_hash(relation_table_name = model.table_name) # :nodoc:
  where_clause.to_h(relation_table_name)
end