Class: ActiveRecord::Schema

Inherits:
Migration::Current show all
Includes:
Definition
Defined in:
lib/active_record/schema.rb

Overview

Active Record Schema

Allows programmers to programmatically define a schema in a portable DSL. This means you can define tables, indexes, etc. without using SQL directly, so your applications can more easily support multiple databases.

Usage:

ActiveRecord::Schema[7.0].define do
create_table :authors do |t|
  t.string :name, null: false
end

add_index :authors, :name, :unique

create_table :posts do |t|
  t.integer :author_id, null: false
  t.string :subject
  t.text :body
  t.boolean :private, default: false
end

add_index :posts, :author_id
end

ActiveRecord::Schema is only supported by database adapters that also support migrations, the two features being very similar.

Defined Under Namespace

Modules: Definition

Class Method Summary collapse

Methods included from Definition

#define

Methods inherited from Migration::Current

#change_table, #compatible_table_definition, #create_join_table, #create_table, #drop_table

Class Method Details

.[](version) ⇒ Object



70
71
72
73
74
75
# File 'lib/active_record/schema.rb', line 70

def self.[](version)
  @class_for_version ||= {}
  @class_for_version[version] ||= Class.new(Migration::Compatibility.find(version)) do
    include Definition
  end
end