Method: ActiveRecord::Scoping::ClassMethods#scope_attributes
- Defined in:
- lib/active_record/scoping.rb
#scope_attributes ⇒ Object
Collects attributes from scopes that should be applied when creating an AR instance for the particular class this is called on.
16 17 18 |
# File 'lib/active_record/scoping.rb', line 16 def scope_attributes all.scope_for_create end |