Method: ActiveRecord::Validations::ClassMethods#validates_associated
- Defined in:
- lib/active_record/validations/associated.rb
#validates_associated(*attr_names) ⇒ Object
Validates whether the associated object or objects are all valid. Works with any kind of association.
class Book < ActiveRecord::Base
has_many :pages
belongs_to :library
validates_associated :pages, :library
end
WARNING: This validation must not be used on both ends of an association. Doing so will lead to a circular dependency and cause infinite recursion.
NOTE: This validation will not fail if the association hasn't been
assigned. If you want to ensure that the association is both present and
guaranteed to be valid, you also need to use validates_presence_of.
Configuration options:
- :message - A custom error message (default is: "is invalid").
- :on - Specifies when this validation is active. Runs in all validation contexts by default (+nil+), other options are :create and :update.
- :if - Specifies a method, proc or string to call to determine
if the validation should occur (e.g. if: :allow_validation,
or if: Proc.new { |user| user.signup_step > 2 }). The method,
proc or string should return or evaluate to a
trueorfalsevalue. - :unless - Specifies a method, proc or string to call to
determine if the validation should not occur (e.g. unless: :skip_validation,
or unless: Proc.new { |user| user.signup_step <= 2 }). The
method, proc or string should return or evaluate to a
trueorfalsevalue.
44 45 46 |
# File 'lib/active_record/validations/associated.rb', line 44 def validates_associated(*attr_names) validates_with AssociatedValidator, _merge_attributes(attr_names) end |