Method: ActiveRecord::AttributeMethods::ClassMethods#attribute_method_suffix
- Defined in:
- lib/active_record/attribute_methods.rb
#attribute_method_suffix(*suffixes) ⇒ Object
Declares a method available for all attributes with the given suffix. Uses method_missing and respond_to? to rewrite the method
#{attr}#{suffix}(*args, &block)
to
attribute#{suffix}(#{attr}, *args, &block)
An attribute#{suffix} instance method must exist and accept at least the attr argument.
For example:
class Person < ActiveRecord::Base
attribute_method_suffix '_changed?'
private
def attribute_changed?(attr)
...
end
end
person = Person.find(1)
person.name_changed? # => false
person.name = 'Hubert'
person.name_changed? # => true
46 47 48 49 |
# File 'lib/active_record/attribute_methods.rb', line 46 def attribute_method_suffix(*suffixes) attribute_method_suffixes.concat suffixes rebuild_attribute_method_regexp end |