Module: ActiveRecord::AttributeMethods::Serialization::ClassMethods
- Defined in:
- lib/active_record/attribute_methods/serialization.rb
Instance Method Summary collapse
-
#serialize(attr_name, class_name_or_coder = Object) ⇒ Object
If you have an attribute that needs to be saved to the database as an object, and retrieved as the same object, then specify the name of that attribute using this method and it will be handled automatically.
Instance Method Details
#serialize(attr_name, class_name_or_coder = Object) ⇒ Object
If you have an attribute that needs to be saved to the database as an
object, and retrieved as the same object, then specify the name of that
attribute using this method and it will be handled automatically. The
serialization is done through YAML. If class_name is specified, the
serialized object must be of that class on retrieval or
SerializationTypeMismatch will be raised.
A notable side effect of serialized attributes is that the model will be updated on every save, even if it is not dirty.
Parameters
attr_name- The field name that should be serialized.class_name_or_coder- Optional, a coder object, which responds to.load/.dumpor a class name that the object type should be equal to.
Example
Serialize a preferences attribute.
class User < ActiveRecord::Base serialize :preferences end
Serialize preferences using JSON as coder.
class User < ActiveRecord::Base serialize :preferences, JSON end
Serialize preferences as Hash using YAML coder.
class User < ActiveRecord::Base serialize :preferences, Hash end
52 53 54 55 56 57 58 59 60 61 62 63 64 |
# File 'lib/active_record/attribute_methods/serialization.rb', line 52 def serialize(attr_name, class_name_or_coder = Object) include Behavior coder = if [:load, :dump].all? { |x| class_name_or_coder.respond_to?(x) } class_name_or_coder else Coders::YAMLColumn.new(class_name_or_coder) end # merge new serialized attribute and create new hash to ensure that each class in inheritance hierarchy # has its own hash of own serialized attributes self.serialized_attributes = serialized_attributes.merge(attr_name.to_s => coder) end |