Method: ActiveRecord::AttributeMethods#attribute_for_inspect
- Defined in:
- lib/active_record/attribute_methods.rb
#attribute_for_inspect(attr_name) ⇒ Object
Returns an #inspect-like string for the value of the attribute attr_name. String attributes are truncated upto 50 characters, Date and Time attributes are returned in the :db format, Array attributes are truncated upto 10 values. Other attributes return the value of #inspect without modification.
person = Person.create!(name: 'David Heinemeier Hansson ' * 3)
person.attribute_for_inspect(:name)
# => "\"David Heinemeier Hansson David Heinemeier Hansson ...\""
person.attribute_for_inspect(:created_at)
# => "\"2012-10-22 00:15:07\""
person.attribute_for_inspect(:tag_ids)
# => "[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...]"
308 309 310 311 312 313 314 315 316 317 318 319 320 321 |
# File 'lib/active_record/attribute_methods.rb', line 308 def attribute_for_inspect(attr_name) value = read_attribute(attr_name) if value.is_a?(String) && value.length > 50 "#{value[0, 50]}...".inspect elsif value.is_a?(Date) || value.is_a?(Time) %("#{value.to_s(:db)}") elsif value.is_a?(Array) && value.size > 10 inspected = value.first(10).inspect %(#{inspected[0...-1]}, ...]) else value.inspect end end |