Method: ActiveRecord::Base#encode_with

Defined in:
lib/active_record/base.rb

#encode_with(coder) ⇒ Object

Populate coder with attributes about this record that should be serialized. The structure of coder defined in this method is guaranteed to match the structure of coder passed to the init_with method.

Example:

class Post < ActiveRecord::Base
end
coder = {}
Post.new.encode_with(coder)
coder # => { 'id' => nil, ... }


580
581
582
# File 'lib/active_record/base.rb', line 580

def encode_with(coder)
  coder['attributes'] = attributes
end