Method: ActiveRecord::Calculations#calculate
- Defined in:
- lib/active_record/relation/calculations.rb
#calculate(operation, column_name, options = {}) ⇒ Object
This calculates aggregate values in the given column. Methods for count, sum, average, minimum, and maximum have been added as shortcuts.
There are two basic forms of output:
* Single aggregate value: The single value is type cast to Fixnum for COUNT, Float
for AVG, and the given column's type for everything else.
* Grouped values: This returns an ordered hash of the values and groups them. It
takes either a column name, or the name of a belongs_to association.
values = Person.group('last_name').maximum(:age)
puts values["Drake"]
# => 43
drake = Family.find_by(last_name: 'Drake')
values = Person.group(:family).maximum(:age) # Person belongs_to :family
puts values[drake]
# => 43
values.each do |family, max_age|
...
end
Person.calculate(:count, :all) # The same as Person.count
Person.average(:age) # SELECT AVG(age) FROM people...
# Selects the minimum age for any family without any minors
Person.group(:last_name).having("min(age) > 17").minimum(:age)
Person.sum("2 * age")
101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 |
# File 'lib/active_record/relation/calculations.rb', line 101 def calculate(operation, column_name, = {}) relation = with_default_scope if relation.equal?(self) if has_include?(column_name) construct_relation_for_association_calculations.calculate(operation, column_name, ) else perform_calculation(operation, column_name, ) end else relation.calculate(operation, column_name, ) end rescue ThrowResult 0 end |