Method: ActiveRecord::ConnectionAdapters::DatabaseLimits#allowed_index_name_length
- Defined in:
- lib/active_record/connection_adapters/abstract/database_limits.rb
#allowed_index_name_length ⇒ Object
Returns the maximum allowed length for an index name. This limit is enforced by rails and Is less than or equal to index_name_length. The gap between index_name_length is to allow internal rails operations to use prefixes in temporary operations.
25 26 27 |
# File 'lib/active_record/connection_adapters/abstract/database_limits.rb', line 25 def allowed_index_name_length index_name_length end |