Method: ActiveRecord::ConnectionAdapters::DatabaseStatements#exec_delete
- Defined in:
- lib/active_record/connection_adapters/abstract/database_statements.rb
#exec_delete(sql, name = nil, binds = []) ⇒ Object
Executes delete sql statement in the context of this connection using binds as the bind substitutes. name is logged along with the executed sql statement.
128 129 130 |
# File 'lib/active_record/connection_adapters/abstract/database_statements.rb', line 128 def exec_delete(sql, name = nil, binds = []) exec_query(sql, name, binds) end |