Method: ActiveRecord::ConnectionAdapters::DatabaseStatements#exec_delete
- Defined in:
- lib/active_record/connection_adapters/abstract/database_statements.rb
#exec_delete(sql, name, binds) ⇒ Object
Executes delete sql statement in the context of this connection using binds as the bind substitutes. name is the logged along with the executed sql statement.
69 70 71 |
# File 'lib/active_record/connection_adapters/abstract/database_statements.rb', line 69 def exec_delete(sql, name, binds) exec_query(sql, name, binds) end |