Method: ActiveRecord::ConnectionAdapters::DatabaseStatements#execute
- Defined in:
- lib/active_record/connection_adapters/abstract/database_statements.rb
#execute(sql, name = nil) ⇒ Object
Executes the SQL statement in the context of this connection and returns the raw result from the connection adapter. Note: depending on your database connector, the result returned by this method may be manually memory managed. Consider using the exec_query wrapper instead.
120 121 122 |
# File 'lib/active_record/connection_adapters/abstract/database_statements.rb', line 120 def execute(sql, name = nil) raise NotImplementedError end |