Module: ActiveRecord::ConnectionAdapters::PostgreSQL::SchemaStatements
- Included in:
- ActiveRecord::ConnectionAdapters::PostgreSQLAdapter
- Defined in:
- lib/active_record/connection_adapters/postgresql/schema_statements.rb
Instance Method Summary collapse
-
#add_column(table_name, column_name, type, **options) ⇒ Object
:nodoc:.
-
#add_exclusion_constraint(table_name, expression, **options) ⇒ Object
Adds a new exclusion constraint to the table.
- #add_foreign_key(from_table, to_table, **options) ⇒ Object
-
#add_index(table_name, column_name, **options) ⇒ Object
:nodoc:.
-
#add_index_options(table_name, column_name, **options) ⇒ Object
:nodoc:.
-
#add_unique_constraint(table_name, column_name = nil, **options) ⇒ Object
Adds a new unique constraint to the table.
-
#build_change_column_default_definition(table_name, column_name, default_or_changes) ⇒ Object
:nodoc:.
-
#build_change_column_definition(table_name, column_name, type, **options) ⇒ Object
Builds a ChangeColumnDefinition object.
-
#build_create_index_definition(table_name, column_name, **options) ⇒ Object
:nodoc:.
-
#change_column(table_name, column_name, type, **options) ⇒ Object
:nodoc:.
-
#change_column_comment(table_name, column_name, comment_or_changes) ⇒ Object
Adds comment for given table column or drops it if
commentis anil. -
#change_column_default(table_name, column_name, default_or_changes) ⇒ Object
Changes the default value of a table column.
-
#change_column_null(table_name, column_name, null, default = nil) ⇒ Object
:nodoc:.
-
#change_table_comment(table_name, comment_or_changes) ⇒ Object
Adds comment for given table or drops it if
commentis anil. -
#check_constraints(table_name) ⇒ Object
:nodoc:.
-
#client_min_messages ⇒ Object
Returns the current client message level.
-
#client_min_messages=(level) ⇒ Object
Set the client message level.
-
#collation ⇒ Object
Returns the current database collation.
-
#columns_for_distinct(columns, orders) ⇒ Object
PostgreSQL requires the ORDER BY columns in the select list for distinct queries, and requires that the ORDER BY include the distinct column.
-
#create_database(name, options = {}) ⇒ Object
Create a new PostgreSQL database.
-
#create_schema(schema_name, force: nil, if_not_exists: nil) ⇒ Object
Creates a schema for the given schema name.
-
#create_schema_dumper(options) ⇒ Object
:nodoc:.
-
#ctype ⇒ Object
Returns the current database ctype.
-
#current_database ⇒ Object
Returns the current database name.
-
#current_schema ⇒ Object
Returns the current schema name.
-
#current_schemas ⇒ Object
Returns an array of the names of all schemas presently in the effective search path, in their priority order.
-
#default_sequence_name(table_name, pk = "id") ⇒ Object
Returns the sequence name for a table's primary key or some other specified key.
-
#drop_database(name) ⇒ Object
Drops a PostgreSQL database.
-
#drop_schema(schema_name, **options) ⇒ Object
Drops the schema for the given schema name.
-
#drop_table(*table_names, **options) ⇒ Object
:nodoc:.
-
#encoding ⇒ Object
Returns the current database encoding format.
-
#exclusion_constraint_options(table_name, expression, options) ⇒ Object
:nodoc:.
-
#exclusion_constraints(table_name) ⇒ Object
Returns an array of exclusion constraints for the given table.
-
#foreign_key_column_for(table_name, column_name) ⇒ Object
:nodoc:.
- #foreign_keys(table_name) ⇒ Object
- #foreign_table_exists?(table_name) ⇒ Boolean
- #foreign_tables ⇒ Object
-
#index_name(table_name, options) ⇒ Object
:nodoc:.
-
#index_name_exists?(table_name, index_name) ⇒ Boolean
Verifies existence of an index with a given name.
-
#indexes(table_name) ⇒ Object
Returns an array of indexes for the given table.
-
#inherited_table_names(table_name) ⇒ Object
Returns the inherited table name of a given table.
-
#pk_and_sequence_for(table) ⇒ Object
Returns a table's primary key and belonging sequence.
-
#primary_keys(table_name) ⇒ Object
:nodoc:.
-
#quoted_include_columns_for_index(column_names) ⇒ Object
:nodoc:.
-
#recreate_database(name, options = {}) ⇒ Object
Drops the database specified on the
nameattribute and creates it again using the providedoptions. -
#remove_exclusion_constraint(table_name, expression = nil, **options) ⇒ Object
Removes the given exclusion constraint from the table.
-
#remove_index(table_name, column_name = nil, **options) ⇒ Object
:nodoc:.
-
#remove_unique_constraint(table_name, column_name = nil, **options) ⇒ Object
Removes the given unique constraint from the table.
-
#rename_column(table_name, column_name, new_column_name) ⇒ Object
Renames a column in a table.
-
#rename_index(table_name, old_name, new_name) ⇒ Object
Renames an index of a table.
-
#rename_schema(schema_name, new_name) ⇒ Object
Renames the schema for the given schema name.
-
#rename_table(table_name, new_name, **options) ⇒ Object
Renames a table.
-
#reset_pk_sequence!(table, pk = nil, sequence = nil) ⇒ Object
Resets the sequence of a table's primary key to the maximum value.
-
#schema_creation ⇒ Object
:nodoc:.
-
#schema_exists?(name) ⇒ Boolean
Returns true if schema exists.
-
#schema_names ⇒ Object
Returns an array of schema names.
-
#schema_search_path ⇒ Object
Returns the active schema search path.
-
#schema_search_path=(schema_csv) ⇒ Object
Sets the schema search path to a string of comma-separated schema names.
- #serial_sequence(table, column) ⇒ Object
-
#set_pk_sequence!(table, value) ⇒ Object
Sets the sequence of a table's primary key to the specified value.
-
#table_comment(table_name) ⇒ Object
Returns a comment stored in database for given table.
-
#table_options(table_name) ⇒ Object
:nodoc:.
-
#table_partition_definition(table_name) ⇒ Object
Returns the partition definition of a given table.
-
#type_to_sql(type, limit: nil, precision: nil, scale: nil, array: nil, enum_type: nil) ⇒ Object
Maps logical Rails types to PostgreSQL-specific data types.
-
#unique_constraint_options(table_name, column_name, options) ⇒ Object
:nodoc:.
-
#unique_constraints(table_name) ⇒ Object
Returns an array of unique constraints for the given table.
-
#update_table_definition(table_name, base) ⇒ Object
:nodoc:.
-
#validate_check_constraint(table_name, **options) ⇒ Object
Validates the given check constraint.
-
#validate_constraint(table_name, constraint_name) ⇒ Object
Validates the given constraint.
-
#validate_foreign_key(from_table, to_table = nil, **options) ⇒ Object
Validates the given foreign key.
Instance Method Details
#add_column(table_name, column_name, type, **options) ⇒ Object
:nodoc:
482 483 484 485 486 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 482 def add_column(table_name, column_name, type, **) # :nodoc: clear_cache! super change_column_comment(table_name, column_name, [:comment]) if .key?(:comment) end |
#add_exclusion_constraint(table_name, expression, **options) ⇒ Object
Adds a new exclusion constraint to the table. expression is a String
representation of a list of exclusion elements and operators.
add_exclusion_constraint :products, "price WITH =, availability_range WITH &&", using: :gist, name: "price_check"
generates:
ALTER TABLE "products" ADD CONSTRAINT price_check EXCLUDE USING gist (price WITH =, availability_range WITH &&)
The options hash can include the following keys:
[:name]
The constraint name. Defaults to excl_rails_false or :immediate or :deferred to specify the default behavior. Defaults to false.
[:using]
Specify which index method to use when creating this exclusion constraint (e.g. :btree, :gist etc).
[:where]
Specify an exclusion constraint on a subset of the table (internally PostgreSQL creates a partial index for this).
789 790 791 792 793 794 795 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 789 def add_exclusion_constraint(table_name, expression, **) = (table_name, expression, ) at = create_alter_table(table_name) at.add_exclusion_constraint(expression, ) execute schema_creation.accept(at) end |
#add_foreign_key(from_table, to_table, **options) ⇒ Object
605 606 607 608 609 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 605 def add_foreign_key(from_table, to_table, **) assert_valid_deferrable([:deferrable]) super end |
#add_index(table_name, column_name, **options) ⇒ Object
:nodoc:
551 552 553 554 555 556 557 558 559 560 561 562 563 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 551 def add_index(table_name, column_name, **) # :nodoc: create_index = build_create_index_definition(table_name, column_name, **) result = execute schema_creation.accept(create_index) index = create_index.index if index.comment name = Utils.extract_schema_qualified_name(table_name.to_s) index_name = name.schema ? PostgreSQL::Name.new(name.schema, index.name).to_s : index.name execute "COMMENT ON INDEX #{quote_table_name(index_name)} IS #{quote(index.comment)}" end result end |
#add_index_options(table_name, column_name, **options) ⇒ Object
:nodoc:
981 982 983 984 985 986 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 981 def (table_name, column_name, **) # :nodoc: if (where = [:where]) && table_exists?(table_name) && column_exists?(table_name, where) [:where] = quote_column_name(where) end super end |
#add_unique_constraint(table_name, column_name = nil, **options) ⇒ Object
Adds a new unique constraint to the table.
add_unique_constraint :sections, [:position], deferrable: :deferred, name: "unique_position", nulls_not_distinct: true
generates:
ALTER TABLE "sections" ADD CONSTRAINT unique_position UNIQUE (position) DEFERRABLE INITIALLY DEFERRED
If you want to change an existing unique index to deferrable, you can use :using_index to create deferrable unique constraints.
add_unique_constraint :sections, deferrable: :deferred, name: "unique_position", using_index: "index_sections_on_position"
The options hash can include the following keys:
[:name]
The constraint name. Defaults to uniq_rails_false or :immediate or :deferred to specify the default behavior. Defaults to false.
[:using_index]
To specify an existing unique index name. Defaults to nil.
[:nulls_not_distinct]
Create a unique constraint where NULLs are treated equally.
Note: only supported by PostgreSQL version 15.0.0 and greater.
840 841 842 843 844 845 846 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 840 def add_unique_constraint(table_name, column_name = nil, **) = (table_name, column_name, ) at = create_alter_table(table_name) at.add_unique_constraint(column_name, ) execute schema_creation.accept(at) end |
#build_change_column_default_definition(table_name, column_name, default_or_changes) ⇒ Object
:nodoc:
511 512 513 514 515 516 517 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 511 def build_change_column_default_definition(table_name, column_name, default_or_changes) # :nodoc: column = column_for(table_name, column_name) return unless column default = extract_new_default_value(default_or_changes) ChangeColumnDefaultDefinition.new(column, default) end |
#build_change_column_definition(table_name, column_name, type, **options) ⇒ Object
Builds a ChangeColumnDefinition object.
This definition object contains information about the column change that would occur
if the same arguments were passed to #change_column. See #change_column for information about
passing a table_name, column_name, type and other options that can be passed.
500 501 502 503 504 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 500 def build_change_column_definition(table_name, column_name, type, **) # :nodoc: td = create_table_definition(table_name) cd = td.new_column_definition(column_name, type, **) ChangeColumnDefinition.new(cd, column_name) end |
#build_create_index_definition(table_name, column_name, **options) ⇒ Object
:nodoc:
565 566 567 568 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 565 def build_create_index_definition(table_name, column_name, **) # :nodoc: index, algorithm, if_not_exists = (table_name, column_name, **) CreateIndexDefinition.new(index, algorithm, if_not_exists) end |
#change_column(table_name, column_name, type, **options) ⇒ Object
:nodoc:
488 489 490 491 492 493 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 488 def change_column(table_name, column_name, type, **) # :nodoc: clear_cache! sqls, procs = Array(change_column_for_alter(table_name, column_name, type, **)).partition { |v| v.is_a?(String) } execute "ALTER TABLE #{quote_table_name(table_name)} #{sqls.join(", ")}" procs.each(&:call) end |
#change_column_comment(table_name, column_name, comment_or_changes) ⇒ Object
Adds comment for given table column or drops it if comment is a nil
531 532 533 534 535 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 531 def change_column_comment(table_name, column_name, comment_or_changes) # :nodoc: clear_cache! comment = extract_new_comment_value(comment_or_changes) execute "COMMENT ON COLUMN #{quote_table_name(table_name)}.#{quote_column_name(column_name)} IS #{quote(comment)}" end |
#change_column_default(table_name, column_name, default_or_changes) ⇒ Object
Changes the default value of a table column.
507 508 509 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 507 def change_column_default(table_name, column_name, default_or_changes) # :nodoc: execute "ALTER TABLE #{quote_table_name(table_name)} #{change_column_default_for_alter(table_name, column_name, default_or_changes)}" end |
#change_column_null(table_name, column_name, null, default = nil) ⇒ Object
:nodoc:
519 520 521 522 523 524 525 526 527 528 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 519 def change_column_null(table_name, column_name, null, default = nil) # :nodoc: validate_change_column_null_argument!(null) clear_cache! unless null || default.nil? column = column_for(table_name, column_name) execute "UPDATE #{quote_table_name(table_name)} SET #{quote_column_name(column_name)}=#{quote_default_expression(default, column)} WHERE #{quote_column_name(column_name)} IS NULL" if column end execute "ALTER TABLE #{quote_table_name(table_name)} ALTER COLUMN #{quote_column_name(column_name)} #{null ? 'DROP' : 'SET'} NOT NULL" end |
#change_table_comment(table_name, comment_or_changes) ⇒ Object
Adds comment for given table or drops it if comment is a nil
538 539 540 541 542 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 538 def change_table_comment(table_name, comment_or_changes) # :nodoc: clear_cache! comment = extract_new_comment_value(comment_or_changes) execute "COMMENT ON TABLE #{quote_table_name(table_name)} IS #{quote(comment)}" end |
#check_constraints(table_name) ⇒ Object
:nodoc:
673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 690 691 692 693 694 695 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 673 def check_constraints(table_name) # :nodoc: scope = quoted_scope(table_name) check_info = internal_exec_query(" SELECT conname, pg_get_constraintdef(c.oid, true) AS constraintdef, c.convalidated AS valid\n FROM pg_constraint c\n JOIN pg_class t ON c.conrelid = t.oid\n JOIN pg_namespace n ON n.oid = c.connamespace\n WHERE c.contype = 'c'\n AND t.relname = \#{scope[:name]}\n AND n.nspname = \#{scope[:schema]}\n SQL\n\n check_info.map do |row|\n options = {\n name: row[\"conname\"],\n validate: row[\"valid\"]\n }\n expression = row[\"constraintdef\"][/CHECK \\((.+)\\)/m, 1]\n\n CheckConstraintDefinition.new(table_name, expression, options)\n end\nend\n", "SCHEMA", allow_retry: true, materialize_transactions: false) |
#client_min_messages ⇒ Object
Returns the current client message level.
316 317 318 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 316 def query_value("SHOW client_min_messages", "SCHEMA") end |
#client_min_messages=(level) ⇒ Object
Set the client message level.
321 322 323 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 321 def (level) internal_execute("SET client_min_messages TO '#{level}'", "SCHEMA") end |
#collation ⇒ Object
Returns the current database collation.
254 255 256 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 254 def collation query_value("SELECT datcollate FROM pg_database WHERE datname = current_database()", "SCHEMA") end |
#columns_for_distinct(columns, orders) ⇒ Object
PostgreSQL requires the ORDER BY columns in the select list for distinct queries, and requires that the ORDER BY include the distinct column.
912 913 914 915 916 917 918 919 920 921 922 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 912 def columns_for_distinct(columns, orders) # :nodoc: order_columns = orders.compact_blank.map { |s| # Convert Arel node to string s = visitor.compile(s) unless s.is_a?(String) # Remove any ASC/DESC modifiers s.gsub(/\s+(?:ASC|DESC)\b/i, "") .gsub(/\s+NULLS\s+(?:FIRST|LAST)\b/i, "") }.compact_blank.map.with_index { |column, i| "#{column} AS alias_#{i}" } (order_columns << super).join(", ") end |
#create_database(name, options = {}) ⇒ Object
Create a new PostgreSQL database. Options include :owner, :template, :encoding (defaults to utf8), :locale_provider, :locale, :collation, :ctype, :tablespace, and :connection_limit (note that MySQL uses :charset while PostgreSQL uses :encoding).
Example:
create_database config[:database], config
create_database 'foo_development', encoding: 'unicode'
23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 23 def create_database(name, = {}) = { encoding: "utf8" }.merge!(.symbolize_keys) option_string = .each_with_object(+"") do |(key, value), memo| memo << case key when :owner " OWNER = \"#{value}\"" when :template " TEMPLATE = \"#{value}\"" when :encoding " ENCODING = '#{value}'" when :locale_provider " LOCALE_PROVIDER = '#{value}'" when :locale " LOCALE = '#{value}'" when :collation " LC_COLLATE = '#{value}'" when :ctype " LC_CTYPE = '#{value}'" when :tablespace " TABLESPACE = \"#{value}\"" when :connection_limit " CONNECTION LIMIT = #{value}" else "" end end execute "CREATE DATABASE #{quote_table_name(name)}#{option_string}" end |
#create_schema(schema_name, force: nil, if_not_exists: nil) ⇒ Object
Creates a schema for the given schema name.
275 276 277 278 279 280 281 282 283 284 285 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 275 def create_schema(schema_name, force: nil, if_not_exists: nil) if force && if_not_exists raise ArgumentError, "Options `:force` and `:if_not_exists` cannot be used simultaneously." end if force drop_schema(schema_name, if_exists: true) end execute("CREATE SCHEMA#{' IF NOT EXISTS' if if_not_exists} #{quote_schema_name(schema_name)}") end |
#create_schema_dumper(options) ⇒ Object
:nodoc:
928 929 930 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 928 def create_schema_dumper() # :nodoc: PostgreSQL::SchemaDumper.create(self, ) end |
#ctype ⇒ Object
Returns the current database ctype.
259 260 261 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 259 def ctype query_value("SELECT datctype FROM pg_database WHERE datname = current_database()", "SCHEMA") end |
#current_database ⇒ Object
Returns the current database name.
231 232 233 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 231 def current_database query_value("SELECT current_database()", "SCHEMA") end |
#current_schema ⇒ Object
Returns the current schema name.
236 237 238 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 236 def current_schema query_value("SELECT current_schema", "SCHEMA") end |
#current_schemas ⇒ Object
Returns an array of the names of all schemas presently in the effective search path, in their priority order.
242 243 244 245 246 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 242 def current_schemas # :nodoc: schemas = query_value("SELECT current_schemas(false)", "SCHEMA") decoder = PG::TextDecoder::Array.new decoder.decode(schemas) end |
#default_sequence_name(table_name, pk = "id") ⇒ Object
Returns the sequence name for a table's primary key or some other specified key.
326 327 328 329 330 331 332 333 334 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 326 def default_sequence_name(table_name, pk = "id") # :nodoc: return nil if pk.is_a?(Array) result = serial_sequence(table_name, pk) return nil unless result Utils.extract_schema_qualified_name(result).to_s rescue ActiveRecord::StatementInvalid PostgreSQL::Name.new(nil, "#{table_name}_#{pk}_seq").to_s end |
#drop_database(name) ⇒ Object
Drops a PostgreSQL database.
Example:
drop_database 'matt_development'
58 59 60 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 58 def drop_database(name) # :nodoc: execute "DROP DATABASE IF EXISTS #{quote_table_name(name)}" end |
#drop_schema(schema_name, **options) ⇒ Object
Drops the schema for the given schema name.
288 289 290 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 288 def drop_schema(schema_name, **) execute "DROP SCHEMA#{' IF EXISTS' if options[:if_exists]} #{quote_schema_name(schema_name)} CASCADE" end |
#drop_table(*table_names, **options) ⇒ Object
:nodoc:
62 63 64 65 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 62 def drop_table(*table_names, **) # :nodoc: table_names.each { |table_name| schema_cache.clear_data_source_cache!(table_name.to_s) } execute "DROP TABLE#{' IF EXISTS' if options[:if_exists]} #{table_names.map { |table_name| quote_table_name(table_name) }.join(', ')}#{' CASCADE' if options[:force] == :cascade}" end |
#encoding ⇒ Object
Returns the current database encoding format.
249 250 251 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 249 def encoding query_value("SELECT pg_encoding_to_char(encoding) FROM pg_database WHERE datname = current_database()", "SCHEMA") end |
#exclusion_constraint_options(table_name, expression, options) ⇒ Object
:nodoc:
797 798 799 800 801 802 803 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 797 def (table_name, expression, ) # :nodoc: assert_valid_deferrable([:deferrable]) = .dup [:name] ||= exclusion_constraint_name(table_name, expression: expression, **) end |
#exclusion_constraints(table_name) ⇒ Object
Returns an array of exclusion constraints for the given table. The exclusion constraints are represented as ExclusionConstraintDefinition objects.
699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 699 def exclusion_constraints(table_name) scope = quoted_scope(table_name) exclusion_info = internal_exec_query(" SELECT conname, pg_get_constraintdef(c.oid) AS constraintdef, c.condeferrable, c.condeferred\n FROM pg_constraint c\n JOIN pg_class t ON c.conrelid = t.oid\n JOIN pg_namespace n ON n.oid = c.connamespace\n WHERE c.contype = 'x'\n AND t.relname = \#{scope[:name]}\n AND n.nspname = \#{scope[:schema]}\n SQL\n\n exclusion_info.map do |row|\n method_and_elements, predicate = row[\"constraintdef\"].split(\" WHERE \")\n method_and_elements_parts = method_and_elements.match(/EXCLUDE(?: USING (?<using>\\S+))? \\((?<expression>.+)\\)/m)\n predicate.remove!(/ DEFERRABLE(?: INITIALLY (?:IMMEDIATE|DEFERRED))?/) if predicate\n predicate = predicate.from(2).to(-3) if predicate # strip 2 opening and closing parentheses\n\n deferrable = extract_constraint_deferrable(row[\"condeferrable\"], row[\"condeferred\"])\n\n options = {\n name: row[\"conname\"],\n using: method_and_elements_parts[\"using\"].to_sym,\n where: predicate,\n deferrable: deferrable\n }\n\n ExclusionConstraintDefinition.new(table_name, method_and_elements_parts[\"expression\"], options)\n end\nend\n", "SCHEMA") |
#foreign_key_column_for(table_name, column_name) ⇒ Object
:nodoc:
976 977 978 979 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 976 def foreign_key_column_for(table_name, column_name) # :nodoc: _schema, table_name = extract_schema_qualified_name(table_name) super end |
#foreign_keys(table_name) ⇒ Object
611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 611 def foreign_keys(table_name) scope = quoted_scope(table_name) fk_info = internal_exec_query(" SELECT t2.oid::regclass::text AS to_table, c.conname AS name, c.confupdtype AS on_update, c.confdeltype AS on_delete, c.convalidated AS valid, c.condeferrable AS deferrable, c.condeferred AS deferred, c.conrelid, c.confrelid,\n (\n SELECT array_agg(a.attname ORDER BY idx)\n FROM (\n SELECT idx, c.conkey[idx] AS conkey_elem\n FROM generate_subscripts(c.conkey, 1) AS idx\n ) indexed_conkeys\n JOIN pg_attribute a ON a.attrelid = t1.oid\n AND a.attnum = indexed_conkeys.conkey_elem\n ) AS conkey_names,\n (\n SELECT array_agg(a.attname ORDER BY idx)\n FROM (\n SELECT idx, c.confkey[idx] AS confkey_elem\n FROM generate_subscripts(c.confkey, 1) AS idx\n ) indexed_confkeys\n JOIN pg_attribute a ON a.attrelid = t2.oid\n AND a.attnum = indexed_confkeys.confkey_elem\n ) AS confkey_names\n FROM pg_constraint c\n JOIN pg_class t1 ON c.conrelid = t1.oid\n JOIN pg_class t2 ON c.confrelid = t2.oid\n JOIN pg_namespace n ON c.connamespace = n.oid\n WHERE c.contype = 'f'\n AND t1.relname = \#{scope[:name]}\n AND n.nspname = \#{scope[:schema]}\n ORDER BY c.conname\n SQL\n\n fk_info.map do |row|\n to_table = Utils.extract_schema_qualified_name(row[\"to_table\"]).to_s\n\n column = decode_string_array(row[\"conkey_names\"])\n primary_key = decode_string_array(row[\"confkey_names\"])\n\n options = {\n column: column.size == 1 ? column.first : column,\n name: row[\"name\"],\n primary_key: primary_key.size == 1 ? primary_key.first : primary_key\n }\n\n options[:on_delete] = extract_foreign_key_action(row[\"on_delete\"])\n options[:on_update] = extract_foreign_key_action(row[\"on_update\"])\n options[:deferrable] = extract_constraint_deferrable(row[\"deferrable\"], row[\"deferred\"])\n\n options[:validate] = row[\"valid\"]\n\n ForeignKeyDefinition.new(table_name, to_table, options)\n end\nend\n", "SCHEMA", allow_retry: true, materialize_transactions: false) |
#foreign_table_exists?(table_name) ⇒ Boolean
669 670 671 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 669 def foreign_table_exists?(table_name) query_values(data_source_sql(table_name, type: "FOREIGN TABLE"), "SCHEMA").any? if table_name.present? end |
#foreign_tables ⇒ Object
665 666 667 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 665 def foreign_tables query_values(data_source_sql(type: "FOREIGN TABLE"), "SCHEMA") end |
#index_name(table_name, options) ⇒ Object
:nodoc:
600 601 602 603 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 600 def index_name(table_name, ) # :nodoc: _schema, table_name = extract_schema_qualified_name(table_name.to_s) super end |
#index_name_exists?(table_name, index_name) ⇒ Boolean
Verifies existence of an index with a given name.
73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 73 def index_name_exists?(table_name, index_name) table = quoted_scope(table_name) index = quoted_scope(index_name) query_value(" SELECT COUNT(*)\n FROM pg_class t\n INNER JOIN pg_index d ON t.oid = d.indrelid\n INNER JOIN pg_class i ON d.indexrelid = i.oid\n LEFT JOIN pg_namespace n ON n.oid = t.relnamespace\n WHERE i.relkind IN ('i', 'I')\n AND i.relname = \#{index[:name]}\n AND t.relname = \#{table[:name]}\n AND n.nspname = \#{table[:schema]}\n SQL\nend\n", "SCHEMA").to_i > 0 |
#indexes(table_name) ⇒ Object
Returns an array of indexes for the given table.
91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 91 def indexes(table_name) # :nodoc: scope = quoted_scope(table_name) result = query(" SELECT distinct i.relname, d.indisunique, d.indkey, pg_get_indexdef(d.indexrelid),\n pg_catalog.obj_description(i.oid, 'pg_class') AS comment, d.indisvalid,\n ARRAY(\n SELECT pg_get_indexdef(d.indexrelid, k + 1, true)\n FROM generate_subscripts(d.indkey, 1) AS k\n ORDER BY k\n ) AS columns\n FROM pg_class t\n INNER JOIN pg_index d ON t.oid = d.indrelid\n INNER JOIN pg_class i ON d.indexrelid = i.oid\n LEFT JOIN pg_namespace n ON n.oid = t.relnamespace\n WHERE i.relkind IN ('i', 'I')\n AND d.indisprimary = 'f'\n AND t.relname = \#{scope[:name]}\n AND n.nspname = \#{scope[:schema]}\n ORDER BY i.relname\n SQL\n\n result.map do |row|\n index_name = row[0]\n unique = row[1]\n indkey = row[2].split(\" \").map(&:to_i)\n inddef = row[3]\n comment = row[4]\n valid = row[5]\n columns = decode_string_array(row[6]).map { |c| Utils.unquote_identifier(c.strip.gsub('\"\"', '\"')) }\n\n using, expressions, include, nulls_not_distinct, where = inddef.scan(/ USING (\\w+?) \\((.+?)\\)(?: INCLUDE \\((.+?)\\))?( NULLS NOT DISTINCT)?(?: WHERE (.+))?\\z/m).flatten\n\n orders = {}\n opclasses = {}\n include_columns = include ? include.split(\",\").map { |c| Utils.unquote_identifier(c.strip.gsub('\"\"', '\"')) } : []\n\n if indkey.include?(0)\n columns = expressions\n else\n # prevent INCLUDE columns from being matched\n columns.reject! { |c| include_columns.include?(c) }\n\n # add info on sort order (only desc order is explicitly specified, asc is the default)\n # and non-default opclasses\n expressions.scan(/(?<column>\\w+)\"?\\s?(?<opclass>(?:\\w+\\.)?\\w+_ops(_\\w+)?)?\\s?(?<desc>DESC)?\\s?(?<nulls>NULLS (?:FIRST|LAST))?/).each do |column, opclass, desc, nulls|\n opclasses[column] = opclass.split(\".\").last.to_sym if opclass\n\n if nulls\n orders[column] = [desc, nulls].compact.join(\" \")\n else\n orders[column] = :desc if desc\n end\n end\n end\n\n IndexDefinition.new(\n table_name,\n index_name,\n unique,\n columns,\n orders: orders,\n opclasses: opclasses,\n where: where,\n using: using.to_sym,\n include: include_columns.presence,\n nulls_not_distinct: nulls_not_distinct.present?,\n comment: comment.presence,\n valid: valid\n )\n end\nend\n", "SCHEMA") |
#inherited_table_names(table_name) ⇒ Object
Returns the inherited table name of a given table
214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 214 def inherited_table_names(table_name) # :nodoc: scope = quoted_scope(table_name, type: "BASE TABLE") query_values(" SELECT parent.relname\n FROM pg_catalog.pg_inherits i\n JOIN pg_catalog.pg_class child ON i.inhrelid = child.oid\n JOIN pg_catalog.pg_class parent ON i.inhparent = parent.oid\n LEFT JOIN pg_namespace n ON n.oid = child.relnamespace\n WHERE child.relname = \#{scope[:name]}\n AND child.relkind IN (\#{scope[:type]})\n AND n.nspname = \#{scope[:schema]}\n ORDER BY i.inhseqno\n SQL\nend\n", "SCHEMA") |
#pk_and_sequence_for(table) ⇒ Object
Returns a table's primary key and belonging sequence.
384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 384 def pk_and_sequence_for(table) # :nodoc: # First try looking for a sequence with a dependency on the # given table's primary key. result = query(" SELECT attr.attname, nsp.nspname, seq.relname\n FROM pg_class seq,\n pg_attribute attr,\n pg_depend dep,\n pg_constraint cons,\n pg_namespace nsp\n WHERE seq.oid = dep.objid\n AND seq.relkind = 'S'\n AND attr.attrelid = dep.refobjid\n AND attr.attnum = dep.refobjsubid\n AND attr.attrelid = cons.conrelid\n AND attr.attnum = cons.conkey[1]\n AND seq.relnamespace = nsp.oid\n AND cons.contype = 'p'\n AND dep.classid = 'pg_class'::regclass\n AND dep.refobjid = \#{quote(quote_table_name(table))}::regclass\n SQL\n\n if result.nil? || result.empty?\n result = query(<<~SQL, \"SCHEMA\")[0]\n SELECT attr.attname, nsp.nspname,\n CASE\n WHEN pg_get_expr(def.adbin, def.adrelid) !~* 'nextval' THEN NULL\n WHEN split_part(pg_get_expr(def.adbin, def.adrelid), '''', 2) ~ '.' THEN\n substr(split_part(pg_get_expr(def.adbin, def.adrelid), '''', 2),\n strpos(split_part(pg_get_expr(def.adbin, def.adrelid), '''', 2), '.')+1)\n ELSE split_part(pg_get_expr(def.adbin, def.adrelid), '''', 2)\n END\n FROM pg_class t\n JOIN pg_attribute attr ON (t.oid = attrelid)\n JOIN pg_attrdef def ON (adrelid = attrelid AND adnum = attnum)\n JOIN pg_constraint cons ON (conrelid = adrelid AND adnum = conkey[1])\n JOIN pg_namespace nsp ON (t.relnamespace = nsp.oid)\n WHERE t.oid = \#{quote(quote_table_name(table))}::regclass\n AND cons.contype = 'p'\n AND pg_get_expr(def.adbin, def.adrelid) ~* 'nextval|uuid_generate|gen_random_uuid'\n SQL\n end\n\n pk = result.shift\n if result.last\n [pk, PostgreSQL::Name.new(*result)]\n else\n [pk, nil]\n end\nrescue\n nil\nend\n", "SCHEMA")[0] |
#primary_keys(table_name) ⇒ Object
:nodoc:
437 438 439 440 441 442 443 444 445 446 447 448 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 437 def primary_keys(table_name) # :nodoc: query_values(" SELECT a.attname\n FROM pg_index i\n JOIN pg_attribute a\n ON a.attrelid = i.indrelid\n AND a.attnum = ANY(i.indkey)\n WHERE i.indrelid = \#{quote(quote_table_name(table_name))}::regclass\n AND i.indisprimary\n ORDER BY array_position(i.indkey, a.attnum)\n SQL\nend\n", "SCHEMA") |
#quoted_include_columns_for_index(column_names) ⇒ Object
:nodoc:
988 989 990 991 992 993 994 995 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 988 def quoted_include_columns_for_index(column_names) # :nodoc: return quote_column_name(column_names) if column_names.is_a?(Symbol) quoted_columns = column_names.each_with_object({}) do |name, result| result[name.to_sym] = quote_column_name(name).dup end (quoted_columns).values.join(", ") end |
#recreate_database(name, options = {}) ⇒ Object
Drops the database specified on the name attribute
and creates it again using the provided options.
9 10 11 12 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 9 def recreate_database(name, = {}) # :nodoc: drop_database(name) create_database(name, ) end |
#remove_exclusion_constraint(table_name, expression = nil, **options) ⇒ Object
Removes the given exclusion constraint from the table.
remove_exclusion_constraint :products, name: "price_check"
The expression parameter will be ignored if present. It can be helpful
to provide this in a migration's change method so it can be reverted.
In that case, expression will be used by #add_exclusion_constraint.
812 813 814 815 816 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 812 def remove_exclusion_constraint(table_name, expression = nil, **) excl_name_to_delete = exclusion_constraint_for!(table_name, expression: expression, **).name remove_constraint(table_name, excl_name_to_delete) end |
#remove_index(table_name, column_name = nil, **options) ⇒ Object
:nodoc:
570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 570 def remove_index(table_name, column_name = nil, **) # :nodoc: table = Utils.extract_schema_qualified_name(table_name.to_s) if .key?(:name) provided_index = Utils.extract_schema_qualified_name([:name].to_s) [:name] = provided_index.identifier table = PostgreSQL::Name.new(provided_index.schema, table.identifier) unless table.schema.present? if provided_index.schema.present? && table.schema != provided_index.schema raise ArgumentError.new("Index schema '#{provided_index.schema}' does not match table schema '#{table.schema}'") end end return if [:if_exists] && !index_exists?(table_name, column_name, **) index_to_remove = PostgreSQL::Name.new(table.schema, index_name_for_remove(table.to_s, column_name, )) execute "DROP INDEX #{index_algorithm(options[:algorithm])} #{quote_table_name(index_to_remove)}" end |
#remove_unique_constraint(table_name, column_name = nil, **options) ⇒ Object
Removes the given unique constraint from the table.
remove_unique_constraint :sections, name: "unique_position"
The column_name parameter will be ignored if present. It can be helpful
to provide this in a migration's change method so it can be reverted.
In that case, column_name will be used by #add_unique_constraint.
867 868 869 870 871 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 867 def remove_unique_constraint(table_name, column_name = nil, **) unique_name_to_delete = unique_constraint_for!(table_name, column: column_name, **).name remove_constraint(table_name, unique_name_to_delete) end |
#rename_column(table_name, column_name, new_column_name) ⇒ Object
Renames a column in a table.
545 546 547 548 549 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 545 def rename_column(table_name, column_name, new_column_name) # :nodoc: clear_cache! execute("ALTER TABLE #{quote_table_name(table_name)} #{rename_column_sql(table_name, column_name, new_column_name)}") rename_column_indexes(table_name, column_name, new_column_name) end |
#rename_index(table_name, old_name, new_name) ⇒ Object
Renames an index of a table. Raises error if length of new index name is greater than allowed limit.
593 594 595 596 597 598 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 593 def rename_index(table_name, old_name, new_name) validate_index_length!(table_name, new_name) schema, = extract_schema_qualified_name(table_name) execute "ALTER INDEX #{quote_table_name(schema) + '.' if schema}#{quote_column_name(old_name)} RENAME TO #{quote_table_name(new_name)}" end |
#rename_schema(schema_name, new_name) ⇒ Object
Renames the schema for the given schema name.
293 294 295 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 293 def rename_schema(schema_name, new_name) execute "ALTER SCHEMA #{quote_schema_name(schema_name)} RENAME TO #{quote_schema_name(new_name)}" end |
#rename_table(table_name, new_name, **options) ⇒ Object
Renames a table. Also renames a table's primary key sequence if the sequence name exists and matches the Active Record default.
Example:
rename_table('octopuses', 'octopi')
456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 456 def rename_table(table_name, new_name, **) validate_table_length!(new_name) unless [:_uses_legacy_table_name] clear_cache! schema_cache.clear_data_source_cache!(table_name.to_s) schema_cache.clear_data_source_cache!(new_name.to_s) execute "ALTER TABLE #{quote_table_name(table_name)} RENAME TO #{quote_table_name(new_name)}" pk, seq = pk_and_sequence_for(new_name) if pk # PostgreSQL automatically creates an index for PRIMARY KEY with name consisting of # truncated table name and "_pkey" suffix fitting into max_identifier_length number of characters. max_pkey_prefix = max_identifier_length - "_pkey".size idx = "#{table_name[0, max_pkey_prefix]}_pkey" new_idx = "#{new_name[0, max_pkey_prefix]}_pkey" execute "ALTER INDEX #{quote_table_name(idx)} RENAME TO #{quote_table_name(new_idx)}" # PostgreSQL automatically creates a sequence for PRIMARY KEY with name consisting of # truncated table name and "#{primary_key}_seq" suffix fitting into max_identifier_length number of characters. max_seq_prefix = max_identifier_length - "_#{pk}_seq".size if seq && seq.identifier == "#{table_name[0, max_seq_prefix]}_#{pk}_seq" new_seq = "#{new_name[0, max_seq_prefix]}_#{pk}_seq" execute "ALTER TABLE #{seq.quoted} RENAME TO #{quote_table_name(new_seq)}" end end rename_table_indexes(table_name, new_name, **) end |
#reset_pk_sequence!(table, pk = nil, sequence = nil) ⇒ Object
Resets the sequence of a table's primary key to the maximum value.
356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 356 def reset_pk_sequence!(table, pk = nil, sequence = nil) # :nodoc: unless pk && sequence default_pk, default_sequence = pk_and_sequence_for(table) pk ||= default_pk sequence ||= default_sequence end if @logger && pk && !sequence @logger.warn "#{table} has primary key #{pk} with no default sequence." end if pk && sequence quoted_sequence = quote_table_name(sequence) max_pk = query_value("SELECT MAX(#{quote_column_name pk}) FROM #{quote_table_name(table)}", "SCHEMA") if max_pk.nil? if database_version >= 10_00_00 minvalue = query_value("SELECT seqmin FROM pg_sequence WHERE seqrelid = #{quote(quoted_sequence)}::regclass", "SCHEMA") else minvalue = query_value("SELECT min_value FROM #{quoted_sequence}", "SCHEMA") end end internal_execute("SELECT setval(#{quote(quoted_sequence)}, #{max_pk || minvalue}, #{max_pk ? true : false})", "SCHEMA") end end |
#schema_creation ⇒ Object
:nodoc:
997 998 999 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 997 def schema_creation # :nodoc: PostgreSQL::SchemaCreation.new(self) end |
#schema_exists?(name) ⇒ Boolean
Returns true if schema exists.
68 69 70 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 68 def schema_exists?(name) query_value("SELECT COUNT(*) FROM pg_namespace WHERE nspname = #{quote(name)}", "SCHEMA").to_i > 0 end |
#schema_names ⇒ Object
Returns an array of schema names.
264 265 266 267 268 269 270 271 272 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 264 def schema_names query_values(" SELECT nspname\n FROM pg_namespace\n WHERE nspname !~ '^pg_.*'\n AND nspname NOT IN ('information_schema')\n ORDER by nspname;\n SQL\nend\n", "SCHEMA") |
#schema_search_path ⇒ Object
Returns the active schema search path.
311 312 313 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 311 def schema_search_path @schema_search_path ||= query_value("SHOW search_path", "SCHEMA") end |
#schema_search_path=(schema_csv) ⇒ Object
Sets the schema search path to a string of comma-separated schema names. Names beginning with $ have to be quoted (e.g. $user => '$user'). See: https://www.postgresql.org/docs/current/static/ddl-schemas.html
This should be not be called manually but set in database.yml.
302 303 304 305 306 307 308 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 302 def schema_search_path=(schema_csv) return if schema_csv == @schema_search_path if schema_csv internal_execute("SET search_path TO #{schema_csv}") @schema_search_path = schema_csv end end |
#serial_sequence(table, column) ⇒ Object
336 337 338 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 336 def serial_sequence(table, column) query_value("SELECT pg_get_serial_sequence(#{quote(table)}, #{quote(column)})", "SCHEMA") end |
#set_pk_sequence!(table, value) ⇒ Object
Sets the sequence of a table's primary key to the specified value.
341 342 343 344 345 346 347 348 349 350 351 352 353 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 341 def set_pk_sequence!(table, value) # :nodoc: pk, sequence = pk_and_sequence_for(table) if pk if sequence quoted_sequence = quote_table_name(sequence) internal_execute("SELECT setval(#{quote(quoted_sequence)}, #{value})", "SCHEMA") else @logger.warn "#{table} has primary key #{pk} with no default sequence." if @logger end end end |
#table_comment(table_name) ⇒ Object
Returns a comment stored in database for given table
185 186 187 188 189 190 191 192 193 194 195 196 197 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 185 def table_comment(table_name) # :nodoc: scope = quoted_scope(table_name, type: "BASE TABLE") if scope[:name] query_value(" SELECT pg_catalog.obj_description(c.oid, 'pg_class')\n FROM pg_catalog.pg_class c\n LEFT JOIN pg_namespace n ON n.oid = c.relnamespace\n WHERE c.relname = \#{scope[:name]}\n AND c.relkind IN (\#{scope[:type]})\n AND n.nspname = \#{scope[:schema]}\n SQL\n end\nend\n", "SCHEMA") |
#table_options(table_name) ⇒ Object
:nodoc:
164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 164 def (table_name) # :nodoc: = {} comment = table_comment(table_name) [:comment] = comment if comment inherited_table_names = inherited_table_names(table_name).presence [:options] = "INHERITS (#{inherited_table_names.join(", ")})" if inherited_table_names if ![:options] && supports_native_partitioning? partition_definition = table_partition_definition(table_name) [:options] = "PARTITION BY #{partition_definition}" if partition_definition end end |
#table_partition_definition(table_name) ⇒ Object
Returns the partition definition of a given table
200 201 202 203 204 205 206 207 208 209 210 211 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 200 def table_partition_definition(table_name) # :nodoc: scope = quoted_scope(table_name, type: "BASE TABLE") query_value(" SELECT pg_catalog.pg_get_partkeydef(c.oid)\n FROM pg_catalog.pg_class c\n LEFT JOIN pg_namespace n ON n.oid = c.relnamespace\n WHERE c.relname = \#{scope[:name]}\n AND c.relkind IN (\#{scope[:type]})\n AND n.nspname = \#{scope[:schema]}\n SQL\nend\n", "SCHEMA") |
#type_to_sql(type, limit: nil, precision: nil, scale: nil, array: nil, enum_type: nil) ⇒ Object
Maps logical Rails types to PostgreSQL-specific data types.
874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 874 def type_to_sql(type, limit: nil, precision: nil, scale: nil, array: nil, enum_type: nil, **) # :nodoc: sql = \ case type.to_s when "binary" # PostgreSQL doesn't support limits on binary (bytea) columns. # The hard limit is 1GB, because of a 32-bit size field, and TOAST. case limit when nil, 0..0x3fffffff; super(type) else raise ArgumentError, "No binary type has byte size #{limit}. The limit on binary can be at most 1GB - 1byte." end when "text" # PostgreSQL doesn't support limits on text columns. # The hard limit is 1GB, according to section 8.3 in the manual. case limit when nil, 0..0x3fffffff; super(type) else raise ArgumentError, "No text type has byte size #{limit}. The limit on text can be at most 1GB - 1byte." end when "integer" case limit when 1, 2; "smallint" when nil, 3, 4; "integer" when 5..8; "bigint" else raise ArgumentError, "No integer type has byte size #{limit}. Use a numeric with scale 0 instead." end when "enum" raise ArgumentError, "enum_type is required for enums" if enum_type.nil? enum_type else super end sql = "#{sql}[]" if array && type != :primary_key sql end |
#unique_constraint_options(table_name, column_name, options) ⇒ Object
:nodoc:
848 849 850 851 852 853 854 855 856 857 858 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 848 def (table_name, column_name, ) # :nodoc: assert_valid_deferrable([:deferrable]) if column_name && [:using_index] raise ArgumentError, "Cannot specify both column_name and :using_index options." end = .dup [:name] ||= unique_constraint_name(table_name, column: column_name, **) end |
#unique_constraints(table_name) ⇒ Object
Returns an array of unique constraints for the given table. The unique constraints are represented as UniqueConstraintDefinition objects.
733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 760 761 762 763 764 765 766 767 768 769 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 733 def unique_constraints(table_name) scope = quoted_scope(table_name) unique_info = internal_exec_query(" SELECT c.conname, c.conrelid, c.condeferrable, c.condeferred, pg_get_constraintdef(c.oid) AS constraintdef,\n (\n SELECT array_agg(a.attname ORDER BY idx)\n FROM (\n SELECT idx, c.conkey[idx] AS conkey_elem\n FROM generate_subscripts(c.conkey, 1) AS idx\n ) indexed_conkeys\n JOIN pg_attribute a ON a.attrelid = t.oid\n AND a.attnum = indexed_conkeys.conkey_elem\n ) AS conkey_names\n FROM pg_constraint c\n JOIN pg_class t ON c.conrelid = t.oid\n JOIN pg_namespace n ON n.oid = c.connamespace\n WHERE c.contype = 'u'\n AND t.relname = \#{scope[:name]}\n AND n.nspname = \#{scope[:schema]}\n SQL\n\n unique_info.map do |row|\n columns = decode_string_array(row[\"conkey_names\"])\n\n nulls_not_distinct = row[\"constraintdef\"].start_with?(\"UNIQUE NULLS NOT DISTINCT\")\n deferrable = extract_constraint_deferrable(row[\"condeferrable\"], row[\"condeferred\"])\n\n options = {\n name: row[\"conname\"],\n nulls_not_distinct: nulls_not_distinct,\n deferrable: deferrable\n }\n\n UniqueConstraintDefinition.new(table_name, columns, options)\n end\nend\n", "SCHEMA", allow_retry: true, materialize_transactions: false) |
#update_table_definition(table_name, base) ⇒ Object
:nodoc:
924 925 926 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 924 def update_table_definition(table_name, base) # :nodoc: PostgreSQL::Table.new(table_name, base) end |
#validate_check_constraint(table_name, **options) ⇒ Object
Validates the given check constraint.
validate_check_constraint :products, name: "price_check"
The options hash accepts the same keys as add_check_constraint.
970 971 972 973 974 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 970 def validate_check_constraint(table_name, **) chk_name_to_validate = check_constraint_for!(table_name, **).name validate_constraint table_name, chk_name_to_validate end |
#validate_constraint(table_name, constraint_name) ⇒ Object
Validates the given constraint.
Validates the constraint named constraint_name on accounts.
validate_constraint :accounts, :constraint_name
937 938 939 940 941 942 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 937 def validate_constraint(table_name, constraint_name) at = create_alter_table table_name at.validate_constraint constraint_name execute schema_creation.accept(at) end |
#validate_foreign_key(from_table, to_table = nil, **options) ⇒ Object
Validates the given foreign key.
Validates the foreign key on accounts.branch_id.
validate_foreign_key :accounts, :branches
Validates the foreign key on accounts.owner_id.
validate_foreign_key :accounts, column: :owner_id
Validates the foreign key named special_fk_name on the accounts table.
validate_foreign_key :accounts, name: :special_fk_name
The options hash accepts the same keys as SchemaStatements#add_foreign_key.
959 960 961 962 963 |
# File 'lib/active_record/connection_adapters/postgresql/schema_statements.rb', line 959 def validate_foreign_key(from_table, to_table = nil, **) fk_name_to_validate = foreign_key_for!(from_table, to_table: to_table, **).name validate_constraint from_table, fk_name_to_validate end |