Method: ActiveRecord::ConnectionAdapters::Quoting#cast_bound_value
- Defined in:
- lib/active_record/connection_adapters/abstract/quoting.rb
#cast_bound_value(value) ⇒ Object
Cast a value to be used as a bound parameter of unknown type. For example, MySQL might perform dangerous castings when comparing a string to a number, so this method will cast numbers to string.
113 114 115 |
# File 'lib/active_record/connection_adapters/abstract/quoting.rb', line 113 def cast_bound_value(value) # :nodoc: value end |