Class: ActiveRecord::ConnectionAdapters::Table

Inherits:
Object
  • Object
show all
Defined in:
lib/active_record/connection_adapters/abstract/schema_definitions.rb

Overview

Represents an SQL table in an abstract way for updating a table. Also see TableDefinition and SchemaStatements#create_table

Available transformations are:

change_table :table do |t|
t.column
t.index
t.timestamps
t.change
t.change_default
t.rename
t.references
t.belongs_to
t.string
t.text
t.integer
t.float
t.decimal
t.datetime
t.timestamp
t.time
t.date
t.binary
t.boolean
t.remove
t.remove_references
t.remove_belongs_to
t.remove_index
t.remove_timestamps
end

Instance Method Summary collapse

Constructor Details

#initialize(table_name, base) ⇒ Table

Returns a new instance of Table.



316
317
318
319
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 316

def initialize(table_name, base)
  @table_name = table_name
  @base = base
end

Instance Method Details

#change(column_name, type, options = {}) ⇒ Object

Changes the column's definition according to the new options. See TableDefinition#column for details of the options you can use.

Examples

t.change(:name, :string, :limit => 80) t.change(:description, :text)



366
367
368
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 366

def change(column_name, type, options = {})
  @base.change_column(@table_name, column_name, type, options)
end

#change_default(column_name, default) ⇒ Object

Sets a new default value for a column. See SchemaStatements#change_column_default

Examples

t.change_default(:qualification, 'new') t.change_default(:authorized, 1)



374
375
376
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 374

def change_default(column_name, default)
  @base.change_column_default(@table_name, column_name, default)
end

#column(column_name, type, options = {}) ⇒ Object

Adds a new column to the named table. See TableDefinition#column for details of the options you can use.

Example
Creating a simple column

t.column(:name, :string)



326
327
328
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 326

def column(column_name, type, options = {})
  @base.add_column(@table_name, column_name, type, options)
end

#column_exists?(column_name, type = nil, options = {}) ⇒ Boolean

Checks to see if a column exists. See SchemaStatements#column_exists?

Returns:

  • (Boolean)


331
332
333
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 331

def column_exists?(column_name, type = nil, options = {})
  @base.column_exists?(@table_name, column_name, type, options)
end

#index(column_name, options = {}) ⇒ Object

Adds a new index to the table. column_name can be a single Symbol, or an Array of Symbols. See SchemaStatements#add_index

Examples
Creating a simple index

t.index(:name)

Creating a unique index

t.index([:branch_id, :party_id], :unique => true)

Creating a named index

t.index([:branch_id, :party_id], :unique => true, :name => 'by_branch_party')



345
346
347
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 345

def index(column_name, options = {})
  @base.add_index(@table_name, column_name, options)
end

#index_exists?(column_name, options = {}) ⇒ Boolean

Checks to see if an index exists. See SchemaStatements#index_exists?

Returns:

  • (Boolean)


350
351
352
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 350

def index_exists?(column_name, options = {})
  @base.index_exists?(@table_name, column_name, options)
end

#references(*args) ⇒ Object Also known as: belongs_to

Adds a reference. Optionally adds a type column, if :polymorphic option is provided. references and belongs_to are acceptable.

Examples

t.references(:goat) t.references(:goat, :polymorphic => true) t.belongs_to(:goat)



421
422
423
424
425
426
427
428
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 421

def references(*args)
  options = args.extract_options!
  polymorphic = options.delete(:polymorphic)
  args.each do |col|
    @base.add_column(@table_name, "#{col}_id", :integer, options)
    @base.add_column(@table_name, "#{col}_type", :string, polymorphic.is_a?(Hash) ? polymorphic : options) unless polymorphic.nil?
  end
end

#remove(*column_names) ⇒ Object

Removes the column(s) from the table definition.

Examples

t.remove(:qualification) t.remove(:qualification, :experience)



382
383
384
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 382

def remove(*column_names)
  @base.remove_column(@table_name, column_names)
end

#remove_index(options = {}) ⇒ Object

Removes the given index from the table.

Examples
Remove the index_table_name_on_column in the table_name table

t.remove_index :column

Remove the index named index_table_name_on_branch_id in the table_name table

t.remove_index :column => :branch_id

Remove the index named index_table_name_on_branch_id_and_party_id in the table_name table

t.remove_index :column => [:branch_id, :party_id]

Remove the index named by_branch_party in the table_name table

t.remove_index :name => :by_branch_party



397
398
399
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 397

def remove_index(options = {})
  @base.remove_index(@table_name, options)
end

#remove_references(*args) ⇒ Object Also known as: remove_belongs_to

Removes a reference. Optionally removes a type column. remove_references and remove_belongs_to are acceptable.

Examples

t.remove_references(:goat) t.remove_references(:goat, :polymorphic => true) t.remove_belongs_to(:goat)



437
438
439
440
441
442
443
444
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 437

def remove_references(*args)
  options = args.extract_options!
  polymorphic = options.delete(:polymorphic)
  args.each do |col|
    @base.remove_column(@table_name, "#{col}_id")
    @base.remove_column(@table_name, "#{col}_type") unless polymorphic.nil?
  end
end

#remove_timestampsObject

Removes the timestamp columns (+created_at+ and updated_at) from the table.

Example

t.remove_timestamps



404
405
406
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 404

def remove_timestamps
  @base.remove_timestamps(@table_name)
end

#rename(column_name, new_column_name) ⇒ Object

Renames a column.

Example

t.rename(:description, :name)



411
412
413
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 411

def rename(column_name, new_column_name)
  @base.rename_column(@table_name, column_name, new_column_name)
end

#timestampsObject

Adds timestamps (+created_at+ and updated_at) columns to the table. See SchemaStatements#add_timestamps

Example

t.timestamps



357
358
359
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 357

def timestamps
  @base.add_timestamps(@table_name)
end