Class: ActiveRecord::ConnectionAdapters::Table

Inherits:
Object
  • Object
show all
Defined in:
lib/active_record/connection_adapters/abstract/schema_definitions.rb

Overview

Represents a SQL table in an abstract way for updating a table. Also see TableDefinition and SchemaStatements#create_table

Available transformations are:

change_table :table do |t|
t.column
t.index
t.timestamps
t.change
t.change_default
t.rename
t.references
t.belongs_to
t.string
t.text
t.integer
t.float
t.decimal
t.datetime
t.timestamp
t.time
t.date
t.binary
t.boolean
t.remove
t.remove_references
t.remove_belongs_to
t.remove_index
t.remove_timestamps
end

Instance Method Summary collapse

Constructor Details

#initialize(table_name, base) ⇒ Table

Returns a new instance of Table.



566
567
568
569
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 566

def initialize(table_name, base)
  @table_name = table_name
  @base = base
end

Instance Method Details

#change(column_name, type, options = {}) ⇒ Object

Changes the column's definition according to the new options. See TableDefinition#column for details of the options you can use.

Examples

t.change(:name, :string, :limit => 80) t.change(:description, :text)



606
607
608
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 606

def change(column_name, type, options = {})
  @base.change_column(@table_name, column_name, type, options)
end

#change_default(column_name, default) ⇒ Object

Sets a new default value for a column. See SchemaStatements#change_column_default

Examples

t.change_default(:qualification, 'new') t.change_default(:authorized, 1)



614
615
616
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 614

def change_default(column_name, default)
  @base.change_column_default(@table_name, column_name, default)
end

#column(column_name, type, options = {}) ⇒ Object

Adds a new column to the named table. See TableDefinition#column for details of the options you can use.

Example
Creating a simple column

t.column(:name, :string)



576
577
578
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 576

def column(column_name, type, options = {})
  @base.add_column(@table_name, column_name, type, options)
end

#index(column_name, options = {}) ⇒ Object

Adds a new index to the table. column_name can be a single Symbol, or an Array of Symbols. See SchemaStatements#add_index

Examples
Creating a simple index

t.index(:name)

Creating a unique index

t.index([:branch_id, :party_id], :unique => true)

Creating a named index

t.index([:branch_id, :party_id], :unique => true, :name => 'by_branch_party')



590
591
592
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 590

def index(column_name, options = {})
  @base.add_index(@table_name, column_name, options)
end

#references(*args) ⇒ Object Also known as: belongs_to

Adds a reference. Optionally adds a type column. references and belongs_to are acceptable.

Examples

t.references(:goat) t.references(:goat, :polymorphic => true) t.belongs_to(:goat)



661
662
663
664
665
666
667
668
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 661

def references(*args)
  options = args.extract_options!
  polymorphic = options.delete(:polymorphic)
  args.each do |col|
    @base.add_column(@table_name, "#{col}_id", :integer, options)
    @base.add_column(@table_name, "#{col}_type", :string, polymorphic.is_a?(Hash) ? polymorphic : options) unless polymorphic.nil?
  end
end

#remove(*column_names) ⇒ Object

Removes the column(s) from the table definition.

Examples

t.remove(:qualification) t.remove(:qualification, :experience)



622
623
624
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 622

def remove(*column_names)
  @base.remove_column(@table_name, column_names)
end

#remove_index(options = {}) ⇒ Object

Removes the given index from the table.

Examples
Remove the suppliers_name_index in the suppliers table

t.remove_index :name

Remove the index named accounts_branch_id_index in the accounts table

t.remove_index :column => :branch_id

Remove the index named accounts_branch_id_party_id_index in the accounts table

t.remove_index :column => [:branch_id, :party_id]

Remove the index named by_branch_party in the accounts table

t.remove_index :name => :by_branch_party



637
638
639
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 637

def remove_index(options = {})
  @base.remove_index(@table_name, options)
end

#remove_references(*args) ⇒ Object Also known as: remove_belongs_to

Removes a reference. Optionally removes a type column. remove_references and remove_belongs_to are acceptable.

Examples

t.remove_references(:goat) t.remove_references(:goat, :polymorphic => true) t.remove_belongs_to(:goat)



677
678
679
680
681
682
683
684
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 677

def remove_references(*args)
  options = args.extract_options!
  polymorphic = options.delete(:polymorphic)
  args.each do |col|
    @base.remove_column(@table_name, "#{col}_id")
    @base.remove_column(@table_name, "#{col}_type") unless polymorphic.nil?
  end
end

#remove_timestampsObject

Removes the timestamp columns (created_at and updated_at) from the table.

Example

t.remove_timestamps



644
645
646
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 644

def remove_timestamps
  @base.remove_timestamps(@table_name)
end

#rename(column_name, new_column_name) ⇒ Object

Renames a column.

Example

t.rename(:description, :name)



651
652
653
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 651

def rename(column_name, new_column_name)
  @base.rename_column(@table_name, column_name, new_column_name)
end

#timestampsObject

Adds timestamps (created_at and updated_at) columns to the table. See SchemaStatements#add_timestamps

Example

t.timestamps



597
598
599
# File 'lib/active_record/connection_adapters/abstract/schema_definitions.rb', line 597

def timestamps
  @base.add_timestamps(@table_name)
end