Method: ActiveRecord::ConnectionHandling#while_preventing_writes
- Defined in:
- lib/active_record/connection_handling.rb
#while_preventing_writes(enabled = true, &block) ⇒ Object
Prevent writing to the database regardless of role.
In some cases you may want to prevent writes to the database even if you are on a database that can write. while_preventing_writes will prevent writes to the database for the duration of the block.
This method does not provide the same protection as a readonly user and is meant to be a safeguard against accidental writes.
See READ_QUERY for the queries that are blocked by this method.
238 239 240 |
# File 'lib/active_record/connection_handling.rb', line 238 def while_preventing_writes(enabled = true, &block) connected_to(role: current_role, prevent_writes: enabled, &block) end |