Method: ActiveRecord::DatabaseConfigurations::HashConfig#schema_cache_path
- Defined in:
- lib/active_record/database_configurations/hash_config.rb
#schema_cache_path ⇒ Object
The path to the schema cache dump file for a database. If omitted, the filename will be read from ENV or a default will be derived.
103 104 105 |
# File 'lib/active_record/database_configurations/hash_config.rb', line 103 def schema_cache_path configuration_hash[:schema_cache_path] end |