Method: ActiveRecord::Persistence#increment
- Defined in:
- lib/active_record/persistence.rb
#increment(attribute, by = 1) ⇒ Object
Initializes attribute to zero if nil and adds the value passed as by (default is 1). The increment is performed directly on the underlying attribute, no setter is invoked. Only makes sense for number-based attributes. Returns self.
656 657 658 659 660 |
# File 'lib/active_record/persistence.rb', line 656 def increment(attribute, by = 1) self[attribute] ||= 0 self[attribute] += by self end |