Method: ActiveRecord::Persistence#update_attribute
- Defined in:
- lib/active_record/persistence.rb
#update_attribute(name, value) ⇒ Object
Updates a single attribute and saves the record. This is especially useful for boolean flags on existing records. Also note that
-
Validation is skipped.
-
Callbacks are invoked.
-
updated_at/updated_on column is updated if that column is available.
-
Updates all the attributes that are dirty in this object.
This method raises an ActiveRecord::ActiveRecordError if the attribute is marked as readonly.
216 217 218 219 220 221 |
# File 'lib/active_record/persistence.rb', line 216 def update_attribute(name, value) name = name.to_s verify_readonly_attribute(name) send("#{name}=", value) save(validate: false) end |