Module: ActiveRecord::QueryMethods
- Includes:
- ActiveModel::ForbiddenAttributesProtection
- Included in:
- Relation
- Defined in:
- lib/active_record/relation/query_methods.rb
Defined Under Namespace
Classes: CTEJoin, WhereChain
Constant Summary collapse
- FROZEN_EMPTY_ARRAY =
[].freeze
- FROZEN_EMPTY_HASH =
{}.freeze
- VALID_UNSCOPING_VALUES =
Set.new([:where, :select, :group, :order, :lock, :limit, :offset, :joins, :left_outer_joins, :annotate, :includes, :eager_load, :preload, :from, :readonly, :having, :optimizer_hints, :with])
Instance Method Summary collapse
-
#_select!(*fields) ⇒ Object
:nodoc:.
-
#and(other) ⇒ Object
Returns a new relation, which is the logical intersection of this relation and the one passed as an argument.
-
#and!(other) ⇒ Object
:nodoc:.
-
#annotate(*args) ⇒ Object
Adds an SQL comment to queries generated from this relation.
-
#annotate!(*args) ⇒ Object
Like #annotate, but modifies relation in place.
-
#arel(aliases = nil) ⇒ Object
Returns the Arel object associated with the relation.
-
#construct_join_dependency(associations, join_type) ⇒ Object
:nodoc:.
-
#create_with(value) ⇒ Object
Sets attributes to be used when creating new records from a relation object.
-
#create_with!(value) ⇒ Object
:nodoc:.
-
#distinct(value = true) ⇒ Object
Specifies whether the records should be unique or not.
-
#distinct!(value = true) ⇒ Object
Like #distinct, but modifies relation in place.
-
#eager_load(*args) ⇒ Object
Specify associations
argsto be eager loaded using aLEFT OUTER JOIN. -
#eager_load!(*args) ⇒ Object
:nodoc:.
-
#excluding(*records) ⇒ Object
(also: #without)
Excludes the specified record (or collection of records) from the resulting relation.
-
#excluding!(records) ⇒ Object
:nodoc:.
-
#extending(*modules, &block) ⇒ Object
Used to extend a scope with additional methods, either through a module or through a block provided.
-
#extending!(*modules, &block) ⇒ Object
:nodoc:.
-
#extract_associated(association) ⇒ Object
Extracts a named
associationfrom the relation. -
#from(value, subquery_name = nil) ⇒ Object
Specifies the table from which the records will be fetched.
-
#from!(value, subquery_name = nil) ⇒ Object
:nodoc:.
-
#group(*args) ⇒ Object
Allows to specify a group attribute:.
-
#group!(*args) ⇒ Object
:nodoc:.
-
#having(opts, *rest) ⇒ Object
Allows to specify a HAVING clause.
-
#having!(opts, *rest) ⇒ Object
:nodoc:.
-
#in_order_of(column, values) ⇒ Object
Applies an
ORDER BYclause based on a givencolumn, ordered and filtered by a specific set ofvalues. -
#includes(*args) ⇒ Object
Specify associations
argsto be eager loaded to prevent N + 1 queries. -
#includes!(*args) ⇒ Object
:nodoc:.
-
#invert_where ⇒ Object
Allows you to invert an entire where clause instead of manually applying conditions.
-
#invert_where! ⇒ Object
:nodoc:.
-
#joins(*args) ⇒ Object
Performs JOINs on
args. -
#joins!(*args) ⇒ Object
:nodoc:.
-
#left_outer_joins(*args) ⇒ Object
(also: #left_joins)
Performs LEFT OUTER JOINs on
args:. -
#left_outer_joins!(*args) ⇒ Object
:nodoc:.
-
#limit(value) ⇒ Object
Specifies a limit for the number of records to retrieve.
-
#limit!(value) ⇒ Object
:nodoc:.
-
#lock(locks = true) ⇒ Object
Specifies locking settings (default to
true). -
#lock!(locks = true) ⇒ Object
:nodoc:.
-
#none ⇒ Object
Returns a chainable relation with zero records.
-
#none! ⇒ Object
:nodoc:.
-
#null_relation? ⇒ Boolean
:nodoc:.
-
#offset(value) ⇒ Object
Specifies the number of rows to skip before returning rows.
-
#offset!(value) ⇒ Object
:nodoc:.
-
#optimizer_hints(*args) ⇒ Object
Specify optimizer hints to be used in the SELECT statement.
-
#optimizer_hints!(*args) ⇒ Object
:nodoc:.
-
#or(other) ⇒ Object
Returns a new relation, which is the logical union of this relation and the one passed as an argument.
-
#or!(other) ⇒ Object
:nodoc:.
-
#order(*args) ⇒ Object
Applies an
ORDER BYclause to a query. -
#order!(*args) ⇒ Object
Same as #order but operates on relation in-place instead of copying.
-
#preload(*args) ⇒ Object
Specify associations
argsto be eager loaded using separate queries. -
#preload!(*args) ⇒ Object
:nodoc:.
-
#readonly(value = true) ⇒ Object
Mark a relation as readonly.
-
#readonly!(value = true) ⇒ Object
:nodoc:.
-
#references(*table_names) ⇒ Object
Use to indicate that the given
table_namesare referenced by an SQL string, and should therefore be JOINed in any query rather than loaded separately. -
#references!(*table_names) ⇒ Object
:nodoc:.
-
#regroup(*args) ⇒ Object
Allows you to change a previously set group statement.
-
#regroup!(*args) ⇒ Object
Same as #regroup but operates on relation in-place instead of copying.
-
#reorder(*args) ⇒ Object
Replaces any existing order defined on the relation with the specified order.
-
#reorder!(*args) ⇒ Object
Same as #reorder but operates on relation in-place instead of copying.
-
#reselect(*args) ⇒ Object
Allows you to change a previously set select statement.
-
#reselect!(*args) ⇒ Object
Same as #reselect but operates on relation in-place instead of copying.
-
#reverse_order ⇒ Object
Reverse the existing order clause on the relation.
-
#reverse_order! ⇒ Object
:nodoc:.
-
#rewhere(conditions) ⇒ Object
Allows you to change a previously set where condition for a given attribute, instead of appending to that condition.
-
#select(*fields) ⇒ Object
Works in two unique ways.
-
#skip_preloading! ⇒ Object
:nodoc:.
-
#skip_query_cache!(value = true) ⇒ Object
:nodoc:.
-
#strict_loading(value = true) ⇒ Object
Sets the returned relation to strict_loading mode.
-
#strict_loading!(value = true) ⇒ Object
:nodoc:.
-
#structurally_compatible?(other) ⇒ Boolean
Checks whether the given relation is structurally compatible with this relation, to determine if it’s possible to use the #and and #or methods without raising an error.
-
#uniq!(name) ⇒ Object
Deduplicate multiple values.
-
#unscope(*args) ⇒ Object
Removes an unwanted relation that is already defined on a chain of relations.
-
#unscope!(*args) ⇒ Object
:nodoc:.
-
#where(*args) ⇒ Object
Returns a new relation, which is the result of filtering the current relation according to the conditions in the arguments.
-
#where!(opts, *rest) ⇒ Object
:nodoc:.
-
#with(*args) ⇒ Object
Add a Common Table Expression (CTE) that you can then reference within another SELECT statement.
-
#with!(*args) ⇒ Object
Like #with, but modifies relation in place.
-
#with_recursive(*args) ⇒ Object
Add a recursive Common Table Expression (CTE) that you can then reference within another SELECT statement.
-
#with_recursive!(*args) ⇒ Object
Like #with_recursive but modifies the relation in place.
Instance Method Details
#_select!(*fields) ⇒ Object
:nodoc:
421 422 423 424 |
# File 'lib/active_record/relation/query_methods.rb', line 421 def _select!(*fields) # :nodoc: self.select_values |= fields self end |
#and(other) ⇒ Object
Returns a new relation, which is the logical intersection of this relation and the one passed as an argument.
The two relations must be structurally compatible: they must be scoping the same model, and they must differ only by #where (if no #group has been defined) or #having (if a #group is present).
Post.where(id: [1, 2]).and(Post.where(id: [2, 3]))
# SELECT `posts`.* FROM `posts` WHERE `posts`.`id` IN (1, 2) AND `posts`.`id` IN (2, 3)
1115 1116 1117 1118 1119 1120 1121 |
# File 'lib/active_record/relation/query_methods.rb', line 1115 def and(other) if other.is_a?(Relation) spawn.and!(other) else raise ArgumentError, "You have passed #{other.class.name} object to #and. Pass an ActiveRecord::Relation object instead." end end |
#and!(other) ⇒ Object
:nodoc:
1123 1124 1125 1126 1127 1128 1129 1130 1131 1132 1133 1134 1135 |
# File 'lib/active_record/relation/query_methods.rb', line 1123 def and!(other) # :nodoc: incompatible_values = structurally_incompatible_values_for(other) unless incompatible_values.empty? raise ArgumentError, "Relation passed to #and must be structurally compatible. Incompatible values: #{incompatible_values}" end self.where_clause |= other.where_clause self.having_clause |= other.having_clause self.references_values |= other.references_values self end |
#annotate(*args) ⇒ Object
Adds an SQL comment to queries generated from this relation. For example:
User.annotate("selecting user names").select(:name)
# SELECT "users"."name" FROM "users" /* selecting user names */
User.annotate("selecting", "user", "names").select(:name)
# SELECT "users"."name" FROM "users" /* selecting */ /* user */ /* names */
The SQL block comment delimiters, “/*” and “*/”, will be added automatically.
Some escaping is performed, however untrusted user input should not be used.
1509 1510 1511 1512 |
# File 'lib/active_record/relation/query_methods.rb', line 1509 def annotate(*args) check_if_method_has_arguments!(__callee__, args) spawn.annotate!(*args) end |
#annotate!(*args) ⇒ Object
Like #annotate, but modifies relation in place.
1515 1516 1517 1518 |
# File 'lib/active_record/relation/query_methods.rb', line 1515 def annotate!(*args) # :nodoc: self.annotate_values += args self end |
#arel(aliases = nil) ⇒ Object
Returns the Arel object associated with the relation.
1574 1575 1576 |
# File 'lib/active_record/relation/query_methods.rb', line 1574 def arel(aliases = nil) # :nodoc: @arel ||= with_connection { |c| build_arel(c, aliases) } end |
#construct_join_dependency(associations, join_type) ⇒ Object
:nodoc:
1578 1579 1580 1581 1582 |
# File 'lib/active_record/relation/query_methods.rb', line 1578 def construct_join_dependency(associations, join_type) # :nodoc: ActiveRecord::Associations::JoinDependency.new( klass, table, associations, join_type ) end |
#create_with(value) ⇒ Object
Sets attributes to be used when creating new records from a relation object.
users = User.where(name: 'Oscar')
users.new.name # => 'Oscar'
users = users.create_with(name: 'DHH')
users.new.name # => 'DHH'
You can pass nil to #create_with to reset attributes:
users = users.create_with(nil)
users.new.name # => 'Oscar'
1326 1327 1328 |
# File 'lib/active_record/relation/query_methods.rb', line 1326 def create_with(value) spawn.create_with!(value) end |
#create_with!(value) ⇒ Object
:nodoc:
1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 |
# File 'lib/active_record/relation/query_methods.rb', line 1330 def create_with!(value) # :nodoc: if value value = sanitize_forbidden_attributes(value) self.create_with_value = create_with_value.merge(value) else self.create_with_value = FROZEN_EMPTY_HASH end self end |
#distinct(value = true) ⇒ Object
Specifies whether the records should be unique or not. For example:
User.select(:name)
# Might return two records with the same name
User.select(:name).distinct
# Returns 1 record per distinct name
User.select(:name).distinct.distinct(false)
# You can also remove the uniqueness
1390 1391 1392 |
# File 'lib/active_record/relation/query_methods.rb', line 1390 def distinct(value = true) spawn.distinct!(value) end |
#distinct!(value = true) ⇒ Object
Like #distinct, but modifies relation in place.
1395 1396 1397 1398 |
# File 'lib/active_record/relation/query_methods.rb', line 1395 def distinct!(value = true) # :nodoc: self.distinct_value = value self end |
#eager_load(*args) ⇒ Object
Specify associations args to be eager loaded using a LEFT OUTER JOIN. Performs a single query joining all specified associations. For example:
users = User.eager_load(:address).limit(5)
users.each do |user|
user.address.city
end
# SELECT "users"."id" AS t0_r0, "users"."name" AS t0_r1, ... FROM "users"
# LEFT OUTER JOIN "addresses" ON "addresses"."id" = "users"."address_id"
# LIMIT 5
Instead of loading the 5 addresses with 5 separate queries, all addresses are loaded with a single joined query.
Loading multiple and nested associations is possible using Hashes and Arrays, similar to #includes:
User.eager_load(:address, friends: [:address, :followers])
# SELECT "users"."id" AS t0_r0, "users"."name" AS t0_r1, ... FROM "users"
# LEFT OUTER JOIN "addresses" ON "addresses"."id" = "users"."address_id"
# LEFT OUTER JOIN "friends" ON "friends"."user_id" = "users"."id"
# ...
NOTE: Loading the associations in a join can result in many rows that contain redundant data and it performs poorly at scale.
283 284 285 286 |
# File 'lib/active_record/relation/query_methods.rb', line 283 def eager_load(*args) check_if_method_has_arguments!(__callee__, args) spawn.eager_load!(*args) end |
#eager_load!(*args) ⇒ Object
:nodoc:
288 289 290 291 |
# File 'lib/active_record/relation/query_methods.rb', line 288 def eager_load!(*args) # :nodoc: self.eager_load_values |= args self end |
#excluding(*records) ⇒ Object Also known as: without
Excludes the specified record (or collection of records) from the resulting relation. For example:
Post.excluding(post)
# SELECT "posts".* FROM "posts" WHERE "posts"."id" != 1
Post.excluding(post_one, post_two)
# SELECT "posts".* FROM "posts" WHERE "posts"."id" NOT IN (1, 2)
Post.excluding(Post.drafts)
# SELECT "posts".* FROM "posts" WHERE "posts"."id" NOT IN (3, 4, 5)
This can also be called on associations. As with the above example, either a single record of collection thereof may be specified:
post = Post.find(1)
comment = Comment.find(2)
post.comments.excluding(comment)
# SELECT "comments".* FROM "comments" WHERE "comments"."post_id" = 1 AND "comments"."id" != 2
This is short-hand for .where.not(id: post.id) and .where.not(id: [post_one.id, post_two.id]).
An ArgumentError will be raised if either no records are specified, or if any of the records in the collection (if a collection is passed in) are not instances of the same model that the relation is scoping.
1554 1555 1556 1557 1558 1559 1560 1561 1562 1563 1564 |
# File 'lib/active_record/relation/query_methods.rb', line 1554 def excluding(*records) relations = records.extract! { |element| element.is_a?(Relation) } records.flatten!(1) records.compact! unless records.all?(klass) && relations.all? { |relation| relation.klass == klass } raise ArgumentError, "You must only pass a single or collection of #{klass.name} objects to ##{__callee__}." end spawn.excluding!(records + relations.flat_map(&:ids)) end |
#excluding!(records) ⇒ Object
:nodoc:
1567 1568 1569 1570 1571 |
# File 'lib/active_record/relation/query_methods.rb', line 1567 def excluding!(records) # :nodoc: predicates = [ predicate_builder[primary_key, records].invert ] self.where_clause += Relation::WhereClause.new(predicates) self end |
#extending(*modules, &block) ⇒ Object
Used to extend a scope with additional methods, either through a module or through a block provided.
The object returned is a relation, which can be further extended.
Using a Module
module Pagination
def page(number)
# pagination code goes here
end
end
scope = Model.all.extending(Pagination)
scope.page(params[:page])
You can also pass a list of modules:
scope = Model.all.extending(Pagination, SomethingElse)
Using a Block
scope = Model.all.extending do
def page(number)
# pagination code goes here
end
end
scope.page(params[:page])
You can also use a block and a module list:
scope = Model.all.extending(Pagination) do
def per_page(number)
# pagination code goes here
end
end
1436 1437 1438 1439 1440 1441 1442 |
# File 'lib/active_record/relation/query_methods.rb', line 1436 def extending(*modules, &block) if modules.any? || block spawn.extending!(*modules, &block) else self end end |
#extending!(*modules, &block) ⇒ Object
:nodoc:
1444 1445 1446 1447 1448 1449 1450 1451 1452 |
# File 'lib/active_record/relation/query_methods.rb', line 1444 def extending!(*modules, &block) # :nodoc: modules << Module.new(&block) if block modules.flatten! self.extending_values += modules extend(*extending_values) if extending_values.any? self end |
#extract_associated(association) ⇒ Object
Extracts a named association from the relation. The named association is first preloaded, then the individual association records are collected from the relation. Like so:
account.memberships.extract_associated(:user)
# => Returns collection of User records
This is short-hand for:
account.memberships.preload(:user).collect(&:user)
334 335 336 |
# File 'lib/active_record/relation/query_methods.rb', line 334 def extract_associated(association) preload(association).collect(&association) end |
#from(value, subquery_name = nil) ⇒ Object
Specifies the table from which the records will be fetched. For example:
Topic.select('title').from('posts')
# SELECT title FROM posts
Can accept other relation objects. For example:
Topic.select('title').from(Topic.approved)
# SELECT title FROM (SELECT * FROM topics WHERE approved = 't') subquery
Passing a second argument (string or symbol), creates the alias for the SQL from clause. Otherwise the alias “subquery” is used:
Topic.select('a.title').from(Topic.approved, :a)
# SELECT a.title FROM (SELECT * FROM topics WHERE approved = 't') a
It does not add multiple arguments to the SQL from clause. The last from chained is the one used:
Topic.select('title').from(Topic.approved).from(Topic.inactive)
# SELECT title FROM (SELECT topics.* FROM topics WHERE topics.active = 'f') subquery
For multiple arguments for the SQL from clause, you can pass a string with the exact elements in the SQL from list:
color = "red"
Color
.from("colors c, JSONB_ARRAY_ELEMENTS(colored_things) AS colorvalues(colorvalue)")
.where("colorvalue->>'color' = ?", color)
.select("c.*").to_a
# SELECT c.*
# FROM colors c, JSONB_ARRAY_ELEMENTS(colored_things) AS colorvalues(colorvalue)
# WHERE (colorvalue->>'color' = 'red')
1371 1372 1373 |
# File 'lib/active_record/relation/query_methods.rb', line 1371 def from(value, subquery_name = nil) spawn.from!(value, subquery_name) end |
#from!(value, subquery_name = nil) ⇒ Object
:nodoc:
1375 1376 1377 1378 |
# File 'lib/active_record/relation/query_methods.rb', line 1375 def from!(value, subquery_name = nil) # :nodoc: self.from_clause = Relation::FromClause.new(value, subquery_name) self end |
#group(*args) ⇒ Object
Allows to specify a group attribute:
User.group(:name)
# SELECT "users".* FROM "users" GROUP BY name
Returns an array with distinct records based on the group attribute:
User.select([:id, :name])
# => [#<User id: 1, name: "Oscar">, #<User id: 2, name: "Oscar">, #<User id: 3, name: "Foo">]
User.group(:name)
# => [#<User id: 3, name: "Foo", ...>, #<User id: 2, name: "Oscar", ...>]
User.group('name AS grouped_name, age')
# => [#<User id: 3, name: "Foo", age: 21, ...>, #<User id: 2, name: "Oscar", age: 21, ...>, #<User id: 5, name: "Foo", age: 23, ...>]
Passing in an array of attributes to group by is also supported.
User.select([:id, :first_name]).group(:id, :first_name).first(3)
# => [#<User id: 1, first_name: "Bill">, #<User id: 2, first_name: "Earl">, #<User id: 3, first_name: "Beto">]
566 567 568 569 |
# File 'lib/active_record/relation/query_methods.rb', line 566 def group(*args) check_if_method_has_arguments!(__callee__, args) spawn.group!(*args) end |
#group!(*args) ⇒ Object
:nodoc:
571 572 573 574 |
# File 'lib/active_record/relation/query_methods.rb', line 571 def group!(*args) # :nodoc: self.group_values += args self end |
#having(opts, *rest) ⇒ Object
Allows to specify a HAVING clause. Note that you can’t use HAVING without also specifying a GROUP clause.
Order.having('SUM(price) > 30').group('user_id')
1177 1178 1179 |
# File 'lib/active_record/relation/query_methods.rb', line 1177 def having(opts, *rest) opts.blank? ? self : spawn.having!(opts, *rest) end |
#having!(opts, *rest) ⇒ Object
:nodoc:
1181 1182 1183 1184 |
# File 'lib/active_record/relation/query_methods.rb', line 1181 def having!(opts, *rest) # :nodoc: self.having_clause += build_having_clause(opts, rest) self end |
#in_order_of(column, values) ⇒ Object
Applies an ORDER BY clause based on a given column, ordered and filtered by a specific set of values.
User.in_order_of(:id, [1, 5, 3])
# SELECT "users".* FROM "users"
# WHERE "users"."id" IN (1, 5, 3)
# ORDER BY CASE
# WHEN "users"."id" = 1 THEN 1
# WHEN "users"."id" = 5 THEN 2
# WHEN "users"."id" = 3 THEN 3
# END ASC
column can point to an enum column; the actual query generated may be different depending on the database adapter and the column definition.
class Conversation < ActiveRecord::Base
enum :status, [ :active, :archived ]
end
Conversation.in_order_of(:status, [:archived, :active])
# SELECT "conversations".* FROM "conversations"
# WHERE "conversations"."status" IN (1, 0)
# ORDER BY CASE
# WHEN "conversations"."status" = 1 THEN 1
# WHEN "conversations"."status" = 0 THEN 2
# END ASC
values can also include nil.
Conversation.in_order_of(:status, [nil, :archived, :active])
# SELECT "conversations".* FROM "conversations"
# WHERE ("conversations"."status" IN (1, 0) OR "conversations"."status" IS NULL)
# ORDER BY CASE
# WHEN "conversations"."status" IS NULL THEN 1
# WHEN "conversations"."status" = 1 THEN 2
# WHEN "conversations"."status" = 0 THEN 3
# END ASC
701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 |
# File 'lib/active_record/relation/query_methods.rb', line 701 def in_order_of(column, values) klass.disallow_raw_sql!([column], permit: model.adapter_class.column_name_with_order_matcher) return spawn.none! if values.empty? references = column_references([column]) self.references_values |= references unless references.empty? values = values.map { |value| type_caster.type_cast_for_database(column, value) } arel_column = column.is_a?(Arel::Nodes::SqlLiteral) ? column : order_column(column.to_s) where_clause = if values.include?(nil) arel_column.in(values.compact).or(arel_column.eq(nil)) else arel_column.in(values) end spawn .order!(build_case_for_value_position(arel_column, values)) .where!(where_clause) end |
#includes(*args) ⇒ Object
Specify associations args to be eager loaded to prevent N + 1 queries. A separate query is performed for each association, unless a join is required by conditions.
For example:
users = User.includes(:address).limit(5)
users.each do |user|
user.address.city
end
# SELECT "users".* FROM "users" LIMIT 5
# SELECT "addresses".* FROM "addresses" WHERE "addresses"."id" IN (1,2,3,4,5)
Instead of loading the 5 addresses with 5 separate queries, all addresses are loaded with a single query.
Loading the associations in a separate query will often result in a performance improvement over a simple join, as a join can result in many rows that contain redundant data and it performs poorly at scale.
You can also specify multiple associations. Each association will result in an additional query:
User.includes(:address, :friends).to_a
# SELECT "users".* FROM "users"
# SELECT "addresses".* FROM "addresses" WHERE "addresses"."id" IN (1,2,3,4,5)
# SELECT "friends".* FROM "friends" WHERE "friends"."user_id" IN (1,2,3,4,5)
Loading nested associations is possible using a Hash:
User.includes(:address, friends: [:address, :followers])
Conditions
If you want to add string conditions to your included models, you’ll have to explicitly reference them. For example:
User.includes(:posts).where('posts.name = ?', 'example').to_a
Will throw an error, but this will work:
User.includes(:posts).where('posts.name = ?', 'example').references(:posts).to_a
# SELECT "users"."id" AS t0_r0, ... FROM "users"
# LEFT OUTER JOIN "posts" ON "posts"."user_id" = "users"."id"
# WHERE "posts"."name" = ? [["name", "example"]]
As the LEFT OUTER JOIN already contains the posts, the second query for the posts is no longer performed.
Note that #includes works with association names while #references needs the actual table name.
If you pass the conditions via a Hash, you don’t need to call #references explicitly, as #where references the tables for you. For example, this will work correctly:
User.includes(:posts).where(posts: { name: 'example' })
NOTE: Conditions affect both sides of an association. For example, the above code will return only users that have a post named “example”, and will only include posts named “example”, even when a matching user has other additional posts.
247 248 249 250 |
# File 'lib/active_record/relation/query_methods.rb', line 247 def includes(*args) check_if_method_has_arguments!(__callee__, args) spawn.includes!(*args) end |
#includes!(*args) ⇒ Object
:nodoc:
252 253 254 255 |
# File 'lib/active_record/relation/query_methods.rb', line 252 def includes!(*args) # :nodoc: self.includes_values |= args self end |
#invert_where ⇒ Object
Allows you to invert an entire where clause instead of manually applying conditions.
class User
scope :active, -> { where(accepted: true, locked: false) }
end
User.where(accepted: true)
# WHERE `accepted` = 1
User.where(accepted: true).invert_where
# WHERE `accepted` != 1
User.active
# WHERE `accepted` = 1 AND `locked` = 0
User.active.invert_where
# WHERE NOT (`accepted` = 1 AND `locked` = 0)
Be careful because this inverts all conditions before invert_where call.
class User
scope :active, -> { where(accepted: true, locked: false) }
scope :inactive, -> { active.invert_where } # Do not attempt it
end
# It also inverts `where(role: 'admin')` unexpectedly.
User.where(role: 'admin').inactive
# WHERE NOT (`role` = 'admin' AND `accepted` = 1 AND `locked` = 0)
1081 1082 1083 |
# File 'lib/active_record/relation/query_methods.rb', line 1081 def invert_where spawn.invert_where! end |
#invert_where! ⇒ Object
:nodoc:
1085 1086 1087 1088 |
# File 'lib/active_record/relation/query_methods.rb', line 1085 def invert_where! # :nodoc: self.where_clause = where_clause.invert self end |
#joins(*args) ⇒ Object
Performs JOINs on args. The given symbol(s) should match the name of the association(s).
User.joins(:posts)
# SELECT "users".*
# FROM "users"
# INNER JOIN "posts" ON "posts"."user_id" = "users"."id"
Multiple joins:
User.joins(:posts, :account)
# SELECT "users".*
# FROM "users"
# INNER JOIN "posts" ON "posts"."user_id" = "users"."id"
# INNER JOIN "accounts" ON "accounts"."id" = "users"."account_id"
Nested joins:
User.joins(posts: [:comments])
# SELECT "users".*
# FROM "users"
# INNER JOIN "posts" ON "posts"."user_id" = "users"."id"
# INNER JOIN "comments" ON "comments"."post_id" = "posts"."id"
You can use strings in order to customize your joins:
User.joins("LEFT JOIN bookmarks ON bookmarks.bookmarkable_type = 'Post' AND bookmarks.user_id = users.id")
# SELECT "users".* FROM "users" LEFT JOIN bookmarks ON bookmarks.bookmarkable_type = 'Post' AND bookmarks.user_id = users.id
848 849 850 851 |
# File 'lib/active_record/relation/query_methods.rb', line 848 def joins(*args) check_if_method_has_arguments!(__callee__, args) spawn.joins!(*args) end |
#joins!(*args) ⇒ Object
:nodoc:
853 854 855 856 |
# File 'lib/active_record/relation/query_methods.rb', line 853 def joins!(*args) # :nodoc: self.joins_values |= args self end |
#left_outer_joins(*args) ⇒ Object Also known as: left_joins
Performs LEFT OUTER JOINs on args:
User.left_outer_joins(:posts)
# SELECT "users".* FROM "users" LEFT OUTER JOIN "posts" ON "posts"."user_id" = "users"."id"
863 864 865 866 |
# File 'lib/active_record/relation/query_methods.rb', line 863 def left_outer_joins(*args) check_if_method_has_arguments!(__callee__, args) spawn.left_outer_joins!(*args) end |
#left_outer_joins!(*args) ⇒ Object
:nodoc:
869 870 871 872 |
# File 'lib/active_record/relation/query_methods.rb', line 869 def left_outer_joins!(*args) # :nodoc: self.left_outer_joins_values |= args self end |
#limit(value) ⇒ Object
Specifies a limit for the number of records to retrieve.
User.limit(10) # generated SQL has 'LIMIT 10'
User.limit(10).limit(20) # generated SQL has 'LIMIT 20'
1191 1192 1193 |
# File 'lib/active_record/relation/query_methods.rb', line 1191 def limit(value) spawn.limit!(value) end |
#limit!(value) ⇒ Object
:nodoc:
1195 1196 1197 1198 |
# File 'lib/active_record/relation/query_methods.rb', line 1195 def limit!(value) # :nodoc: self.limit_value = value self end |
#lock(locks = true) ⇒ Object
Specifies locking settings (default to true). For more information on locking, please see ActiveRecord::Locking.
1218 1219 1220 |
# File 'lib/active_record/relation/query_methods.rb', line 1218 def lock(locks = true) spawn.lock!(locks) end |
#lock!(locks = true) ⇒ Object
:nodoc:
1222 1223 1224 1225 1226 1227 1228 1229 1230 1231 |
# File 'lib/active_record/relation/query_methods.rb', line 1222 def lock!(locks = true) # :nodoc: case locks when String, TrueClass, NilClass self.lock_value = locks || true else self.lock_value = false end self end |
#none ⇒ Object
Returns a chainable relation with zero records.
The returned relation implements the Null Object pattern. It is an object with defined null behavior and always returns an empty array of records without querying the database.
Any subsequent condition chained to the returned relation will continue generating an empty relation and will not fire any query to the database.
Used in cases where a method or scope could return zero records but the result needs to be chainable.
For example:
@posts = current_user.visible_posts.where(name: params[:name])
# the visible_posts method is expected to return a chainable Relation
def visible_posts
case role
when 'Country Manager'
Post.where(country: country)
when 'Reviewer'
Post.published
when 'Bad User'
Post.none # It can't be chained if [] is returned.
end
end
1261 1262 1263 |
# File 'lib/active_record/relation/query_methods.rb', line 1261 def none spawn.none! end |
#none! ⇒ Object
:nodoc:
1265 1266 1267 1268 1269 1270 1271 |
# File 'lib/active_record/relation/query_methods.rb', line 1265 def none! # :nodoc: unless @none where!("1=0") @none = true end self end |
#null_relation? ⇒ Boolean
:nodoc:
1273 1274 1275 |
# File 'lib/active_record/relation/query_methods.rb', line 1273 def null_relation? # :nodoc: @none end |
#offset(value) ⇒ Object
Specifies the number of rows to skip before returning rows.
User.offset(10) # generated SQL has "OFFSET 10"
Should be used with order.
User.offset(10).order("name ASC")
1207 1208 1209 |
# File 'lib/active_record/relation/query_methods.rb', line 1207 def offset(value) spawn.offset!(value) end |
#offset!(value) ⇒ Object
:nodoc:
1211 1212 1213 1214 |
# File 'lib/active_record/relation/query_methods.rb', line 1211 def offset!(value) # :nodoc: self.offset_value = value self end |
#optimizer_hints(*args) ⇒ Object
Specify optimizer hints to be used in the SELECT statement.
Example (for MySQL):
Topic.optimizer_hints("MAX_EXECUTION_TIME(50000)", "NO_INDEX_MERGE(topics)")
# SELECT /*+ MAX_EXECUTION_TIME(50000) NO_INDEX_MERGE(topics) */ `topics`.* FROM `topics`
Example (for PostgreSQL with pg_hint_plan):
Topic.optimizer_hints("SeqScan(topics)", "Parallel(topics 8)")
# SELECT /*+ SeqScan(topics) Parallel(topics 8) */ "topics".* FROM "topics"
1465 1466 1467 1468 |
# File 'lib/active_record/relation/query_methods.rb', line 1465 def optimizer_hints(*args) check_if_method_has_arguments!(__callee__, args) spawn.optimizer_hints!(*args) end |
#optimizer_hints!(*args) ⇒ Object
:nodoc:
1470 1471 1472 1473 |
# File 'lib/active_record/relation/query_methods.rb', line 1470 def optimizer_hints!(*args) # :nodoc: self.optimizer_hints_values |= args self end |
#or(other) ⇒ Object
Returns a new relation, which is the logical union of this relation and the one passed as an argument.
The two relations must be structurally compatible: they must be scoping the same model, and they must differ only by #where (if no #group has been defined) or #having (if a #group is present).
Post.where("id = 1").or(Post.where("author_id = 3"))
# SELECT `posts`.* FROM `posts` WHERE ((id = 1) OR (author_id = 3))
1147 1148 1149 1150 1151 1152 1153 1154 1155 1156 1157 |
# File 'lib/active_record/relation/query_methods.rb', line 1147 def or(other) if other.is_a?(Relation) if @none other.spawn else spawn.or!(other) end else raise ArgumentError, "You have passed #{other.class.name} object to #or. Pass an ActiveRecord::Relation object instead." end end |
#or!(other) ⇒ Object
:nodoc:
1159 1160 1161 1162 1163 1164 1165 1166 1167 1168 1169 1170 1171 |
# File 'lib/active_record/relation/query_methods.rb', line 1159 def or!(other) # :nodoc: incompatible_values = structurally_incompatible_values_for(other) unless incompatible_values.empty? raise ArgumentError, "Relation passed to #or must be structurally compatible. Incompatible values: #{incompatible_values}" end self.where_clause = where_clause.or(other.where_clause) self.having_clause = having_clause.or(other.having_clause) self.references_values |= other.references_values self end |
#order(*args) ⇒ Object
Applies an ORDER BY clause to a query.
#order accepts arguments in one of several formats.
symbols
The symbol represents the name of the column you want to order the results by.
User.order(:name)
# SELECT "users".* FROM "users" ORDER BY "users"."name" ASC
By default, the order is ascending. If you want descending order, you can map the column name symbol to :desc.
User.order(email: :desc)
# SELECT "users".* FROM "users" ORDER BY "users"."email" DESC
Multiple columns can be passed this way, and they will be applied in the order specified.
User.order(:name, email: :desc)
# SELECT "users".* FROM "users" ORDER BY "users"."name" ASC, "users"."email" DESC
strings
Strings are passed directly to the database, allowing you to specify simple SQL expressions.
This could be a source of SQL injection, so only strings composed of plain column names and simple function(column_name) expressions with optional ASC/DESC modifiers are allowed.
User.order('name')
# SELECT "users".* FROM "users" ORDER BY name
User.order('name DESC')
# SELECT "users".* FROM "users" ORDER BY name DESC
User.order('name DESC, email')
# SELECT "users".* FROM "users" ORDER BY name DESC, email
Arel
If you need to pass in complicated expressions that you have verified are safe for the database, you can use Arel.
User.order(Arel.sql('end_date - start_date'))
# SELECT "users".* FROM "users" ORDER BY end_date - start_date
Custom query syntax, like JSON columns for PostgreSQL, is supported in this way.
User.order(Arel.sql("payload->>'kind'"))
# SELECT "users".* FROM "users" ORDER BY payload->>'kind'
649 650 651 652 653 654 |
# File 'lib/active_record/relation/query_methods.rb', line 649 def order(*args) check_if_method_has_arguments!(__callee__, args) do sanitize_order_arguments(args) end spawn.order!(*args) end |
#order!(*args) ⇒ Object
Same as #order but operates on relation in-place instead of copying.
657 658 659 660 661 |
# File 'lib/active_record/relation/query_methods.rb', line 657 def order!(*args) # :nodoc: preprocess_order_args(args) unless args.empty? self.order_values |= args self end |
#preload(*args) ⇒ Object
Specify associations args to be eager loaded using separate queries. A separate query is performed for each association.
users = User.preload(:address).limit(5)
users.each do |user|
user.address.city
end
# SELECT "users".* FROM "users" LIMIT 5
# SELECT "addresses".* FROM "addresses" WHERE "addresses"."id" IN (1,2,3,4,5)
Instead of loading the 5 addresses with 5 separate queries, all addresses are loaded with a separate query.
Loading multiple and nested associations is possible using Hashes and Arrays, similar to #includes:
User.preload(:address, friends: [:address, :followers])
# SELECT "users".* FROM "users"
# SELECT "addresses".* FROM "addresses" WHERE "addresses"."id" IN (1,2,3,4,5)
# SELECT "friends".* FROM "friends" WHERE "friends"."user_id" IN (1,2,3,4,5)
# SELECT ...
315 316 317 318 |
# File 'lib/active_record/relation/query_methods.rb', line 315 def preload(*args) check_if_method_has_arguments!(__callee__, args) spawn.preload!(*args) end |
#preload!(*args) ⇒ Object
:nodoc:
320 321 322 323 |
# File 'lib/active_record/relation/query_methods.rb', line 320 def preload!(*args) # :nodoc: self.preload_values |= args self end |
#readonly(value = true) ⇒ Object
Mark a relation as readonly. Attempting to update a record will result in an error.
users = User.readonly
users.first.save
# => ActiveRecord::ReadOnlyRecord: User is marked as readonly
To make a readonly relation writable, pass false.
users.readonly(false)
users.first.save
# => true
1289 1290 1291 |
# File 'lib/active_record/relation/query_methods.rb', line 1289 def readonly(value = true) spawn.readonly!(value) end |
#readonly!(value = true) ⇒ Object
:nodoc:
1293 1294 1295 1296 |
# File 'lib/active_record/relation/query_methods.rb', line 1293 def readonly!(value = true) # :nodoc: self.readonly_value = value self end |
#references(*table_names) ⇒ Object
Use to indicate that the given table_names are referenced by an SQL string, and should therefore be JOINed in any query rather than loaded separately. This method only works in conjunction with #includes. See #includes for more details.
User.includes(:posts).where("posts.name = 'foo'")
# Doesn't JOIN the posts table, resulting in an error.
User.includes(:posts).where("posts.name = 'foo'").references(:posts)
# Query now knows the string references posts, so adds a JOIN
348 349 350 351 |
# File 'lib/active_record/relation/query_methods.rb', line 348 def references(*table_names) check_if_method_has_arguments!(__callee__, table_names) spawn.references!(*table_names) end |
#references!(*table_names) ⇒ Object
:nodoc:
353 354 355 356 |
# File 'lib/active_record/relation/query_methods.rb', line 353 def references!(*table_names) # :nodoc: self.references_values |= table_names self end |
#regroup(*args) ⇒ Object
Allows you to change a previously set group statement.
Post.group(:title, :body)
# SELECT `posts`.`*` FROM `posts` GROUP BY `posts`.`title`, `posts`.`body`
Post.group(:title, :body).regroup(:title)
# SELECT `posts`.`*` FROM `posts` GROUP BY `posts`.`title`
This is short-hand for unscope(:group).group(fields). Note that we’re unscoping the entire group statement.
586 587 588 589 |
# File 'lib/active_record/relation/query_methods.rb', line 586 def regroup(*args) check_if_method_has_arguments!(__callee__, args) spawn.regroup!(*args) end |
#regroup!(*args) ⇒ Object
Same as #regroup but operates on relation in-place instead of copying.
592 593 594 595 |
# File 'lib/active_record/relation/query_methods.rb', line 592 def regroup!(*args) # :nodoc: self.group_values = args self end |
#reorder(*args) ⇒ Object
Replaces any existing order defined on the relation with the specified order.
User.order('email DESC').reorder('id ASC') # generated SQL has 'ORDER BY id ASC'
Subsequent calls to order on the same relation will be appended. For example:
User.order('email DESC').reorder('id ASC').order('name ASC')
generates a query with ORDER BY id ASC, name ASC.
732 733 734 735 736 737 |
# File 'lib/active_record/relation/query_methods.rb', line 732 def reorder(*args) check_if_method_has_arguments!(__callee__, args) do sanitize_order_arguments(args) end spawn.reorder!(*args) end |
#reorder!(*args) ⇒ Object
Same as #reorder but operates on relation in-place instead of copying.
740 741 742 743 744 745 746 |
# File 'lib/active_record/relation/query_methods.rb', line 740 def reorder!(*args) # :nodoc: preprocess_order_args(args) args.uniq! self.reordering_value = true self.order_values = args self end |
#reselect(*args) ⇒ Object
Allows you to change a previously set select statement.
Post.select(:title, :body)
# SELECT `posts`.`title`, `posts`.`body` FROM `posts`
Post.select(:title, :body).reselect(:created_at)
# SELECT `posts`.`created_at` FROM `posts`
This is short-hand for unscope(:select).select(fields). Note that we’re unscoping the entire select statement.
534 535 536 537 538 |
# File 'lib/active_record/relation/query_methods.rb', line 534 def reselect(*args) check_if_method_has_arguments!(__callee__, args) args = process_select_args(args) spawn.reselect!(*args) end |
#reselect!(*args) ⇒ Object
Same as #reselect but operates on relation in-place instead of copying.
541 542 543 544 |
# File 'lib/active_record/relation/query_methods.rb', line 541 def reselect!(*args) # :nodoc: self.select_values = args self end |
#reverse_order ⇒ Object
Reverse the existing order clause on the relation.
User.order('name ASC').reverse_order # generated SQL has 'ORDER BY name DESC'
1478 1479 1480 |
# File 'lib/active_record/relation/query_methods.rb', line 1478 def reverse_order spawn.reverse_order! end |
#reverse_order! ⇒ Object
:nodoc:
1482 1483 1484 1485 1486 |
# File 'lib/active_record/relation/query_methods.rb', line 1482 def reverse_order! # :nodoc: orders = order_values.compact_blank self.order_values = reverse_sql_order(orders) self end |
#rewhere(conditions) ⇒ Object
Allows you to change a previously set where condition for a given attribute, instead of appending to that condition.
Post.where(trashed: true).where(trashed: false)
# WHERE `trashed` = 1 AND `trashed` = 0
Post.where(trashed: true).rewhere(trashed: false)
# WHERE `trashed` = 0
Post.where(active: true).where(trashed: true).rewhere(trashed: false)
# WHERE `active` = 1 AND `trashed` = 0
This is short-hand for unscope(where: conditions.keys).where(conditions). Note that unlike reorder, we’re only unscoping the named conditions – not the entire where statement.
1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 |
# File 'lib/active_record/relation/query_methods.rb', line 1041 def rewhere(conditions) return unscope(:where) if conditions.nil? scope = spawn where_clause = scope.build_where_clause(conditions) scope.unscope!(where: where_clause.extract_attributes) scope.where_clause += where_clause scope end |
#select(*fields) ⇒ Object
Works in two unique ways.
First: takes a block so it can be used just like Array#select.
Model.all.select { |m| m.field == value }
This will build an array of objects from the database for the scope, converting them into an array and iterating through them using Array#select.
Second: Modifies the SELECT statement for the query so that only certain fields are retrieved:
Model.select(:field)
# => [#<Model id: nil, field: "value">]
Although in the above example it looks as though this method returns an array, it actually returns a relation object and can have other query methods appended to it, such as the other methods in ActiveRecord::QueryMethods.
The argument to the method can also be an array of fields.
Model.select(:field, :other_field, :and_one_more)
# => [#<Model id: nil, field: "value", other_field: "value", and_one_more: "value">]
The argument also can be a hash of fields and aliases.
Model.select(models: { field: :alias, other_field: :other_alias })
# => [#<Model id: nil, alias: "value", other_alias: "value">]
Model.select(models: [:field, :other_field])
# => [#<Model id: nil, field: "value", other_field: "value">]
You can also use one or more strings, which will be used unchanged as SELECT fields.
Model.select('field AS field_one', 'other_field AS field_two')
# => [#<Model id: nil, field_one: "value", field_two: "value">]
If an alias was specified, it will be accessible from the resulting objects:
Model.select('field AS field_one').first.field_one
# => "value"
Accessing attributes of an object that do not have fields retrieved by a select except id will throw ActiveModel::MissingAttributeError:
Model.select(:field).first.other_field
# => ActiveModel::MissingAttributeError: missing attribute 'other_field' for Model
406 407 408 409 410 411 412 413 414 415 416 417 418 419 |
# File 'lib/active_record/relation/query_methods.rb', line 406 def select(*fields) if block_given? if fields.any? raise ArgumentError, "`select' with block doesn't take arguments." end return super() end check_if_method_has_arguments!(__callee__, fields, "Call `select' with at least one field.") fields = process_select_args(fields) spawn._select!(*fields) end |
#skip_preloading! ⇒ Object
:nodoc:
1493 1494 1495 1496 |
# File 'lib/active_record/relation/query_methods.rb', line 1493 def skip_preloading! # :nodoc: self.skip_preloading_value = true self end |
#skip_query_cache!(value = true) ⇒ Object
:nodoc:
1488 1489 1490 1491 |
# File 'lib/active_record/relation/query_methods.rb', line 1488 def skip_query_cache!(value = true) # :nodoc: self.skip_query_cache_value = value self end |
#strict_loading(value = true) ⇒ Object
Sets the returned relation to strict_loading mode. This will raise an error if the record tries to lazily load an association.
user = User.strict_loading.first
user.comments.to_a
# => ActiveRecord::StrictLoadingViolationError
1304 1305 1306 |
# File 'lib/active_record/relation/query_methods.rb', line 1304 def strict_loading(value = true) spawn.strict_loading!(value) end |
#strict_loading!(value = true) ⇒ Object
:nodoc:
1308 1309 1310 1311 |
# File 'lib/active_record/relation/query_methods.rb', line 1308 def strict_loading!(value = true) # :nodoc: self.strict_loading_value = value self end |
#structurally_compatible?(other) ⇒ Boolean
Checks whether the given relation is structurally compatible with this relation, to determine if it’s possible to use the #and and #or methods without raising an error. Structurally compatible is defined as: they must be scoping the same model, and they must differ only by #where (if no #group has been defined) or #having (if a #group is present).
Post.where("id = 1").structurally_compatible?(Post.where("author_id = 3"))
# => true
Post.joins(:comments).structurally_compatible?(Post.where("id = 1"))
# => false
1101 1102 1103 |
# File 'lib/active_record/relation/query_methods.rb', line 1101 def structurally_compatible?(other) structurally_incompatible_values_for(other).empty? end |
#uniq!(name) ⇒ Object
Deduplicate multiple values.
1521 1522 1523 1524 1525 1526 |
# File 'lib/active_record/relation/query_methods.rb', line 1521 def uniq!(name) if values = @values[name] values.uniq! if values.is_a?(Array) && !values.empty? end self end |
#unscope(*args) ⇒ Object
Removes an unwanted relation that is already defined on a chain of relations. This is useful when passing around chains of relations and would like to modify the relations without reconstructing the entire chain.
User.order('email DESC').unscope(:order) == User.all
The method arguments are symbols which correspond to the names of the methods which should be unscoped. The valid arguments are given in VALID_UNSCOPING_VALUES. The method can also be called with multiple arguments. For example:
User.order('email DESC').select('id').where(name: "John")
.unscope(:order, :select, :where) == User.all
One can additionally pass a hash as an argument to unscope specific :where values. This is done by passing a hash with a single key-value pair. The key should be :where and the value should be the where value to unscope. For example:
User.where(name: "John", active: true).unscope(where: :name)
== User.where(active: true)
This method is similar to #except, but unlike #except, it persists across merges:
User.order('email').merge(User.except(:order))
== User.order('email')
User.order('email').merge(User.unscope(:order))
== User.all
This means it can be used in association definitions:
has_many :comments, -> { unscope(where: :trashed) }
786 787 788 789 |
# File 'lib/active_record/relation/query_methods.rb', line 786 def unscope(*args) check_if_method_has_arguments!(__callee__, args) spawn.unscope!(*args) end |
#unscope!(*args) ⇒ Object
:nodoc:
791 792 793 794 795 796 797 798 799 800 801 802 803 804 805 806 807 808 809 810 811 812 813 814 815 816 817 818 |
# File 'lib/active_record/relation/query_methods.rb', line 791 def unscope!(*args) # :nodoc: self.unscope_values += args args.each do |scope| case scope when Symbol scope = :left_outer_joins if scope == :left_joins if !VALID_UNSCOPING_VALUES.include?(scope) raise ArgumentError, "Called unscope() with invalid unscoping argument ':#{scope}'. Valid arguments are :#{VALID_UNSCOPING_VALUES.to_a.join(", :")}." end assert_modifiable! @values.delete(scope) when Hash scope.each do |key, target_value| if key != :where raise ArgumentError, "Hash arguments in .unscope(*args) must have :where as the key." end target_values = resolve_arel_attributes(Array.wrap(target_value)) self.where_clause = where_clause.except(*target_values) end else raise ArgumentError, "Unrecognized scoping: #{args.inspect}. Use .unscope(where: :attribute_name) or .unscope(:order), for example." end end self end |
#where(*args) ⇒ Object
Returns a new relation, which is the result of filtering the current relation according to the conditions in the arguments.
#where accepts conditions in one of several formats. In the examples below, the resulting SQL is given as an illustration; the actual query generated may be different depending on the database adapter.
String
A single string, without additional arguments, is passed to the query constructor as an SQL fragment, and used in the where clause of the query.
Client.where("orders_count = '2'")
# SELECT * from clients where orders_count = '2';
Note that building your own string from user input may expose your application to injection attacks if not done properly. As an alternative, it is recommended to use one of the following methods.
Array
If an array is passed, then the first element of the array is treated as a template, and the remaining elements are inserted into the template to generate the condition. Active Record takes care of building the query to avoid injection attacks, and will convert from the ruby type to the database type where needed. Elements are inserted into the string in the order in which they appear.
User.where(["name = ? and email = ?", "Joe", "[email protected]"])
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]';
Alternatively, you can use named placeholders in the template, and pass a hash as the second element of the array. The names in the template are replaced with the corresponding values from the hash.
User.where(["name = :name and email = :email", { name: "Joe", email: "[email protected]" }])
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]';
This can make for more readable code in complex queries.
Lastly, you can use sprintf-style % escapes in the template. This works slightly differently than the previous methods; you are responsible for ensuring that the values in the template are properly quoted. The values are passed to the connector for quoting, but the caller is responsible for ensuring they are enclosed in quotes in the resulting SQL. After quoting, the values are inserted using the same escapes as the Ruby core method Kernel::sprintf.
User.where(["name = '%s' and email = '%s'", "Joe", "[email protected]"])
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]';
If #where is called with multiple arguments, these are treated as if they were passed as the elements of a single array.
User.where("name = :name and email = :email", { name: "Joe", email: "[email protected]" })
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]';
When using strings to specify conditions, you can use any operator available from the database. While this provides the most flexibility, you can also unintentionally introduce dependencies on the underlying database. If your code is intended for general consumption, test with multiple database backends.
Hash
#where will also accept a hash condition, in which the keys are fields and the values are values to be searched for.
Fields can be symbols or strings. Values can be single values, arrays, or ranges.
User.where(name: "Joe", email: "[email protected]")
# SELECT * FROM users WHERE name = 'Joe' AND email = '[email protected]'
User.where(name: ["Alice", "Bob"])
# SELECT * FROM users WHERE name IN ('Alice', 'Bob')
User.where(created_at: (Time.now.midnight - 1.day)..Time.now.midnight)
# SELECT * FROM users WHERE (created_at BETWEEN '2012-06-09 07:00:00.000000' AND '2012-06-10 07:00:00.000000')
In the case of a belongs_to relationship, an association key can be used to specify the model if an ActiveRecord object is used as the value.
= Author.find(1)
# The following queries will be equivalent:
Post.where(author: )
Post.where(author_id: )
This also works with polymorphic belongs_to relationships:
treasure = Treasure.create(name: 'gold coins')
treasure.price_estimates << PriceEstimate.create(price: 125)
# The following queries will be equivalent:
PriceEstimate.where(estimate_of: treasure)
PriceEstimate.where(estimate_of_type: 'Treasure', estimate_of_id: treasure)
Hash conditions may also be specified in a tuple-like syntax. Hash keys may be an array of columns with an array of tuples as values.
Article.where([:author_id, :id] => [[15, 1], [15, 2]])
# SELECT * FROM articles WHERE author_id = 15 AND id = 1 OR author_id = 15 AND id = 2
Joins
If the relation is the result of a join, you may create a condition which uses any of the tables in the join. For string and array conditions, use the table name in the condition.
User.joins(:posts).where("posts.created_at < ?", Time.now)
For hash conditions, you can either use the table name in the key, or use a sub-hash.
User.joins(:posts).where("posts.published" => true)
User.joins(:posts).where(posts: { published: true })
No Argument
If no argument is passed, #where returns a new instance of WhereChain, that can be chained with WhereChain#not, WhereChain#missing, or WhereChain#associated.
Chaining with WhereChain#not:
User.where.not(name: "Jon")
# SELECT * FROM users WHERE name != 'Jon'
Chaining with WhereChain#associated:
Post.where.associated(:author)
# SELECT "posts".* FROM "posts"
# INNER JOIN "authors" ON "authors"."id" = "posts"."author_id"
# WHERE "authors"."id" IS NOT NULL
Chaining with WhereChain#missing:
Post.where.missing(:author)
# SELECT "posts".* FROM "posts"
# LEFT OUTER JOIN "authors" ON "authors"."id" = "posts"."author_id"
# WHERE "authors"."id" IS NULL
Blank Condition
If the condition is any blank-ish object, then #where is a no-op and returns the current relation.
1013 1014 1015 1016 1017 1018 1019 1020 1021 |
# File 'lib/active_record/relation/query_methods.rb', line 1013 def where(*args) if args.empty? WhereChain.new(spawn) elsif args.length == 1 && args.first.blank? self else spawn.where!(*args) end end |
#where!(opts, *rest) ⇒ Object
:nodoc:
1023 1024 1025 1026 |
# File 'lib/active_record/relation/query_methods.rb', line 1023 def where!(opts, *rest) # :nodoc: self.where_clause += build_where_clause(opts, rest) self end |
#with(*args) ⇒ Object
Add a Common Table Expression (CTE) that you can then reference within another SELECT statement.
Note: CTE’s are only supported in MySQL for versions 8.0 and above. You will not be able to use CTE’s with MySQL 5.7.
Post.with(posts_with_tags: Post.where("tags_count > ?", 0))
# => ActiveRecord::Relation
# WITH posts_with_tags AS (
# SELECT * FROM posts WHERE (tags_count > 0)
# )
# SELECT * FROM posts
You can also pass an array of sub-queries to be joined in a UNION ALL.
Post.with(posts_with_tags_or_comments: [Post.where("tags_count > ?", 0), Post.where("comments_count > ?", 0)])
# => ActiveRecord::Relation
# WITH posts_with_tags_or_comments AS (
# (SELECT * FROM posts WHERE (tags_count > 0))
# UNION ALL
# (SELECT * FROM posts WHERE (comments_count > 0))
# )
# SELECT * FROM posts
Once you define Common Table Expression you can use custom FROM value or JOIN to reference it.
Post.with(posts_with_tags: Post.where("tags_count > ?", 0)).from("posts_with_tags AS posts")
# => ActiveRecord::Relation
# WITH posts_with_tags AS (
# SELECT * FROM posts WHERE (tags_count > 0)
# )
# SELECT * FROM posts_with_tags AS posts
Post.with(posts_with_tags: Post.where("tags_count > ?", 0)).joins("JOIN posts_with_tags ON posts_with_tags.id = posts.id")
# => ActiveRecord::Relation
# WITH posts_with_tags AS (
# SELECT * FROM posts WHERE (tags_count > 0)
# )
# SELECT * FROM posts JOIN posts_with_tags ON posts_with_tags.id = posts.id
It is recommended to pass a query as ActiveRecord::Relation. If that is not possible and you have verified it is safe for the database, you can pass it as SQL literal using Arel.
Post.with(popular_posts: Arel.sql("... complex sql to calculate posts popularity ..."))
Great caution should be taken to avoid SQL injection vulnerabilities. This method should not be used with unsafe values that include unsanitized input.
To add multiple CTEs just pass multiple key-value pairs
Post.with(
posts_with_comments: Post.where("comments_count > ?", 0),
posts_with_tags: Post.where("tags_count > ?", 0)
)
or chain multiple .with calls
Post
.with(posts_with_comments: Post.where("comments_count > ?", 0))
.with(posts_with_tags: Post.where("tags_count > ?", 0))
486 487 488 489 490 |
# File 'lib/active_record/relation/query_methods.rb', line 486 def with(*args) raise ArgumentError, "ActiveRecord::Relation#with does not accept a block" if block_given? check_if_method_has_arguments!(__callee__, args) spawn.with!(*args) end |
#with!(*args) ⇒ Object
Like #with, but modifies relation in place.
493 494 495 496 497 |
# File 'lib/active_record/relation/query_methods.rb', line 493 def with!(*args) # :nodoc: args = process_with_args(args) self.with_values |= args self end |
#with_recursive(*args) ⇒ Object
Add a recursive Common Table Expression (CTE) that you can then reference within another SELECT statement.
Post.with_recursive(post_and_replies: [Post.where(id: 42), Post.joins('JOIN post_and_replies ON posts.in_reply_to_id = post_and_replies.id')])
# => ActiveRecord::Relation
# WITH post_and_replies AS (
# (SELECT * FROM posts WHERE id = 42)
# UNION ALL
# (SELECT * FROM posts JOIN post_and_replies ON posts.in_reply_to_id = post_and_replies.id)
# )
# SELECT * FROM posts
See #with for more information.
511 512 513 514 |
# File 'lib/active_record/relation/query_methods.rb', line 511 def with_recursive(*args) check_if_method_has_arguments!(__callee__, args) spawn.with_recursive!(*args) end |
#with_recursive!(*args) ⇒ Object
Like #with_recursive but modifies the relation in place.
517 518 519 520 521 522 |
# File 'lib/active_record/relation/query_methods.rb', line 517 def with_recursive!(*args) # :nodoc: args = process_with_args(args) self.with_values |= args @with_is_recursive = true self end |