Method: ActiveRecord::QueryMethods#or
- Defined in:
- lib/active_record/relation/query_methods.rb
#or(other) ⇒ Object
Returns a new relation, which is the logical union of this relation and the one passed as an argument.
The two relations must be structurally compatible: they must be scoping the same model, and they must differ only by #where (if no #group has been defined) or #having (if a #group is present). Neither relation may have a #limit, #offset, or #distinct set.
Post.where("id = 1").or(Post.where("author_id = 3"))
# SELECT `posts`.* FROM `posts` WHERE ((id = 1) OR (author_id = 3))
643 644 645 646 647 648 649 |
# File 'lib/active_record/relation/query_methods.rb', line 643 def or(other) unless other.is_a? Relation raise ArgumentError, "You have passed #{other.class.name} object to #or. Pass an ActiveRecord::Relation object instead." end spawn.or!(other) end |