Module: ActiveRecord::Serialization
- Extended by:
- ActiveSupport::Concern
- Includes:
- ActiveModel::Serializers::JSON, ActiveModel::Serializers::Xml
- Defined in:
- lib/active_record/serialization.rb,
lib/active_record/serializers/xml_serializer.rb
Overview
Active Record Serialization
Instance Method Summary collapse
- #serializable_hash(options = nil) ⇒ Object
-
#to_xml(options = {}, &block) ⇒ Object
Builds an XML document to represent the model.
Instance Method Details
#serializable_hash(options = nil) ⇒ Object
7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 |
# File 'lib/active_record/serialization.rb', line 7 def serializable_hash( = nil) = .try(:clone) || {} [:except] = Array.wrap([:except]).map { |n| n.to_s } [:except] |= Array.wrap(self.class.inheritance_column) hash = super() serializable_add_includes() do |association, records, opts| hash[association] = records.is_a?(Enumerable) ? records.map { |r| r.serializable_hash(opts) } : records.serializable_hash(opts) end hash end |
#to_xml(options = {}, &block) ⇒ Object
Builds an XML document to represent the model. Some configuration is
available through options. However more complicated cases should
override ActiveRecord::Base#to_xml.
By default the generated XML document will include the processing instruction and all the object's attributes. For example:
This behavior can be controlled with :only, :except,
:skip_instruct, :skip_types, :dasherize and :camelize .
The :only and :except options are the same as for the
attributes method. The default is to dasherize all column names, but you
can disable this setting :dasherize to false. Setting :camelize
to true will camelize all column names - this also overrides :dasherize.
To not have the column type included in the XML output set :skip_types to true.
For instance:
topic.to_xml(:skip_instruct => true, :except => [ :id, :bonus_time, :written_on, :replies_count ])
To include first level associations use :include:
firm.to_xml :include => [ :account, :clients ]
Additionally, the record being serialized will be passed to a Proc's second parameter. This allows for ad hoc additions to the resultant document that incorporate the context of the record being serialized. And by leveraging the closure created by a Proc, to_xml can be used to add elements that normally fall outside of the scope of the model -- for example, generating and appending URLs associated with models.
proc = Proc.new { |, record| [:builder].tag!('name-reverse', record.name.reverse) }
firm.to_xml :procs => [ proc ]
To include deeper levels of associations pass a hash like this:
firm.to_xml :include => {:account => {}, :clients => {:include => :address}}
<?xml version="1.0" encoding="UTF-8"?>
<firm>
<id type="integer">1</id>
<rating type="integer">1</rating>
<name>37signals</name>
<clients type="array">
<client>
<rating type="integer">1</rating>
<name>Summit</name>
<address>
...
</address>
</client>
<client>
<rating type="integer">1</rating>
<name>Microsoft</name>
<address>
...
</address>
</client>
</clients>
<account>
<id type="integer">1</id>
<credit-limit type="integer">50</credit-limit>
</account>
</firm>
To include any methods on the model being called use :methods:
firm.to_xml :methods => [ :calculated_earnings, :real_earnings ]
To call any additional Procs use :procs. The Procs are passed a
modified version of the options hash that was given to to_xml:
proc = Proc.new { || [:builder].tag!('abc', 'def') }
firm.to_xml :procs => [ proc ]
Alternatively, you can yield the builder object as part of the to_xml call:
firm.to_xml do |xml|
xml.creator do
xml.first_name "David"
xml.last_name "Heinemeier Hansson"
end
end
As noted above, you may override to_xml in your ActiveRecord::Base
subclasses to have complete control about what's generated. The general
form of doing this is:
class IHaveMyOwnXML < ActiveRecord::Base
def to_xml( = {})
[:indent] ||= 2
xml = [:builder] ||= Builder::XmlMarkup.new(:indent => [:indent])
xml.instruct! unless [:skip_instruct]
xml.level_one do
xml.tag!(:second_level, 'content')
end
end
end
174 175 176 |
# File 'lib/active_record/serializers/xml_serializer.rb', line 174 def to_xml( = {}, &block) XmlSerializer.new(self, ).serialize(&block) end |