Method: ActiveRecord::Validations::ClassMethods#validates_each
- Defined in:
- lib/active_record/validations.rb
#validates_each(*attrs) ⇒ Object
Validates each attribute against a block.
class Person < ActiveRecord::Base
validates_each :first_name, :last_name do |record, attr, value|
record.errors.add attr, 'starts with z.' if value[0] == ?z
end
end
Options:
-
:on- Specifies when this validation is active (default is:save, other options:create,:update). -
:allow_nil- Skip validation if attribute isnil. -
:allow_blank- Skip validation if attribute is blank. -
:if- Specifies a method, proc or string to call to determine if the validation should occur (e.g.:if => :allow_validation, or:if => Proc.new { |user| user.signup_step > 2 }). The method, proc or string should return or evaluate to a true or false value. -
:unless- Specifies a method, proc or string to call to determine if the validation should not occur (e.g.:unless => :skip_validation, or:unless => Proc.new { |user| user.signup_step <= 2 }). The method, proc or string should return or evaluate to a true or false value.
470 471 472 473 474 475 476 477 478 479 480 481 482 |
# File 'lib/active_record/validations.rb', line 470 def validates_each(*attrs) = attrs..symbolize_keys attrs = attrs.flatten # Declare the validation. send(validation_method([:on] || :save), ) do |record| attrs.each do |attr| value = record.send(attr) next if (value.nil? && [:allow_nil]) || (value.blank? && [:allow_blank]) yield record, attr, value end end end |