Method: ActiveRecord::Validations::ClassMethods#validates_format_of
- Defined in:
- lib/active_record/validations.rb
#validates_format_of(*attr_names) ⇒ Object
Validates whether the value of the specified attribute is of the correct form by matching it against the regular expression provided.
class Person < ActiveRecord::Base
validates_format_of :email, :with => /\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\Z/i, :on => :create
end
Note: use \A and \Z to match the start and end of the string, ^ and $ match the start/end of a line.
A regular expression must be provided or else an exception will be raised.
Configuration options:
-
:message- A custom error message (default is: “is invalid”). -
:allow_nil- If set to true, skips this validation if the attribute isnil(default isfalse). -
:allow_blank- If set to true, skips this validation if the attribute is blank (default isfalse). -
:with- The regular expression used to validate the format with (note: must be supplied!). -
:on- Specifies when this validation is active (default is:save, other options:create,:update). -
:if- Specifies a method, proc or string to call to determine if the validation should occur (e.g.:if => :allow_validation, or:if => Proc.new { |user| user.signup_step > 2 }). The method, proc or string should return or evaluate to a true or false value. -
:unless- Specifies a method, proc or string to call to determine if the validation should not occur (e.g.:unless => :skip_validation, or:unless => Proc.new { |user| user.signup_step <= 2 }). The method, proc or string should return or evaluate to a true or false value.
789 790 791 792 793 794 795 796 797 798 799 800 |
# File 'lib/active_record/validations.rb', line 789 def validates_format_of(*attr_names) configuration = { :on => :save, :with => nil } configuration.update(attr_names.) raise(ArgumentError, "A regular expression must be supplied as the :with option of the configuration hash") unless configuration[:with].is_a?(Regexp) validates_each(attr_names, configuration) do |record, attr_name, value| unless value.to_s =~ configuration[:with] record.errors.add(attr_name, :invalid, :default => configuration[:message], :value => value) end end end |